/********************************************************************************
** Form generated from reading UI file 'timezonetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIMEZONETAB_H
#define UI_TIMEZONETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_TimezoneTab
{
public:
    QVBoxLayout *verticalLayout_2;
    QSplitter *splitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLineEdit *tzSearchLine;
    QTreeView *tzView;
    QTreeView *offsetView;

    void setupUi(QWidget *GammaRay__TimezoneTab)
    {
        if (GammaRay__TimezoneTab->objectName().isEmpty())
            GammaRay__TimezoneTab->setObjectName("GammaRay__TimezoneTab");
        GammaRay__TimezoneTab->resize(400, 300);
        verticalLayout_2 = new QVBoxLayout(GammaRay__TimezoneTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        splitter = new QSplitter(GammaRay__TimezoneTab);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(splitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tzSearchLine = new QLineEdit(layoutWidget);
        tzSearchLine->setObjectName("tzSearchLine");

        verticalLayout->addWidget(tzSearchLine);

        tzView = new QTreeView(layoutWidget);
        tzView->setObjectName("tzView");
        tzView->setRootIsDecorated(false);
        tzView->setUniformRowHeights(true);

        verticalLayout->addWidget(tzView);

        splitter->addWidget(layoutWidget);
        offsetView = new QTreeView(splitter);
        offsetView->setObjectName("offsetView");
        offsetView->setRootIsDecorated(false);
        offsetView->setUniformRowHeights(true);
        splitter->addWidget(offsetView);

        verticalLayout_2->addWidget(splitter);


        retranslateUi(GammaRay__TimezoneTab);

        QMetaObject::connectSlotsByName(GammaRay__TimezoneTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__TimezoneTab)
    {
        (void)GammaRay__TimezoneTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class TimezoneTab: public Ui_TimezoneTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_TIMEZONETAB_H
