/****************************************************************************
** Meta object code from reading C++ file 'remotemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/client/remotemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'remotemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay11RemoteModelE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::RemoteModel::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay11RemoteModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::RemoteModel",
        "proxyDynamicSortFilterChanged",
        "",
        "proxyFilterCaseSensitivityChanged",
        "proxyFilterKeyColumnChanged",
        "proxyFilterRegExpChanged",
        "declarationCreationLocationsReceived",
        "QVariant",
        "v",
        "v2",
        "newMessage",
        "GammaRay::Message",
        "msg",
        "serverRegistered",
        "objectName",
        "Protocol::ObjectAddress",
        "objectAddress",
        "serverUnregistered",
        "doRequests",
        "dynamicSortFilter",
        "filterCaseSensitivity",
        "Qt::CaseSensitivity",
        "filterKeyColumn",
        "filterRegularExpression"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'proxyDynamicSortFilterChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'proxyFilterCaseSensitivityChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'proxyFilterKeyColumnChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'proxyFilterRegExpChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'declarationCreationLocationsReceived'
        QtMocHelpers::SignalData<void(const QVariant &, const QVariant &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 },
        }}),
        // Slot 'newMessage'
        QtMocHelpers::SlotData<void(const GammaRay::Message &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'serverRegistered'
        QtMocHelpers::SlotData<void(const QString &, Protocol::ObjectAddress)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'serverUnregistered'
        QtMocHelpers::SlotData<void(const QString &, Protocol::ObjectAddress)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'doRequests'
        QtMocHelpers::SlotData<void() const>(18, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dynamicSortFilter'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'filterCaseSensitivity'
        QtMocHelpers::PropertyData<Qt::CaseSensitivity>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 1),
        // property 'filterKeyColumn'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'filterRegularExpression'
        QtMocHelpers::PropertyData<QRegularExpression>(23, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RemoteModel, qt_meta_tag_ZN8GammaRay11RemoteModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::RemoteModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay11RemoteModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay11RemoteModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay11RemoteModelE_t>.metaTypes,
    nullptr
} };

void GammaRay::RemoteModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->proxyDynamicSortFilterChanged(); break;
        case 1: _t->proxyFilterCaseSensitivityChanged(); break;
        case 2: _t->proxyFilterKeyColumnChanged(); break;
        case 3: _t->proxyFilterRegExpChanged(); break;
        case 4: _t->declarationCreationLocationsReceived((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 5: _t->newMessage((*reinterpret_cast<std::add_pointer_t<GammaRay::Message>>(_a[1]))); break;
        case 6: _t->serverRegistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Protocol::ObjectAddress>>(_a[2]))); break;
        case 7: _t->serverUnregistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Protocol::ObjectAddress>>(_a[2]))); break;
        case 8: _t->doRequests(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RemoteModel::*)()>(_a, &RemoteModel::proxyDynamicSortFilterChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteModel::*)()>(_a, &RemoteModel::proxyFilterCaseSensitivityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteModel::*)()>(_a, &RemoteModel::proxyFilterKeyColumnChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteModel::*)()>(_a, &RemoteModel::proxyFilterRegExpChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteModel::*)(const QVariant & , const QVariant & )>(_a, &RemoteModel::declarationCreationLocationsReceived, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->proxyDynamicSortFilter(); break;
        case 1: *reinterpret_cast<Qt::CaseSensitivity*>(_v) = _t->proxyFilterCaseSensitivity(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->proxyFilterKeyColumn(); break;
        case 3: *reinterpret_cast<QRegularExpression*>(_v) = _t->proxyFilterRegExp(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setProxyDynamicSortFilter(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setProxyFilterCaseSensitivity(*reinterpret_cast<Qt::CaseSensitivity*>(_v)); break;
        case 2: _t->setProxyFilterKeyColumn(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setProxyFilterRegExp(*reinterpret_cast<QRegularExpression*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *GammaRay::RemoteModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::RemoteModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay11RemoteModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int GammaRay::RemoteModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::RemoteModel::proxyDynamicSortFilterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::RemoteModel::proxyFilterCaseSensitivityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GammaRay::RemoteModel::proxyFilterKeyColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GammaRay::RemoteModel::proxyFilterRegExpChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GammaRay::RemoteModel::declarationCreationLocationsReceived(const QVariant & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
