/********************************************************************************
** Form generated from reading UI file 'webinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WEBINSPECTORWIDGET_H
#define UI_WEBINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWebEngineWidgets/QWebEngineView>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_WebInspectorWidget
{
public:
    QVBoxLayout *verticalLayout;
    QComboBox *webPageComboBox;
    QWebEngineView *webView;

    void setupUi(QWidget *GammaRay__WebInspectorWidget)
    {
        if (GammaRay__WebInspectorWidget->objectName().isEmpty())
            GammaRay__WebInspectorWidget->setObjectName("GammaRay__WebInspectorWidget");
        GammaRay__WebInspectorWidget->resize(727, 530);
        verticalLayout = new QVBoxLayout(GammaRay__WebInspectorWidget);
        verticalLayout->setObjectName("verticalLayout");
        webPageComboBox = new QComboBox(GammaRay__WebInspectorWidget);
        webPageComboBox->setObjectName("webPageComboBox");

        verticalLayout->addWidget(webPageComboBox);

        webView = new QWebEngineView(GammaRay__WebInspectorWidget);
        webView->setObjectName("webView");
        webView->setUrl(QUrl(QString::fromUtf8("about:blank")));

        verticalLayout->addWidget(webView);


        retranslateUi(GammaRay__WebInspectorWidget);

        QMetaObject::connectSlotsByName(GammaRay__WebInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__WebInspectorWidget)
    {
        (void)GammaRay__WebInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class WebInspectorWidget: public Ui_WebInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_WEBINSPECTORWIDGET_H
