/********************************************************************************
** Form generated from reading UI file 'actioninspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACTIONINSPECTORWIDGET_H
#define UI_ACTIONINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ActionInspectorWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *actionSearchLine;
    GammaRay::DeferredTreeView *actionView;

    void setupUi(QWidget *GammaRay__ActionInspectorWidget)
    {
        if (GammaRay__ActionInspectorWidget->objectName().isEmpty())
            GammaRay__ActionInspectorWidget->setObjectName("GammaRay__ActionInspectorWidget");
        GammaRay__ActionInspectorWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__ActionInspectorWidget);
        verticalLayout->setObjectName("verticalLayout");
        actionSearchLine = new QLineEdit(GammaRay__ActionInspectorWidget);
        actionSearchLine->setObjectName("actionSearchLine");

        verticalLayout->addWidget(actionSearchLine);

        actionView = new GammaRay::DeferredTreeView(GammaRay__ActionInspectorWidget);
        actionView->setObjectName("actionView");
        actionView->setContextMenuPolicy(Qt::CustomContextMenu);
        actionView->setRootIsDecorated(false);
        actionView->setUniformRowHeights(true);

        verticalLayout->addWidget(actionView);


        retranslateUi(GammaRay__ActionInspectorWidget);

        QMetaObject::connectSlotsByName(GammaRay__ActionInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ActionInspectorWidget)
    {
        (void)GammaRay__ActionInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ActionInspectorWidget: public Ui_ActionInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_ACTIONINSPECTORWIDGET_H
