/********************************************************************************
** Form generated from reading UI file 'fontbrowserwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONTBROWSERWIDGET_H
#define UI_FONTBROWSERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_FontBrowserWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QSplitter *mainSplitter;
    QWidget *widget;
    QVBoxLayout *verticalLayout_2;
    QLineEdit *fontSearchLine;
    GammaRay::DeferredTreeView *fontTree;
    QWidget *layoutWidget_7;
    QVBoxLayout *verticalLayout;
    QLineEdit *fontText;
    QWidget *fontOptions;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *pointSize;
    QCheckBox *boldBox;
    QCheckBox *italicBox;
    QCheckBox *underlineBox;
    QSpacerItem *horizontalSpacer;
    GammaRay::DeferredTreeView *selectedFontsView;

    void setupUi(QWidget *GammaRay__FontBrowserWidget)
    {
        if (GammaRay__FontBrowserWidget->objectName().isEmpty())
            GammaRay__FontBrowserWidget->setObjectName("GammaRay__FontBrowserWidget");
        GammaRay__FontBrowserWidget->resize(742, 300);
        verticalLayout_3 = new QVBoxLayout(GammaRay__FontBrowserWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        mainSplitter = new QSplitter(GammaRay__FontBrowserWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Orientation::Horizontal);
        widget = new QWidget(mainSplitter);
        widget->setObjectName("widget");
        verticalLayout_2 = new QVBoxLayout(widget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        fontSearchLine = new QLineEdit(widget);
        fontSearchLine->setObjectName("fontSearchLine");

        verticalLayout_2->addWidget(fontSearchLine);

        fontTree = new GammaRay::DeferredTreeView(widget);
        fontTree->setObjectName("fontTree");
        fontTree->setUniformRowHeights(true);

        verticalLayout_2->addWidget(fontTree);

        mainSplitter->addWidget(widget);
        layoutWidget_7 = new QWidget(mainSplitter);
        layoutWidget_7->setObjectName("layoutWidget_7");
        verticalLayout = new QVBoxLayout(layoutWidget_7);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        fontText = new QLineEdit(layoutWidget_7);
        fontText->setObjectName("fontText");

        verticalLayout->addWidget(fontText);

        fontOptions = new QWidget(layoutWidget_7);
        fontOptions->setObjectName("fontOptions");
        horizontalLayout = new QHBoxLayout(fontOptions);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(fontOptions);
        label->setObjectName("label");
        label->setWordWrap(true);

        horizontalLayout->addWidget(label);

        pointSize = new QSpinBox(fontOptions);
        pointSize->setObjectName("pointSize");

        horizontalLayout->addWidget(pointSize);

        boldBox = new QCheckBox(fontOptions);
        boldBox->setObjectName("boldBox");
        boldBox->setMinimumSize(QSize(28, 0));

        horizontalLayout->addWidget(boldBox);

        italicBox = new QCheckBox(fontOptions);
        italicBox->setObjectName("italicBox");
        italicBox->setMinimumSize(QSize(28, 0));

        horizontalLayout->addWidget(italicBox);

        underlineBox = new QCheckBox(fontOptions);
        underlineBox->setObjectName("underlineBox");
        underlineBox->setMinimumSize(QSize(28, 0));

        horizontalLayout->addWidget(underlineBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addWidget(fontOptions);

        selectedFontsView = new GammaRay::DeferredTreeView(layoutWidget_7);
        selectedFontsView->setObjectName("selectedFontsView");
        selectedFontsView->setRootIsDecorated(false);

        verticalLayout->addWidget(selectedFontsView);

        mainSplitter->addWidget(layoutWidget_7);

        verticalLayout_3->addWidget(mainSplitter);


        retranslateUi(GammaRay__FontBrowserWidget);

        QMetaObject::connectSlotsByName(GammaRay__FontBrowserWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__FontBrowserWidget)
    {
        fontSearchLine->setPlaceholderText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "Search", nullptr));
        fontText->setText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "The quick brown fox jumps over the lazy dog", nullptr));
        fontText->setPlaceholderText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "Enter some text here...", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "Point size:", nullptr));
        boldBox->setText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "&Bold", nullptr));
        italicBox->setText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "&Italic", nullptr));
        underlineBox->setText(QCoreApplication::translate("GammaRay::FontBrowserWidget", "&Underline", nullptr));
        (void)GammaRay__FontBrowserWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class FontBrowserWidget: public Ui_FontBrowserWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_FONTBROWSERWIDGET_H
