/********************************************************************************
** Form generated from reading UI file 'timertopwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIMERTOPWIDGET_H
#define UI_TIMERTOPWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_TimerTopWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLineEdit *timerViewFilter;
    QToolButton *clearTimers;
    GammaRay::DeferredTreeView *timerView;

    void setupUi(QWidget *GammaRay__TimerTopWidget)
    {
        if (GammaRay__TimerTopWidget->objectName().isEmpty())
            GammaRay__TimerTopWidget->setObjectName("GammaRay__TimerTopWidget");
        GammaRay__TimerTopWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__TimerTopWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        timerViewFilter = new QLineEdit(GammaRay__TimerTopWidget);
        timerViewFilter->setObjectName("timerViewFilter");

        horizontalLayout->addWidget(timerViewFilter);

        clearTimers = new QToolButton(GammaRay__TimerTopWidget);
        clearTimers->setObjectName("clearTimers");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/gammaray/icons/ui/classes/QCheckBox/default.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        clearTimers->setIcon(icon);

        horizontalLayout->addWidget(clearTimers);


        verticalLayout->addLayout(horizontalLayout);

        timerView = new GammaRay::DeferredTreeView(GammaRay__TimerTopWidget);
        timerView->setObjectName("timerView");
        timerView->setContextMenuPolicy(Qt::CustomContextMenu);
        timerView->setAlternatingRowColors(true);
        timerView->setRootIsDecorated(false);
        timerView->setUniformRowHeights(true);
        timerView->header()->setStretchLastSection(false);

        verticalLayout->addWidget(timerView);


        retranslateUi(GammaRay__TimerTopWidget);

        QMetaObject::connectSlotsByName(GammaRay__TimerTopWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__TimerTopWidget)
    {
        clearTimers->setText(QCoreApplication::translate("GammaRay::TimerTopWidget", "...", nullptr));
        (void)GammaRay__TimerTopWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class TimerTopWidget: public Ui_TimerTopWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_TIMERTOPWIDGET_H
