/********************************************************************************
** Form generated from reading UI file 'networkinterfacewidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NETWORKINTERFACEWIDGET_H
#define UI_NETWORKINTERFACEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_NetworkInterfaceWidget
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *interfaceView;

    void setupUi(QWidget *GammaRay__NetworkInterfaceWidget)
    {
        if (GammaRay__NetworkInterfaceWidget->objectName().isEmpty())
            GammaRay__NetworkInterfaceWidget->setObjectName("GammaRay__NetworkInterfaceWidget");
        GammaRay__NetworkInterfaceWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__NetworkInterfaceWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        interfaceView = new QTreeView(GammaRay__NetworkInterfaceWidget);
        interfaceView->setObjectName("interfaceView");

        verticalLayout->addWidget(interfaceView);


        retranslateUi(GammaRay__NetworkInterfaceWidget);

        QMetaObject::connectSlotsByName(GammaRay__NetworkInterfaceWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__NetworkInterfaceWidget)
    {
        (void)GammaRay__NetworkInterfaceWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class NetworkInterfaceWidget: public Ui_NetworkInterfaceWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_NETWORKINTERFACEWIDGET_H
