/********************************************************************************
** Form generated from reading UI file 'texturetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TEXTURETAB_H
#define UI_TEXTURETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <textureextension/textureviewwidget.h>

namespace GammaRay {

class Ui_TextureTab
{
public:
    QVBoxLayout *layout;
    GammaRay::TextureViewWidget *textureView;
    QWidget *textureInfo;
    QHBoxLayout *textureInfoLayout;
    QLabel *caption;
    QLabel *infoLabel;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *GammaRay__TextureTab)
    {
        if (GammaRay__TextureTab->objectName().isEmpty())
            GammaRay__TextureTab->setObjectName("GammaRay__TextureTab");
        GammaRay__TextureTab->resize(1172, 300);
        layout = new QVBoxLayout(GammaRay__TextureTab);
        layout->setSpacing(0);
        layout->setObjectName("layout");
        layout->setContentsMargins(-1, 0, -1, -1);
        textureView = new GammaRay::TextureViewWidget(GammaRay__TextureTab);
        textureView->setObjectName("textureView");

        layout->addWidget(textureView);

        textureInfo = new QWidget(GammaRay__TextureTab);
        textureInfo->setObjectName("textureInfo");
        textureInfoLayout = new QHBoxLayout(textureInfo);
        textureInfoLayout->setObjectName("textureInfoLayout");
        textureInfoLayout->setContentsMargins(0, 6, 0, 0);
        caption = new QLabel(textureInfo);
        caption->setObjectName("caption");
        caption->setTextFormat(Qt::RichText);
        caption->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        textureInfoLayout->addWidget(caption);

        infoLabel = new QLabel(textureInfo);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setTextFormat(Qt::RichText);

        textureInfoLayout->addWidget(infoLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        textureInfoLayout->addItem(horizontalSpacer);


        layout->addWidget(textureInfo);


        retranslateUi(GammaRay__TextureTab);

        QMetaObject::connectSlotsByName(GammaRay__TextureTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__TextureTab)
    {
        caption->setText(QCoreApplication::translate("GammaRay::TextureTab", "<b>Identified Problems:</b>", nullptr));
        infoLabel->setText(QString());
        (void)GammaRay__TextureTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class TextureTab: public Ui_TextureTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_TEXTURETAB_H
