/********************************************************************************
** Form generated from reading UI file 'quickinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QUICKINSPECTORWIDGET_H
#define UI_QUICKINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>
#include <ui/favoritesitemview.h>
#include <ui/propertywidget.h>

namespace GammaRay {

class Ui_QuickInspectorWidget
{
public:
    QAction *actionSaveAsImage;
    QAction *actionSaveAsImageWithDecoration;
    QAction *actionAnalyzePainting;
    QAction *actionSlowDownMode;
    QVBoxLayout *verticalLayout_6;
    QComboBox *windowComboBox;
    QSplitter *mainSplitter;
    QSplitter *previewTreeSplitter;
    QTabWidget *tabWidget;
    QWidget *itemsTab;
    QVBoxLayout *verticalLayout;
    QLineEdit *itemTreeSearchLine;
    GammaRay::ObjectsFavoriteView *favoritesTreeView;
    GammaRay::DeferredTreeView *itemTreeView;
    QWidget *sceneGraphTab;
    QVBoxLayout *verticalLayout_2;
    QLineEdit *sgTreeSearchLine;
    GammaRay::DeferredTreeView *sgTreeView;
    QStackedWidget *stackedWidget;
    QWidget *page_13;
    QVBoxLayout *verticalLayout_3;
    GammaRay::PropertyWidget *itemPropertyWidget;
    QWidget *page_14;
    QVBoxLayout *verticalLayout_4;
    GammaRay::PropertyWidget *sgPropertyWidget;

    void setupUi(QWidget *GammaRay__QuickInspectorWidget)
    {
        if (GammaRay__QuickInspectorWidget->objectName().isEmpty())
            GammaRay__QuickInspectorWidget->setObjectName("GammaRay__QuickInspectorWidget");
        GammaRay__QuickInspectorWidget->resize(813, 300);
        actionSaveAsImage = new QAction(GammaRay__QuickInspectorWidget);
        actionSaveAsImage->setObjectName("actionSaveAsImage");
        actionSaveAsImageWithDecoration = new QAction(GammaRay__QuickInspectorWidget);
        actionSaveAsImageWithDecoration->setObjectName("actionSaveAsImageWithDecoration");
        actionAnalyzePainting = new QAction(GammaRay__QuickInspectorWidget);
        actionAnalyzePainting->setObjectName("actionAnalyzePainting");
        actionSlowDownMode = new QAction(GammaRay__QuickInspectorWidget);
        actionSlowDownMode->setObjectName("actionSlowDownMode");
        actionSlowDownMode->setCheckable(true);
        verticalLayout_6 = new QVBoxLayout(GammaRay__QuickInspectorWidget);
        verticalLayout_6->setObjectName("verticalLayout_6");
        windowComboBox = new QComboBox(GammaRay__QuickInspectorWidget);
        windowComboBox->setObjectName("windowComboBox");

        verticalLayout_6->addWidget(windowComboBox);

        mainSplitter = new QSplitter(GammaRay__QuickInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        previewTreeSplitter = new QSplitter(mainSplitter);
        previewTreeSplitter->setObjectName("previewTreeSplitter");
        previewTreeSplitter->setOrientation(Qt::Vertical);
        tabWidget = new QTabWidget(previewTreeSplitter);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setMinimumSize(QSize(400, 0));
        tabWidget->setTabPosition(QTabWidget::South);
        itemsTab = new QWidget();
        itemsTab->setObjectName("itemsTab");
        verticalLayout = new QVBoxLayout(itemsTab);
        verticalLayout->setObjectName("verticalLayout");
        itemTreeSearchLine = new QLineEdit(itemsTab);
        itemTreeSearchLine->setObjectName("itemTreeSearchLine");

        verticalLayout->addWidget(itemTreeSearchLine);

        favoritesTreeView = new GammaRay::ObjectsFavoriteView(itemsTab);
        favoritesTreeView->setObjectName("favoritesTreeView");

        verticalLayout->addWidget(favoritesTreeView);

        itemTreeView = new GammaRay::DeferredTreeView(itemsTab);
        itemTreeView->setObjectName("itemTreeView");
        itemTreeView->setContextMenuPolicy(Qt::CustomContextMenu);
        itemTreeView->setUniformRowHeights(true);

        verticalLayout->addWidget(itemTreeView);

        tabWidget->addTab(itemsTab, QString());
        sceneGraphTab = new QWidget();
        sceneGraphTab->setObjectName("sceneGraphTab");
        verticalLayout_2 = new QVBoxLayout(sceneGraphTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        sgTreeSearchLine = new QLineEdit(sceneGraphTab);
        sgTreeSearchLine->setObjectName("sgTreeSearchLine");

        verticalLayout_2->addWidget(sgTreeSearchLine);

        sgTreeView = new GammaRay::DeferredTreeView(sceneGraphTab);
        sgTreeView->setObjectName("sgTreeView");
        sgTreeView->setUniformRowHeights(true);

        verticalLayout_2->addWidget(sgTreeView);

        tabWidget->addTab(sceneGraphTab, QString());
        previewTreeSplitter->addWidget(tabWidget);
        mainSplitter->addWidget(previewTreeSplitter);
        stackedWidget = new QStackedWidget(mainSplitter);
        stackedWidget->setObjectName("stackedWidget");
        page_13 = new QWidget();
        page_13->setObjectName("page_13");
        verticalLayout_3 = new QVBoxLayout(page_13);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        itemPropertyWidget = new GammaRay::PropertyWidget(page_13);
        itemPropertyWidget->setObjectName("itemPropertyWidget");
        itemPropertyWidget->setMinimumSize(QSize(400, 100));

        verticalLayout_3->addWidget(itemPropertyWidget);

        stackedWidget->addWidget(page_13);
        page_14 = new QWidget();
        page_14->setObjectName("page_14");
        verticalLayout_4 = new QVBoxLayout(page_14);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        sgPropertyWidget = new GammaRay::PropertyWidget(page_14);
        sgPropertyWidget->setObjectName("sgPropertyWidget");
        sgPropertyWidget->setMinimumSize(QSize(400, 0));

        verticalLayout_4->addWidget(sgPropertyWidget);

        stackedWidget->addWidget(page_14);
        mainSplitter->addWidget(stackedWidget);

        verticalLayout_6->addWidget(mainSplitter);


        retranslateUi(GammaRay__QuickInspectorWidget);
        QObject::connect(tabWidget, &QTabWidget::currentChanged, stackedWidget, &QStackedWidget::setCurrentIndex);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__QuickInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__QuickInspectorWidget)
    {
        actionSaveAsImage->setText(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Save as &Image...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAsImage->setToolTip(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Save current frame as image.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveAsImageWithDecoration->setText(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Save as &Image (with decoration)...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAsImageWithDecoration->setToolTip(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Save current frame with decoration rendered as image.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAnalyzePainting->setText(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Analyze Painting...", nullptr));
#if QT_CONFIG(tooltip)
        actionAnalyzePainting->setToolTip(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Analyzer software renderer paint operations.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionAnalyzePainting->setShortcut(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "F9", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSlowDownMode->setText(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Enable slow down mode", nullptr));
#if QT_CONFIG(tooltip)
        actionSlowDownMode->setToolTip(QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Enable slow down mode for animations", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidget->setTabText(tabWidget->indexOf(itemsTab), QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Items", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(sceneGraphTab), QCoreApplication::translate("GammaRay::QuickInspectorWidget", "Scene Graph", nullptr));
        (void)GammaRay__QuickInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class QuickInspectorWidget: public Ui_QuickInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_QUICKINSPECTORWIDGET_H
