/********************************************************************************
** Form generated from reading UI file 'metatypebrowserwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_METATYPEBROWSERWIDGET_H
#define UI_METATYPEBROWSERWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_MetaTypeBrowserWidget
{
public:
    QAction *actionRescanTypes;
    QVBoxLayout *verticalLayout;
    QLineEdit *metaTypeSearchLine;
    GammaRay::DeferredTreeView *metaTypeView;

    void setupUi(QWidget *GammaRay__MetaTypeBrowserWidget)
    {
        if (GammaRay__MetaTypeBrowserWidget->objectName().isEmpty())
            GammaRay__MetaTypeBrowserWidget->setObjectName("GammaRay__MetaTypeBrowserWidget");
        GammaRay__MetaTypeBrowserWidget->resize(400, 300);
        actionRescanTypes = new QAction(GammaRay__MetaTypeBrowserWidget);
        actionRescanTypes->setObjectName("actionRescanTypes");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("view-refresh")));
        actionRescanTypes->setIcon(icon);
        verticalLayout = new QVBoxLayout(GammaRay__MetaTypeBrowserWidget);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        metaTypeSearchLine = new QLineEdit(GammaRay__MetaTypeBrowserWidget);
        metaTypeSearchLine->setObjectName("metaTypeSearchLine");

        verticalLayout->addWidget(metaTypeSearchLine);

        metaTypeView = new GammaRay::DeferredTreeView(GammaRay__MetaTypeBrowserWidget);
        metaTypeView->setObjectName("metaTypeView");
        metaTypeView->setContextMenuPolicy(Qt::CustomContextMenu);
        metaTypeView->setRootIsDecorated(false);
        metaTypeView->setUniformRowHeights(true);

        verticalLayout->addWidget(metaTypeView);


        retranslateUi(GammaRay__MetaTypeBrowserWidget);

        QMetaObject::connectSlotsByName(GammaRay__MetaTypeBrowserWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__MetaTypeBrowserWidget)
    {
        actionRescanTypes->setText(QCoreApplication::translate("GammaRay::MetaTypeBrowserWidget", "&Rescan Meta Types", nullptr));
#if QT_CONFIG(tooltip)
        actionRescanTypes->setToolTip(QCoreApplication::translate("GammaRay::MetaTypeBrowserWidget", "Check for changes to the meta type database.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)GammaRay__MetaTypeBrowserWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class MetaTypeBrowserWidget: public Ui_MetaTypeBrowserWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_METATYPEBROWSERWIDGET_H
