/********************************************************************************
** Form generated from reading UI file 'launchpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAUNCHPAGE_H
#define UI_LAUNCHPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_LaunchPage
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QLineEdit *progEdit;
    QLabel *label;
    QPushButton *progSelectButton;
    QLabel *label_5;
    QLineEdit *workDirEdit;
    QPushButton *workDirSelectButton;
    QLabel *label_2;
    QGridLayout *gridLayout;
    QListView *argsBox;
    QPushButton *addArgButton;
    QPushButton *removeArgButton;
    QSpacerItem *verticalSpacer;
    QFormLayout *formLayout;
    QLabel *label_4;
    QComboBox *probeBox;
    QLabel *label_3;
    QComboBox *accessMode;
    QLabel *errorLabel;

    void setupUi(QWidget *GammaRay__LaunchPage)
    {
        if (GammaRay__LaunchPage->objectName().isEmpty())
            GammaRay__LaunchPage->setObjectName("GammaRay__LaunchPage");
        GammaRay__LaunchPage->resize(401, 305);
        verticalLayout = new QVBoxLayout(GammaRay__LaunchPage);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        progEdit = new QLineEdit(GammaRay__LaunchPage);
        progEdit->setObjectName("progEdit");

        gridLayout_2->addWidget(progEdit, 0, 1, 1, 1);

        label = new QLabel(GammaRay__LaunchPage);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        progSelectButton = new QPushButton(GammaRay__LaunchPage);
        progSelectButton->setObjectName("progSelectButton");

        gridLayout_2->addWidget(progSelectButton, 0, 2, 1, 1);

        label_5 = new QLabel(GammaRay__LaunchPage);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 1, 0, 1, 1);

        workDirEdit = new QLineEdit(GammaRay__LaunchPage);
        workDirEdit->setObjectName("workDirEdit");

        gridLayout_2->addWidget(workDirEdit, 1, 1, 1, 1);

        workDirSelectButton = new QPushButton(GammaRay__LaunchPage);
        workDirSelectButton->setObjectName("workDirSelectButton");

        gridLayout_2->addWidget(workDirSelectButton, 1, 2, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        label_2 = new QLabel(GammaRay__LaunchPage);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        argsBox = new QListView(GammaRay__LaunchPage);
        argsBox->setObjectName("argsBox");

        gridLayout->addWidget(argsBox, 0, 0, 3, 1);

        addArgButton = new QPushButton(GammaRay__LaunchPage);
        addArgButton->setObjectName("addArgButton");

        gridLayout->addWidget(addArgButton, 0, 1, 1, 1);

        removeArgButton = new QPushButton(GammaRay__LaunchPage);
        removeArgButton->setObjectName("removeArgButton");

        gridLayout->addWidget(removeArgButton, 1, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 170, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label_4 = new QLabel(GammaRay__LaunchPage);
        label_4->setObjectName("label_4");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_4);

        probeBox = new QComboBox(GammaRay__LaunchPage);
        probeBox->setObjectName("probeBox");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, probeBox);

        label_3 = new QLabel(GammaRay__LaunchPage);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);

        accessMode = new QComboBox(GammaRay__LaunchPage);
        accessMode->addItem(QString());
        accessMode->addItem(QString());
        accessMode->addItem(QString());
        accessMode->setObjectName("accessMode");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, accessMode);


        verticalLayout->addLayout(formLayout);

        errorLabel = new QLabel(GammaRay__LaunchPage);
        errorLabel->setObjectName("errorLabel");
        errorLabel->setWordWrap(true);

        verticalLayout->addWidget(errorLabel);

#if QT_CONFIG(shortcut)
        label->setBuddy(progEdit);
        label_2->setBuddy(argsBox);
        label_4->setBuddy(probeBox);
        label_3->setBuddy(accessMode);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__LaunchPage);

        accessMode->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__LaunchPage);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__LaunchPage)
    {
        label->setText(QCoreApplication::translate("GammaRay::LaunchPage", "&Executable:", nullptr));
        progSelectButton->setText(QCoreApplication::translate("GammaRay::LaunchPage", "...", nullptr));
        label_5->setText(QCoreApplication::translate("GammaRay::LaunchPage", "Working Directory:", nullptr));
        workDirSelectButton->setText(QCoreApplication::translate("GammaRay::LaunchPage", "...", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::LaunchPage", "&Program arguments:", nullptr));
        addArgButton->setText(QCoreApplication::translate("GammaRay::LaunchPage", "&Add", nullptr));
        removeArgButton->setText(QCoreApplication::translate("GammaRay::LaunchPage", "&Remove", nullptr));
        label_4->setText(QCoreApplication::translate("GammaRay::LaunchPage", "Pro&be:", nullptr));
        label_3->setText(QCoreApplication::translate("GammaRay::LaunchPage", "&Access Mode:", nullptr));
        accessMode->setItemText(0, QCoreApplication::translate("GammaRay::LaunchPage", "Out-of-process, local-only", nullptr));
        accessMode->setItemText(1, QCoreApplication::translate("GammaRay::LaunchPage", "Out-of-process, remote debugging enabled", nullptr));
        accessMode->setItemText(2, QCoreApplication::translate("GammaRay::LaunchPage", "In-process", nullptr));

        errorLabel->setText(QCoreApplication::translate("GammaRay::LaunchPage", "TextLabel", nullptr));
        (void)GammaRay__LaunchPage;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class LaunchPage: public Ui_LaunchPage {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_LAUNCHPAGE_H
