/********************************************************************************
** Form generated from reading UI file 'translatorinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSLATORINSPECTORWIDGET_H
#define UI_TRANSLATORINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_TranslatorInspectorWidget
{
public:
    QAction *actionReset;
    QAction *actionSendLanguageChange;
    QVBoxLayout *verticalLayout_2;
    QSplitter *mainSplitter;
    GammaRay::DeferredTreeView *translatorList;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLineEdit *translationsSearchLine;
    GammaRay::DeferredTreeView *translationsView;

    void setupUi(QWidget *GammaRay__TranslatorInspectorWidget)
    {
        if (GammaRay__TranslatorInspectorWidget->objectName().isEmpty())
            GammaRay__TranslatorInspectorWidget->setObjectName("GammaRay__TranslatorInspectorWidget");
        GammaRay__TranslatorInspectorWidget->resize(400, 300);
        actionReset = new QAction(GammaRay__TranslatorInspectorWidget);
        actionReset->setObjectName("actionReset");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("edit-undo")));
        actionReset->setIcon(icon);
        actionSendLanguageChange = new QAction(GammaRay__TranslatorInspectorWidget);
        actionSendLanguageChange->setObjectName("actionSendLanguageChange");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("view-refresh")));
        actionSendLanguageChange->setIcon(icon1);
        verticalLayout_2 = new QVBoxLayout(GammaRay__TranslatorInspectorWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mainSplitter = new QSplitter(GammaRay__TranslatorInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        translatorList = new GammaRay::DeferredTreeView(mainSplitter);
        translatorList->setObjectName("translatorList");
        translatorList->setContextMenuPolicy(Qt::CustomContextMenu);
        translatorList->setRootIsDecorated(false);
        translatorList->setUniformRowHeights(true);
        mainSplitter->addWidget(translatorList);
        layoutWidget = new QWidget(mainSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        translationsSearchLine = new QLineEdit(layoutWidget);
        translationsSearchLine->setObjectName("translationsSearchLine");

        verticalLayout->addWidget(translationsSearchLine);

        translationsView = new GammaRay::DeferredTreeView(layoutWidget);
        translationsView->setObjectName("translationsView");
        translationsView->setContextMenuPolicy(Qt::CustomContextMenu);
        translationsView->setSelectionMode(QAbstractItemView::ContiguousSelection);
        translationsView->setRootIsDecorated(false);

        verticalLayout->addWidget(translationsView);

        mainSplitter->addWidget(layoutWidget);

        verticalLayout_2->addWidget(mainSplitter);


        retranslateUi(GammaRay__TranslatorInspectorWidget);

        QMetaObject::connectSlotsByName(GammaRay__TranslatorInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__TranslatorInspectorWidget)
    {
        actionReset->setText(QCoreApplication::translate("GammaRay::TranslatorInspectorWidget", "Reset Selected Translations", nullptr));
#if QT_CONFIG(tooltip)
        actionReset->setToolTip(QCoreApplication::translate("GammaRay::TranslatorInspectorWidget", "Reset manually changed translations back to their original content.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSendLanguageChange->setText(QCoreApplication::translate("GammaRay::TranslatorInspectorWidget", "Send LanguageChange Event", nullptr));
#if QT_CONFIG(tooltip)
        actionSendLanguageChange->setToolTip(QCoreApplication::translate("GammaRay::TranslatorInspectorWidget", "Sends a QEvent::LanguageChange event to the application to reload changed translations.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)GammaRay__TranslatorInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class TranslatorInspectorWidget: public Ui_TranslatorInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_TRANSLATORINSPECTORWIDGET_H
