/********************************************************************************
** Form generated from reading UI file 'connectpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONNECTPAGE_H
#define UI_CONNECTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_ConnectPage
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLineEdit *host;
    QTreeView *instanceView;

    void setupUi(QWidget *GammaRay__ConnectPage)
    {
        if (GammaRay__ConnectPage->objectName().isEmpty())
            GammaRay__ConnectPage->setObjectName("GammaRay__ConnectPage");
        GammaRay__ConnectPage->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__ConnectPage);
        verticalLayout->setObjectName("verticalLayout");
        label_3 = new QLabel(GammaRay__ConnectPage);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(GammaRay__ConnectPage);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        host = new QLineEdit(GammaRay__ConnectPage);
        host->setObjectName("host");

        horizontalLayout->addWidget(host);


        verticalLayout->addLayout(horizontalLayout);

        instanceView = new QTreeView(GammaRay__ConnectPage);
        instanceView->setObjectName("instanceView");
        instanceView->setRootIsDecorated(false);
        instanceView->setUniformRowHeights(true);

        verticalLayout->addWidget(instanceView);

#if QT_CONFIG(shortcut)
        label->setBuddy(host);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__ConnectPage);

        QMetaObject::connectSlotsByName(GammaRay__ConnectPage);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ConnectPage)
    {
        label_3->setText(QCoreApplication::translate("GammaRay::ConnectPage", "Connect to a running remote instance.", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::ConnectPage", "Url:", nullptr));
        (void)GammaRay__ConnectPage;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ConnectPage: public Ui_ConnectPage {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_CONNECTPAGE_H
