/********************************************************************************
** Form generated from reading UI file 'aboutwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTWIDGET_H
#define UI_ABOUTWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "themedimagelabel.h"

namespace GammaRay {

class Ui_AboutWidget
{
public:
    QHBoxLayout *horizontalLayout;
    ThemedImageLabel *logoLabel;
    QVBoxLayout *verticalLayout;
    QLabel *titleLabel;
    QLabel *textHeader;
    QTextBrowser *textAuthors;
    QLabel *textFooter;

    void setupUi(QWidget *GammaRay__AboutWidget)
    {
        if (GammaRay__AboutWidget->objectName().isEmpty())
            GammaRay__AboutWidget->setObjectName("GammaRay__AboutWidget");
        GammaRay__AboutWidget->resize(400, 300);
        horizontalLayout = new QHBoxLayout(GammaRay__AboutWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        logoLabel = new ThemedImageLabel(GammaRay__AboutWidget);
        logoLabel->setObjectName("logoLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(logoLabel->sizePolicy().hasHeightForWidth());
        logoLabel->setSizePolicy(sizePolicy);
        logoLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        horizontalLayout->addWidget(logoLabel, 0, Qt::AlignTop);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(GammaRay__AboutWidget);
        titleLabel->setObjectName("titleLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(titleLabel->sizePolicy().hasHeightForWidth());
        titleLabel->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(true);
        titleLabel->setFont(font);

        verticalLayout->addWidget(titleLabel);

        textHeader = new QLabel(GammaRay__AboutWidget);
        textHeader->setObjectName("textHeader");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(textHeader->sizePolicy().hasHeightForWidth());
        textHeader->setSizePolicy(sizePolicy2);
        textHeader->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        textHeader->setWordWrap(true);
        textHeader->setOpenExternalLinks(true);
        textHeader->setTextInteractionFlags(Qt::TextBrowserInteraction);

        verticalLayout->addWidget(textHeader);

        textAuthors = new QTextBrowser(GammaRay__AboutWidget);
        textAuthors->setObjectName("textAuthors");
        QPalette palette;
        QBrush brush(QColor(0, 0, 0, 0));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Base, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Base, brush);
        QBrush brush1(QColor(236, 236, 236, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Base, brush1);
        textAuthors->setPalette(palette);
        textAuthors->setFrameShape(QFrame::NoFrame);
        textAuthors->setFrameShadow(QFrame::Plain);
        textAuthors->setOpenExternalLinks(true);

        verticalLayout->addWidget(textAuthors);

        textFooter = new QLabel(GammaRay__AboutWidget);
        textFooter->setObjectName("textFooter");
        sizePolicy2.setHeightForWidth(textFooter->sizePolicy().hasHeightForWidth());
        textFooter->setSizePolicy(sizePolicy2);
        textFooter->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        textFooter->setWordWrap(true);
        textFooter->setOpenExternalLinks(true);
        textFooter->setTextInteractionFlags(Qt::TextBrowserInteraction);

        verticalLayout->addWidget(textFooter);


        horizontalLayout->addLayout(verticalLayout);


        retranslateUi(GammaRay__AboutWidget);

        QMetaObject::connectSlotsByName(GammaRay__AboutWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__AboutWidget)
    {
        (void)GammaRay__AboutWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class AboutWidget: public Ui_AboutWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_ABOUTWIDGET_H
