/********************************************************************************
** Form generated from reading UI file 'localetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOCALETAB_H
#define UI_LOCALETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_LocaleTab
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *mainSplitter;
    QTableView *accessorTable;
    QWidget *widget;
    QVBoxLayout *verticalLayout_2;
    QLineEdit *localeSearchLine;
    QTableView *localeTable;

    void setupUi(QWidget *GammaRay__LocaleTab)
    {
        if (GammaRay__LocaleTab->objectName().isEmpty())
            GammaRay__LocaleTab->setObjectName("GammaRay__LocaleTab");
        GammaRay__LocaleTab->resize(654, 506);
        verticalLayout = new QVBoxLayout(GammaRay__LocaleTab);
        verticalLayout->setObjectName("verticalLayout");
        mainSplitter = new QSplitter(GammaRay__LocaleTab);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Vertical);
        accessorTable = new QTableView(mainSplitter);
        accessorTable->setObjectName("accessorTable");
        accessorTable->setSelectionMode(QAbstractItemView::NoSelection);
        accessorTable->setShowGrid(false);
        mainSplitter->addWidget(accessorTable);
        accessorTable->horizontalHeader()->setVisible(false);
        accessorTable->verticalHeader()->setVisible(false);
        widget = new QWidget(mainSplitter);
        widget->setObjectName("widget");
        verticalLayout_2 = new QVBoxLayout(widget);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        localeSearchLine = new QLineEdit(widget);
        localeSearchLine->setObjectName("localeSearchLine");

        verticalLayout_2->addWidget(localeSearchLine);

        localeTable = new QTableView(widget);
        localeTable->setObjectName("localeTable");
        localeTable->setAlternatingRowColors(true);
        localeTable->setWordWrap(false);

        verticalLayout_2->addWidget(localeTable);

        mainSplitter->addWidget(widget);

        verticalLayout->addWidget(mainSplitter);


        retranslateUi(GammaRay__LocaleTab);

        QMetaObject::connectSlotsByName(GammaRay__LocaleTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__LocaleTab)
    {
        (void)GammaRay__LocaleTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class LocaleTab: public Ui_LocaleTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_LOCALETAB_H
