/********************************************************************************
** Form generated from reading UI file 'messagehandlerwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MESSAGEHANDLERWIDGET_H
#define UI_MESSAGEHANDLERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_MessageHandlerWidget
{
public:
    QHBoxLayout *horizontalLayout_3;
    QTabWidget *tabWidget;
    QWidget *messagesTab;
    QVBoxLayout *verticalLayout_2;
    QSplitter *mainSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLineEdit *messageSearchLine;
    GammaRay::DeferredTreeView *messageView;
    GammaRay::DeferredTreeView *backtraceView;
    QWidget *categoriesTab;
    QVBoxLayout *verticalLayout_5;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents_3;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *saveAllConf;
    QPushButton *saveModConf;
    QPushButton *copyModConf;
    QSpacerItem *horizontalSpacer;
    GammaRay::DeferredTreeView *categoriesView;

    void setupUi(QWidget *GammaRay__MessageHandlerWidget)
    {
        if (GammaRay__MessageHandlerWidget->objectName().isEmpty())
            GammaRay__MessageHandlerWidget->setObjectName("GammaRay__MessageHandlerWidget");
        GammaRay__MessageHandlerWidget->resize(772, 426);
        horizontalLayout_3 = new QHBoxLayout(GammaRay__MessageHandlerWidget);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        tabWidget = new QTabWidget(GammaRay__MessageHandlerWidget);
        tabWidget->setObjectName("tabWidget");
        messagesTab = new QWidget();
        messagesTab->setObjectName("messagesTab");
        verticalLayout_2 = new QVBoxLayout(messagesTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mainSplitter = new QSplitter(messagesTab);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Orientation::Horizontal);
        layoutWidget = new QWidget(mainSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        messageSearchLine = new QLineEdit(layoutWidget);
        messageSearchLine->setObjectName("messageSearchLine");

        verticalLayout->addWidget(messageSearchLine);

        messageView = new GammaRay::DeferredTreeView(layoutWidget);
        messageView->setObjectName("messageView");
        messageView->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        messageView->setRootIsDecorated(false);

        verticalLayout->addWidget(messageView);

        mainSplitter->addWidget(layoutWidget);
        backtraceView = new GammaRay::DeferredTreeView(mainSplitter);
        backtraceView->setObjectName("backtraceView");
        backtraceView->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        backtraceView->setRootIsDecorated(false);
        backtraceView->setUniformRowHeights(true);
        mainSplitter->addWidget(backtraceView);

        verticalLayout_2->addWidget(mainSplitter);

        tabWidget->addTab(messagesTab, QString());
        categoriesTab = new QWidget();
        categoriesTab->setObjectName("categoriesTab");
        verticalLayout_5 = new QVBoxLayout(categoriesTab);
        verticalLayout_5->setObjectName("verticalLayout_5");
        scrollArea = new QScrollArea(categoriesTab);
        scrollArea->setObjectName("scrollArea");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(scrollArea->sizePolicy().hasHeightForWidth());
        scrollArea->setSizePolicy(sizePolicy);
        scrollArea->setFrameShadow(QFrame::Shadow::Sunken);
        scrollArea->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContents);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName("scrollAreaWidgetContents_3");
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 730, 68));
        horizontalLayout_4 = new QHBoxLayout(scrollAreaWidgetContents_3);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        saveAllConf = new QPushButton(scrollAreaWidgetContents_3);
        saveAllConf->setObjectName("saveAllConf");

        horizontalLayout_4->addWidget(saveAllConf);

        saveModConf = new QPushButton(scrollAreaWidgetContents_3);
        saveModConf->setObjectName("saveModConf");

        horizontalLayout_4->addWidget(saveModConf);

        copyModConf = new QPushButton(scrollAreaWidgetContents_3);
        copyModConf->setObjectName("copyModConf");

        horizontalLayout_4->addWidget(copyModConf);

        horizontalSpacer = new QSpacerItem(328, 14, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents_3);

        verticalLayout_5->addWidget(scrollArea);

        categoriesView = new GammaRay::DeferredTreeView(categoriesTab);
        categoriesView->setObjectName("categoriesView");
        categoriesView->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContents);
        categoriesView->setRootIsDecorated(false);
        categoriesView->setUniformRowHeights(true);

        verticalLayout_5->addWidget(categoriesView);

        tabWidget->addTab(categoriesTab, QString());

        horizontalLayout_3->addWidget(tabWidget);


        retranslateUi(GammaRay__MessageHandlerWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__MessageHandlerWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__MessageHandlerWidget)
    {
        tabWidget->setTabText(tabWidget->indexOf(messagesTab), QCoreApplication::translate("GammaRay::MessageHandlerWidget", "Messages", nullptr));
        saveAllConf->setText(QCoreApplication::translate("GammaRay::MessageHandlerWidget", "Save All Logging Config", nullptr));
        saveModConf->setText(QCoreApplication::translate("GammaRay::MessageHandlerWidget", "Save Modified Logging Config", nullptr));
        copyModConf->setText(QCoreApplication::translate("GammaRay::MessageHandlerWidget", "Copy Modified Config for Env Var", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(categoriesTab), QCoreApplication::translate("GammaRay::MessageHandlerWidget", "Categories", nullptr));
        (void)GammaRay__MessageHandlerWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class MessageHandlerWidget: public Ui_MessageHandlerWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_MESSAGEHANDLERWIDGET_H
