/****************************************************************************
** Meta object code from reading C++ file 'networksupport.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/network/networksupport.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'networksupport.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay14NetworkSupportE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::NetworkSupport::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay14NetworkSupportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::NetworkSupport"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NetworkSupport, qt_meta_tag_ZN8GammaRay14NetworkSupportE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::NetworkSupport::staticMetaObject = { {
    QMetaObject::SuperData::link<NetworkSupportInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14NetworkSupportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14NetworkSupportE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay14NetworkSupportE_t>.metaTypes,
    nullptr
} };

void GammaRay::NetworkSupport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NetworkSupport *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::NetworkSupport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::NetworkSupport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14NetworkSupportE_t>.strings))
        return static_cast<void*>(this);
    return NetworkSupportInterface::qt_metacast(_clname);
}

int GammaRay::NetworkSupport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NetworkSupportInterface::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::NetworkSupportFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::NetworkSupportFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NetworkSupportFactory, qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::NetworkSupportFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::NetworkSupportFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NetworkSupportFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::NetworkSupportFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::NetworkSupportFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21NetworkSupportFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QObject,NetworkSupport>"))
        return static_cast< StandardToolFactory<QObject,NetworkSupport>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::NetworkSupportFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_NetworkSupportFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x75,  'N',  'e',  't',  'w',  'o',  'r', 
    'k',  'S',  'u',  'p',  'p',  'o',  'r',  't', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x66,  'h',  'i',  'd',  'd',  'e', 
    'n',  0xf4,  0x62,  'i',  'd',  0x70,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'n', 
    'e',  't',  'w',  'o',  'r',  'k',  0x64,  'n', 
    'a',  'm',  'e',  0x67,  'N',  'e',  't',  'w', 
    'o',  'r',  'k',  0x68,  'n',  'a',  'm',  'e', 
    '[',  'd',  'e',  ']',  0x68,  'N',  'e',  't', 
    'z',  'w',  'e',  'r',  'k',  0x65,  't',  'y', 
    'p',  'e',  's',  0x81,  0x67,  'Q',  'O',  'b', 
    'j',  'e',  'c',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::NetworkSupportFactory, NetworkSupportFactory, qt_pluginMetaDataV2_NetworkSupportFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_NetworkSupportFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x75,  'N',  'e',  't',  'w',  'o',  'r', 
    'k',  'S',  'u',  'p',  'p',  'o',  'r',  't', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x66,  'h',  'i',  'd',  'd',  'e', 
    'n',  0xf4,  0x62,  'i',  'd',  0x70,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'n', 
    'e',  't',  'w',  'o',  'r',  'k',  0x64,  'n', 
    'a',  'm',  'e',  0x67,  'N',  'e',  't',  'w', 
    'o',  'r',  'k',  0x68,  'n',  'a',  'm',  'e', 
    '[',  'd',  'e',  ']',  0x68,  'N',  'e',  't', 
    'z',  'w',  'e',  'r',  'k',  0x65,  't',  'y', 
    'p',  'e',  's',  0x81,  0x67,  'Q',  'O',  'b', 
    'j',  'e',  'c',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::NetworkSupportFactory, NetworkSupportFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
