/********************************************************************************
** Form generated from reading UI file 'modelinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MODELINSPECTORWIDGET_H
#define UI_MODELINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ModelInspectorWidget
{
public:
    QVBoxLayout *verticalLayout_5;
    QSplitter *mainSplitter;
    QSplitter *modelSplitter;
    QWidget *layoutWidget_3;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_3;
    QLineEdit *modelSearchLine;
    GammaRay::DeferredTreeView *modelView;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_6;
    GammaRay::DeferredTreeView *selectionModelsView;
    QWidget *layoutWidget1;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    GammaRay::DeferredTreeView *modelContentView;
    QWidget *layoutWidget2;
    QVBoxLayout *verticalLayout;
    QLabel *label_2;
    QFormLayout *formLayout_2;
    QLabel *label_8;
    QLineEdit *indexLineEdit;
    QLabel *label_4;
    QLineEdit *internalIdLineEdit;
    QLabel *label_5;
    QLineEdit *internalPtrLineEdit;
    QLabel *label_7;
    QPlainTextEdit *flagsPlainTextEdit;
    GammaRay::DeferredTreeView *modelCellView;

    void setupUi(QWidget *GammaRay__ModelInspectorWidget)
    {
        if (GammaRay__ModelInspectorWidget->objectName().isEmpty())
            GammaRay__ModelInspectorWidget->setObjectName("GammaRay__ModelInspectorWidget");
        GammaRay__ModelInspectorWidget->resize(788, 479);
        verticalLayout_5 = new QVBoxLayout(GammaRay__ModelInspectorWidget);
        verticalLayout_5->setObjectName("verticalLayout_5");
        mainSplitter = new QSplitter(GammaRay__ModelInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Orientation::Horizontal);
        modelSplitter = new QSplitter(mainSplitter);
        modelSplitter->setObjectName("modelSplitter");
        modelSplitter->setOrientation(Qt::Orientation::Vertical);
        layoutWidget_3 = new QWidget(modelSplitter);
        layoutWidget_3->setObjectName("layoutWidget_3");
        verticalLayout_3 = new QVBoxLayout(layoutWidget_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        label_3 = new QLabel(layoutWidget_3);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout_3->addWidget(label_3);

        modelSearchLine = new QLineEdit(layoutWidget_3);
        modelSearchLine->setObjectName("modelSearchLine");

        verticalLayout_3->addWidget(modelSearchLine);

        modelView = new GammaRay::DeferredTreeView(layoutWidget_3);
        modelView->setObjectName("modelView");
        modelView->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        modelView->setUniformRowHeights(true);

        verticalLayout_3->addWidget(modelView);

        modelSplitter->addWidget(layoutWidget_3);
        layoutWidget = new QWidget(modelSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout_4 = new QVBoxLayout(layoutWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        label_6 = new QLabel(layoutWidget);
        label_6->setObjectName("label_6");
        label_6->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout_4->addWidget(label_6);

        selectionModelsView = new GammaRay::DeferredTreeView(layoutWidget);
        selectionModelsView->setObjectName("selectionModelsView");
        selectionModelsView->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        selectionModelsView->setRootIsDecorated(false);
        selectionModelsView->setUniformRowHeights(true);

        verticalLayout_4->addWidget(selectionModelsView);

        modelSplitter->addWidget(layoutWidget);
        mainSplitter->addWidget(modelSplitter);
        layoutWidget1 = new QWidget(mainSplitter);
        layoutWidget1->setObjectName("layoutWidget1");
        verticalLayout_2 = new QVBoxLayout(layoutWidget1);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(layoutWidget1);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout_2->addWidget(label);

        modelContentView = new GammaRay::DeferredTreeView(layoutWidget1);
        modelContentView->setObjectName("modelContentView");
        modelContentView->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectItems);

        verticalLayout_2->addWidget(modelContentView);

        mainSplitter->addWidget(layoutWidget1);
        layoutWidget2 = new QWidget(mainSplitter);
        layoutWidget2->setObjectName("layoutWidget2");
        verticalLayout = new QVBoxLayout(layoutWidget2);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(layoutWidget2);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(label_2);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        label_8 = new QLabel(layoutWidget2);
        label_8->setObjectName("label_8");
        label_8->setMinimumSize(QSize(55, 0));
        label_8->setWordWrap(true);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_8);

        indexLineEdit = new QLineEdit(layoutWidget2);
        indexLineEdit->setObjectName("indexLineEdit");
        indexLineEdit->setMinimumSize(QSize(72, 0));
        indexLineEdit->setReadOnly(true);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, indexLineEdit);

        label_4 = new QLabel(layoutWidget2);
        label_4->setObjectName("label_4");
        label_4->setMinimumSize(QSize(55, 0));
        label_4->setWordWrap(true);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label_4);

        internalIdLineEdit = new QLineEdit(layoutWidget2);
        internalIdLineEdit->setObjectName("internalIdLineEdit");
        internalIdLineEdit->setMinimumSize(QSize(72, 0));
        internalIdLineEdit->setReadOnly(true);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, internalIdLineEdit);

        label_5 = new QLabel(layoutWidget2);
        label_5->setObjectName("label_5");
        label_5->setMinimumSize(QSize(55, 0));
        label_5->setWordWrap(true);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, label_5);

        internalPtrLineEdit = new QLineEdit(layoutWidget2);
        internalPtrLineEdit->setObjectName("internalPtrLineEdit");
        internalPtrLineEdit->setMinimumSize(QSize(72, 0));
        internalPtrLineEdit->setReadOnly(true);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, internalPtrLineEdit);

        label_7 = new QLabel(layoutWidget2);
        label_7->setObjectName("label_7");
        label_7->setMinimumSize(QSize(55, 0));

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, label_7);

        flagsPlainTextEdit = new QPlainTextEdit(layoutWidget2);
        flagsPlainTextEdit->setObjectName("flagsPlainTextEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(flagsPlainTextEdit->sizePolicy().hasHeightForWidth());
        flagsPlainTextEdit->setSizePolicy(sizePolicy);
        flagsPlainTextEdit->setMinimumSize(QSize(72, 0));
        flagsPlainTextEdit->setAcceptDrops(false);
        flagsPlainTextEdit->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContents);
        flagsPlainTextEdit->setUndoRedoEnabled(false);
        flagsPlainTextEdit->setReadOnly(true);
        flagsPlainTextEdit->setPlainText(QString::fromUtf8(""));

        formLayout_2->setWidget(3, QFormLayout::ItemRole::FieldRole, flagsPlainTextEdit);


        verticalLayout->addLayout(formLayout_2);

        modelCellView = new GammaRay::DeferredTreeView(layoutWidget2);
        modelCellView->setObjectName("modelCellView");
        modelCellView->setRootIsDecorated(false);
        modelCellView->setUniformRowHeights(true);

        verticalLayout->addWidget(modelCellView);

        mainSplitter->addWidget(layoutWidget2);

        verticalLayout_5->addWidget(mainSplitter);


        retranslateUi(GammaRay__ModelInspectorWidget);

        QMetaObject::connectSlotsByName(GammaRay__ModelInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ModelInspectorWidget)
    {
        label_3->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Models", nullptr));
        label_6->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Selection Models", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Model Content", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Cell Content", nullptr));
        label_8->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Model index:", nullptr));
        indexLineEdit->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Invalid", nullptr));
        label_4->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Internal id:", nullptr));
        label_5->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Internal pointer:", nullptr));
        label_7->setText(QCoreApplication::translate("GammaRay::ModelInspectorWidget", "Flags:", nullptr));
        (void)GammaRay__ModelInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ModelInspectorWidget: public Ui_ModelInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_MODELINSPECTORWIDGET_H
