/****************************************************************************
** Meta object code from reading C++ file 'uistatemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/ui/uistatemanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'uistatemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay14UIStateManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::UIStateManager::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay14UIStateManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::UIStateManager",
        "reset",
        "",
        "restoreState",
        "saveState",
        "restoreWindowState",
        "saveWindowState",
        "restoreSplitterState",
        "QSplitter*",
        "splitter",
        "saveSplitterState",
        "restoreHeaderState",
        "QHeaderView*",
        "header",
        "saveHeaderState",
        "headerSectionCountChanged",
        "widgetResized",
        "QWidget*",
        "widget",
        "widgetCustomized"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'reset'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreState'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveState'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreWindowState'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'saveWindowState'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'restoreSplitterState'
        QtMocHelpers::SlotData<void(QSplitter *)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'restoreSplitterState'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveSplitterState'
        QtMocHelpers::SlotData<void(QSplitter *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveSplitterState'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'restoreHeaderState'
        QtMocHelpers::SlotData<void(QHeaderView *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'restoreHeaderState'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveHeaderState'
        QtMocHelpers::SlotData<void(QHeaderView *)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'saveHeaderState'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'headerSectionCountChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'widgetResized'
        QtMocHelpers::SlotData<void(QWidget *)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'widgetCustomized'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UIStateManager, qt_meta_tag_ZN8GammaRay14UIStateManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::UIStateManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14UIStateManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14UIStateManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay14UIStateManagerE_t>.metaTypes,
    nullptr
} };

void GammaRay::UIStateManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UIStateManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reset(); break;
        case 1: _t->restoreState(); break;
        case 2: _t->saveState(); break;
        case 3: _t->restoreWindowState(); break;
        case 4: _t->saveWindowState(); break;
        case 5: _t->restoreSplitterState((*reinterpret_cast<std::add_pointer_t<QSplitter*>>(_a[1]))); break;
        case 6: _t->restoreSplitterState(); break;
        case 7: _t->saveSplitterState((*reinterpret_cast<std::add_pointer_t<QSplitter*>>(_a[1]))); break;
        case 8: _t->saveSplitterState(); break;
        case 9: _t->restoreHeaderState((*reinterpret_cast<std::add_pointer_t<QHeaderView*>>(_a[1]))); break;
        case 10: _t->restoreHeaderState(); break;
        case 11: _t->saveHeaderState((*reinterpret_cast<std::add_pointer_t<QHeaderView*>>(_a[1]))); break;
        case 12: _t->saveHeaderState(); break;
        case 13: _t->headerSectionCountChanged(); break;
        case 14: _t->widgetResized((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 15: _t->widgetCustomized(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QHeaderView* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QHeaderView* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *GammaRay::UIStateManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::UIStateManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14UIStateManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::UIStateManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
