/********************************************************************************
** Form generated from reading UI file 'paintbufferviewer.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAINTBUFFERVIEWER_H
#define UI_PAINTBUFFERVIEWER_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>
#include "paintanalyzerwidget.h"

namespace GammaRay {

class Ui_PaintBufferViewer
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::PaintAnalyzerWidget *paintAnalyzerWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GammaRay__PaintBufferViewer)
    {
        if (GammaRay__PaintBufferViewer->objectName().isEmpty())
            GammaRay__PaintBufferViewer->setObjectName("GammaRay__PaintBufferViewer");
        GammaRay__PaintBufferViewer->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__PaintBufferViewer);
        verticalLayout->setObjectName("verticalLayout");
        paintAnalyzerWidget = new GammaRay::PaintAnalyzerWidget(GammaRay__PaintBufferViewer);
        paintAnalyzerWidget->setObjectName("paintAnalyzerWidget");

        verticalLayout->addWidget(paintAnalyzerWidget);

        buttonBox = new QDialogButtonBox(GammaRay__PaintBufferViewer);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(GammaRay__PaintBufferViewer);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, GammaRay__PaintBufferViewer, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GammaRay__PaintBufferViewer, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(GammaRay__PaintBufferViewer);
    } // setupUi

    void retranslateUi(QDialog *GammaRay__PaintBufferViewer)
    {
        GammaRay__PaintBufferViewer->setWindowTitle(QCoreApplication::translate("GammaRay::PaintBufferViewer", "Analyze Painting", nullptr));
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PaintBufferViewer: public Ui_PaintBufferViewer {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PAINTBUFFERVIEWER_H
