/********************************************************************************
** Form generated from reading UI file 'classinfotab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CLASSINFOTAB_H
#define UI_CLASSINFOTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ClassInfoTab
{
public:
    QVBoxLayout *verticalLayout_4;
    QLineEdit *classInfoSearchLine;
    GammaRay::DeferredTreeView *classInfoView;

    void setupUi(QWidget *GammaRay__ClassInfoTab)
    {
        if (GammaRay__ClassInfoTab->objectName().isEmpty())
            GammaRay__ClassInfoTab->setObjectName("GammaRay__ClassInfoTab");
        GammaRay__ClassInfoTab->resize(400, 300);
        verticalLayout_4 = new QVBoxLayout(GammaRay__ClassInfoTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        classInfoSearchLine = new QLineEdit(GammaRay__ClassInfoTab);
        classInfoSearchLine->setObjectName("classInfoSearchLine");

        verticalLayout_4->addWidget(classInfoSearchLine);

        classInfoView = new GammaRay::DeferredTreeView(GammaRay__ClassInfoTab);
        classInfoView->setObjectName("classInfoView");
        classInfoView->setRootIsDecorated(false);

        verticalLayout_4->addWidget(classInfoView);


        retranslateUi(GammaRay__ClassInfoTab);

        QMetaObject::connectSlotsByName(GammaRay__ClassInfoTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ClassInfoTab)
    {
        (void)GammaRay__ClassInfoTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ClassInfoTab: public Ui_ClassInfoTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_CLASSINFOTAB_H
