/*
  networkconfigurationmodelroles.h

  This file is part of GammaRay, the Qt application inspection and manipulation tool.

  SPDX-FileCopyrightText: 2017 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
  Author: Volker Krause <volker.krause@kdab.com>

  SPDX-License-Identifier: GPL-2.0-or-later

  Contact KDAB at <info@kdab.com> for commercial licensing options.
*/

#ifndef GAMMARAY_NETWORKCONFIGURATIONMODELROLES_H
#define GAMMARAY_NETWORKCONFIGURATIONMODELROLES_H

#include <common/modelroles.h>

namespace GammaRay {

namespace NetworkConfigurationModelRoles {
enum Role
{
    DefaultConfigRole = UserRole
};
}
}

#endif // GAMMARAY_NETWORKCONFIGURATIONMODELROLES_H
