/********************************************************************************
** Form generated from reading UI file 'inspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INSPECTORWIDGET_H
#define UI_INSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_InspectorWidget
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QTreeView *resourcesView;
    QLabel *resourceInfo;
    QTreeView *clientsView;

    void setupUi(QWidget *Widget)
    {
        if (Widget->objectName().isEmpty())
            Widget->setObjectName("Widget");
        Widget->resize(802, 675);
        gridLayout = new QGridLayout(Widget);
        gridLayout->setObjectName("gridLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        resourcesView = new QTreeView(Widget);
        resourcesView->setObjectName("resourcesView");
        resourcesView->setAutoExpandDelay(-1);

        verticalLayout->addWidget(resourcesView);

        resourceInfo = new QLabel(Widget);
        resourceInfo->setObjectName("resourceInfo");

        verticalLayout->addWidget(resourceInfo);


        gridLayout->addLayout(verticalLayout, 0, 1, 2, 1);

        clientsView = new QTreeView(Widget);
        clientsView->setObjectName("clientsView");
        clientsView->setContextMenuPolicy(Qt::CustomContextMenu);
        clientsView->setSelectionBehavior(QAbstractItemView::SelectRows);
        clientsView->setRootIsDecorated(false);

        gridLayout->addWidget(clientsView, 0, 0, 1, 1);


        retranslateUi(Widget);

        QMetaObject::connectSlotsByName(Widget);
    } // setupUi

    void retranslateUi(QWidget *Widget)
    {
        (void)Widget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class InspectorWidget: public Ui_InspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_INSPECTORWIDGET_H
