/********************************************************************************
** Form generated from reading UI file 'palettedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PALETTEDIALOG_H
#define UI_PALETTEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_PaletteDialog
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *paletteView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GammaRay__PaletteDialog)
    {
        if (GammaRay__PaletteDialog->objectName().isEmpty())
            GammaRay__PaletteDialog->setObjectName("GammaRay__PaletteDialog");
        GammaRay__PaletteDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__PaletteDialog);
        verticalLayout->setObjectName("verticalLayout");
        paletteView = new GammaRay::DeferredTreeView(GammaRay__PaletteDialog);
        paletteView->setObjectName("paletteView");
        paletteView->setRootIsDecorated(false);
        paletteView->setUniformRowHeights(true);
        paletteView->header()->setStretchLastSection(false);

        verticalLayout->addWidget(paletteView);

        buttonBox = new QDialogButtonBox(GammaRay__PaletteDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Save);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(GammaRay__PaletteDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, GammaRay__PaletteDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GammaRay__PaletteDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(GammaRay__PaletteDialog);
    } // setupUi

    void retranslateUi(QDialog *GammaRay__PaletteDialog)
    {
        (void)GammaRay__PaletteDialog;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PaletteDialog: public Ui_PaletteDialog {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PALETTEDIALOG_H
