/****************************************************************************
** Meta object code from reading C++ file 'remoteviewwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/ui/remoteviewwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'remoteviewwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::RemoteViewWidget::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::RemoteViewWidget",
        "zoomChanged",
        "",
        "zoomLevelChanged",
        "zoomLevelIndex",
        "interactionModeChanged",
        "stateChanged",
        "frameChanged",
        "reset",
        "setZoom",
        "zoom",
        "setZoomLevel",
        "index",
        "zoomIn",
        "zoomOut",
        "fitToView",
        "centerView",
        "interactionActionTriggered",
        "QAction*",
        "action",
        "pickElementId",
        "QModelIndex",
        "elementsAtReceived",
        "GammaRay::ObjectIds",
        "ids",
        "bestCandidate",
        "frameUpdated",
        "GammaRay::RemoteViewFrame",
        "frame",
        "enableFPS",
        "showFPS",
        "updateUserViewport",
        "restoreState",
        "state"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'zoomChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomLevelChanged'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'interactionModeChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'frameChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reset'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setZoom'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'setZoomLevel'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fitToView'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'centerView'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'interactionActionTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'pickElementId'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 12 },
        }}),
        // Slot 'elementsAtReceived'
        QtMocHelpers::SlotData<void(const GammaRay::ObjectIds &, int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::Int, 25 },
        }}),
        // Slot 'frameUpdated'
        QtMocHelpers::SlotData<void(const GammaRay::RemoteViewFrame &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'enableFPS'
        QtMocHelpers::SlotData<void(const bool)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Slot 'updateUserViewport'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'restoreState'
        QtMocHelpers::MethodData<void(const QByteArray &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RemoteViewWidget, qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::RemoteViewWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t>.metaTypes,
    nullptr
} };

void GammaRay::RemoteViewWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteViewWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->zoomChanged(); break;
        case 1: _t->zoomLevelChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->interactionModeChanged(); break;
        case 3: _t->stateChanged(); break;
        case 4: _t->frameChanged(); break;
        case 5: _t->reset(); break;
        case 6: _t->setZoom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setZoomLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->zoomIn(); break;
        case 9: _t->zoomOut(); break;
        case 10: _t->fitToView(); break;
        case 11: _t->centerView(); break;
        case 12: _t->interactionActionTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 13: _t->pickElementId((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 14: _t->elementsAtReceived((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectIds>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 15: _t->frameUpdated((*reinterpret_cast<std::add_pointer_t<GammaRay::RemoteViewFrame>>(_a[1]))); break;
        case 16: _t->enableFPS((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->updateUserViewport(); break;
        case 18: _t->restoreState((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectIds >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::RemoteViewFrame >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RemoteViewWidget::*)()>(_a, &RemoteViewWidget::zoomChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewWidget::*)(int )>(_a, &RemoteViewWidget::zoomLevelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewWidget::*)()>(_a, &RemoteViewWidget::interactionModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewWidget::*)()>(_a, &RemoteViewWidget::stateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewWidget::*)()>(_a, &RemoteViewWidget::frameChanged, 4))
            return;
    }
}

const QMetaObject *GammaRay::RemoteViewWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::RemoteViewWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16RemoteViewWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int GammaRay::RemoteViewWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::RemoteViewWidget::zoomChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::RemoteViewWidget::zoomLevelChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GammaRay::RemoteViewWidget::interactionModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GammaRay::RemoteViewWidget::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GammaRay::RemoteViewWidget::frameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
