/********************************************************************************
** Form generated from reading UI file 'stacktracetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STACKTRACETAB_H
#define UI_STACKTRACETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_StackTraceTab
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *stackTraceView;

    void setupUi(QWidget *GammaRay__StackTraceTab)
    {
        if (GammaRay__StackTraceTab->objectName().isEmpty())
            GammaRay__StackTraceTab->setObjectName("GammaRay__StackTraceTab");
        GammaRay__StackTraceTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__StackTraceTab);
        verticalLayout->setObjectName("verticalLayout");
        stackTraceView = new QTreeView(GammaRay__StackTraceTab);
        stackTraceView->setObjectName("stackTraceView");
        stackTraceView->setContextMenuPolicy(Qt::CustomContextMenu);
        stackTraceView->setRootIsDecorated(false);
        stackTraceView->setUniformRowHeights(true);

        verticalLayout->addWidget(stackTraceView);


        retranslateUi(GammaRay__StackTraceTab);

        QMetaObject::connectSlotsByName(GammaRay__StackTraceTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__StackTraceTab)
    {
        (void)GammaRay__StackTraceTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class StackTraceTab: public Ui_StackTraceTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_STACKTRACETAB_H
