/********************************************************************************
** Form generated from reading UI file 'gridsettingswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRIDSETTINGSWIDGET_H
#define UI_GRIDSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_GridSettingsWidget
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *gbEnabled;
    QGridLayout *gridLayout_2;
    QLabel *lXOffset;
    QSpinBox *sbXOffset;
    QLabel *lYOffset;
    QSpinBox *sbYOffset;
    QLabel *lCellWidth;
    QSpinBox *sbCellWidth;
    QLabel *lCellHeight;
    QSpinBox *sbCellHeight;

    void setupUi(QWidget *GammaRay__GridSettingsWidget)
    {
        if (GammaRay__GridSettingsWidget->objectName().isEmpty())
            GammaRay__GridSettingsWidget->setObjectName("GammaRay__GridSettingsWidget");
        GammaRay__GridSettingsWidget->resize(199, 190);
        verticalLayout = new QVBoxLayout(GammaRay__GridSettingsWidget);
        verticalLayout->setObjectName("verticalLayout");
        gbEnabled = new QGroupBox(GammaRay__GridSettingsWidget);
        gbEnabled->setObjectName("gbEnabled");
        gbEnabled->setCheckable(true);
        gbEnabled->setChecked(false);
        gridLayout_2 = new QGridLayout(gbEnabled);
        gridLayout_2->setObjectName("gridLayout_2");
        lXOffset = new QLabel(gbEnabled);
        lXOffset->setObjectName("lXOffset");

        gridLayout_2->addWidget(lXOffset, 0, 0, 1, 1);

        sbXOffset = new QSpinBox(gbEnabled);
        sbXOffset->setObjectName("sbXOffset");
        sbXOffset->setMinimum(0);
        sbXOffset->setMaximum(9999);

        gridLayout_2->addWidget(sbXOffset, 0, 1, 1, 1);

        lYOffset = new QLabel(gbEnabled);
        lYOffset->setObjectName("lYOffset");

        gridLayout_2->addWidget(lYOffset, 1, 0, 1, 1);

        sbYOffset = new QSpinBox(gbEnabled);
        sbYOffset->setObjectName("sbYOffset");
        sbYOffset->setMinimum(0);
        sbYOffset->setMaximum(9999);

        gridLayout_2->addWidget(sbYOffset, 1, 1, 1, 1);

        lCellWidth = new QLabel(gbEnabled);
        lCellWidth->setObjectName("lCellWidth");

        gridLayout_2->addWidget(lCellWidth, 2, 0, 1, 1);

        sbCellWidth = new QSpinBox(gbEnabled);
        sbCellWidth->setObjectName("sbCellWidth");
        sbCellWidth->setMinimum(0);
        sbCellWidth->setMaximum(9999);

        gridLayout_2->addWidget(sbCellWidth, 2, 1, 1, 1);

        lCellHeight = new QLabel(gbEnabled);
        lCellHeight->setObjectName("lCellHeight");

        gridLayout_2->addWidget(lCellHeight, 3, 0, 1, 1);

        sbCellHeight = new QSpinBox(gbEnabled);
        sbCellHeight->setObjectName("sbCellHeight");
        sbCellHeight->setMinimum(0);
        sbCellHeight->setMaximum(9999);

        gridLayout_2->addWidget(sbCellHeight, 3, 1, 1, 1);


        verticalLayout->addWidget(gbEnabled);


        retranslateUi(GammaRay__GridSettingsWidget);

        QMetaObject::connectSlotsByName(GammaRay__GridSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__GridSettingsWidget)
    {
        gbEnabled->setTitle(QCoreApplication::translate("GammaRay::GridSettingsWidget", "Grid enabled", nullptr));
        lXOffset->setText(QCoreApplication::translate("GammaRay::GridSettingsWidget", "X Offset:", nullptr));
        lYOffset->setText(QCoreApplication::translate("GammaRay::GridSettingsWidget", "Y Offset:", nullptr));
        lCellWidth->setText(QCoreApplication::translate("GammaRay::GridSettingsWidget", "Cell Width:", nullptr));
        lCellHeight->setText(QCoreApplication::translate("GammaRay::GridSettingsWidget", "Cell Height:", nullptr));
        (void)GammaRay__GridSettingsWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class GridSettingsWidget: public Ui_GridSettingsWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_GRIDSETTINGSWIDGET_H
