/********************************************************************************
** Form generated from reading UI file 'qmltypetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QMLTYPETAB_H
#define UI_QMLTYPETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_QmlTypeTab
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *typeView;

    void setupUi(QWidget *GammaRay__QmlTypeTab)
    {
        if (GammaRay__QmlTypeTab->objectName().isEmpty())
            GammaRay__QmlTypeTab->setObjectName("GammaRay__QmlTypeTab");
        GammaRay__QmlTypeTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__QmlTypeTab);
        verticalLayout->setObjectName("verticalLayout");
        typeView = new GammaRay::DeferredTreeView(GammaRay__QmlTypeTab);
        typeView->setObjectName("typeView");
        typeView->setContextMenuPolicy(Qt::CustomContextMenu);

        verticalLayout->addWidget(typeView);


        retranslateUi(GammaRay__QmlTypeTab);

        QMetaObject::connectSlotsByName(GammaRay__QmlTypeTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__QmlTypeTab)
    {
        (void)GammaRay__QmlTypeTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class QmlTypeTab: public Ui_QmlTypeTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_QMLTYPETAB_H
