/********************************************************************************
** Form generated from reading UI file 'localeinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOCALEINSPECTORWIDGET_H
#define UI_LOCALEINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "localetab.h"
#include "timezonetab.h"

namespace GammaRay {

class Ui_LocaleInspectorWidget
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    GammaRay::LocaleTab *localeTab;
    GammaRay::TimezoneTab *timezoneTab;

    void setupUi(QWidget *GammaRay__LocaleInspectorWidget)
    {
        if (GammaRay__LocaleInspectorWidget->objectName().isEmpty())
            GammaRay__LocaleInspectorWidget->setObjectName("GammaRay__LocaleInspectorWidget");
        GammaRay__LocaleInspectorWidget->resize(654, 506);
        verticalLayout = new QVBoxLayout(GammaRay__LocaleInspectorWidget);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(GammaRay__LocaleInspectorWidget);
        tabWidget->setObjectName("tabWidget");
        localeTab = new GammaRay::LocaleTab();
        localeTab->setObjectName("localeTab");
        tabWidget->addTab(localeTab, QString());
        timezoneTab = new GammaRay::TimezoneTab();
        timezoneTab->setObjectName("timezoneTab");
        tabWidget->addTab(timezoneTab, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(GammaRay__LocaleInspectorWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__LocaleInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__LocaleInspectorWidget)
    {
        tabWidget->setTabText(tabWidget->indexOf(localeTab), QCoreApplication::translate("GammaRay::LocaleInspectorWidget", "Locales", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(timezoneTab), QCoreApplication::translate("GammaRay::LocaleInspectorWidget", "Timezones", nullptr));
        (void)GammaRay__LocaleInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class LocaleInspectorWidget: public Ui_LocaleInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_LOCALEINSPECTORWIDGET_H
