/****************************************************************************
** Meta object code from reading C++ file 'statemachineviewerserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/statemachineviewer/statemachineviewerserver.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'statemachineviewerserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::StateMachineViewerServer::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::StateMachineViewerServer",
        "stateEntered",
        "",
        "GammaRay::State",
        "state",
        "stateExited",
        "stateConfigurationChanged",
        "handleTransitionTriggered",
        "GammaRay::Transition",
        "transition",
        "stateSelectionChanged",
        "setFilteredStates",
        "QList<GammaRay::State>",
        "states",
        "selectStateMachine",
        "row",
        "setSelectedStateMachine",
        "GammaRay::StateMachineDebugInterface*",
        "machine",
        "updateStartStop",
        "toggleRunning",
        "repopulateGraph",
        "handleLogMessage",
        "label",
        "msg",
        "objectSelected",
        "obj"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'stateEntered'
        QtMocHelpers::SlotData<void(GammaRay::State)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'stateExited'
        QtMocHelpers::SlotData<void(GammaRay::State)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'stateConfigurationChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleTransitionTriggered'
        QtMocHelpers::SlotData<void(GammaRay::Transition)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'stateSelectionChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFilteredStates'
        QtMocHelpers::SlotData<void(const QVector<GammaRay::State> &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'selectStateMachine'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'setSelectedStateMachine'
        QtMocHelpers::SlotData<void(GammaRay::StateMachineDebugInterface *)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'updateStartStop'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleRunning'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'repopulateGraph'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleLogMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 },
        }}),
        // Slot 'objectSelected'
        QtMocHelpers::SlotData<void(QObject *)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateMachineViewerServer, qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::StateMachineViewerServer::staticMetaObject = { {
    QMetaObject::SuperData::link<StateMachineViewerInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t>.metaTypes,
    nullptr
} };

void GammaRay::StateMachineViewerServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateMachineViewerServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateEntered((*reinterpret_cast<std::add_pointer_t<GammaRay::State>>(_a[1]))); break;
        case 1: _t->stateExited((*reinterpret_cast<std::add_pointer_t<GammaRay::State>>(_a[1]))); break;
        case 2: _t->stateConfigurationChanged(); break;
        case 3: _t->handleTransitionTriggered((*reinterpret_cast<std::add_pointer_t<GammaRay::Transition>>(_a[1]))); break;
        case 4: _t->stateSelectionChanged(); break;
        case 5: _t->setFilteredStates((*reinterpret_cast<std::add_pointer_t<QList<GammaRay::State>>>(_a[1]))); break;
        case 6: _t->selectStateMachine((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setSelectedStateMachine((*reinterpret_cast<std::add_pointer_t<GammaRay::StateMachineDebugInterface*>>(_a[1]))); break;
        case 8: _t->updateStartStop(); break;
        case 9: _t->toggleRunning(); break;
        case 10: _t->repopulateGraph(); break;
        case 11: _t->handleLogMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->objectSelected((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::State >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::State >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::Transition >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<GammaRay::State> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateMachineDebugInterface* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *GammaRay::StateMachineViewerServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::StateMachineViewerServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24StateMachineViewerServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.StateMachineViewer"))
        return static_cast< GammaRay::StateMachineViewerInterface*>(this);
    return StateMachineViewerInterface::qt_metacast(_clname);
}

int GammaRay::StateMachineViewerServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StateMachineViewerInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::StateMachineViewerFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::StateMachineViewerFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateMachineViewerFactory, qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::StateMachineViewerFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::StateMachineViewerFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateMachineViewerFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::StateMachineViewerFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::StateMachineViewerFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay25StateMachineViewerFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QStateMachine,StateMachineViewerServer>"))
        return static_cast< StandardToolFactory<QStateMachine,StateMachineViewerServer>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::StateMachineViewerFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_StateMachineViewerFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x78,  0x19,  'S',  't',  'a',  't',  'e', 
    'M',  'a',  'c',  'h',  'i',  'n',  'e',  'V', 
    'i',  'e',  'w',  'e',  'r',  'F',  'a',  'c', 
    't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x78,  0x1b,  'g', 
    'a',  'm',  'm',  'a',  'r',  'a',  'y',  '_', 
    's',  't',  'a',  't',  'e',  'm',  'a',  'c', 
    'h',  'i',  'n',  'e',  'v',  'i',  'e',  'w', 
    'e',  'r',  0x64,  'n',  'a',  'm',  'e',  0x6e, 
    'S',  't',  'a',  't',  'e',  ' ',  'M',  'a', 
    'c',  'h',  'i',  'n',  'e',  's',  0x68,  'n', 
    'a',  'm',  'e',  '[',  'd',  'e',  ']',  0x71, 
    'Z',  'u',  's',  't',  'a',  'n',  'd',  's', 
    'a',  'u',  't',  'o',  'm',  'a',  't',  'e', 
    'n',  0x6f,  's',  'e',  'l',  'e',  'c',  't', 
    'a',  'b',  'l',  'e',  'T',  'y',  'p',  'e', 
    's',  0x81,  0x6e,  'Q',  'A',  'b',  's',  't', 
    'r',  'a',  'c',  't',  'S',  't',  'a',  't', 
    'e',  0x65,  't',  'y',  'p',  'e',  's',  0x82, 
    0x6e,  'Q',  'A',  'b',  's',  't',  'r',  'a', 
    'c',  't',  'S',  't',  'a',  't',  'e',  0x72, 
    'Q',  'S',  'c',  'x',  'm',  'l',  'S',  't', 
    'a',  't',  'e',  'M',  'a',  'c',  'h',  'i', 
    'n',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::StateMachineViewerFactory, StateMachineViewerFactory, qt_pluginMetaDataV2_StateMachineViewerFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_StateMachineViewerFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x78,  0x19,  'S',  't',  'a',  't',  'e', 
    'M',  'a',  'c',  'h',  'i',  'n',  'e',  'V', 
    'i',  'e',  'w',  'e',  'r',  'F',  'a',  'c', 
    't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x78,  0x1b,  'g', 
    'a',  'm',  'm',  'a',  'r',  'a',  'y',  '_', 
    's',  't',  'a',  't',  'e',  'm',  'a',  'c', 
    'h',  'i',  'n',  'e',  'v',  'i',  'e',  'w', 
    'e',  'r',  0x64,  'n',  'a',  'm',  'e',  0x6e, 
    'S',  't',  'a',  't',  'e',  ' ',  'M',  'a', 
    'c',  'h',  'i',  'n',  'e',  's',  0x68,  'n', 
    'a',  'm',  'e',  '[',  'd',  'e',  ']',  0x71, 
    'Z',  'u',  's',  't',  'a',  'n',  'd',  's', 
    'a',  'u',  't',  'o',  'm',  'a',  't',  'e', 
    'n',  0x6f,  's',  'e',  'l',  'e',  'c',  't', 
    'a',  'b',  'l',  'e',  'T',  'y',  'p',  'e', 
    's',  0x81,  0x6e,  'Q',  'A',  'b',  's',  't', 
    'r',  'a',  'c',  't',  'S',  't',  'a',  't', 
    'e',  0x65,  't',  'y',  'p',  'e',  's',  0x82, 
    0x6e,  'Q',  'A',  'b',  's',  't',  'r',  'a', 
    'c',  't',  'S',  't',  'a',  't',  'e',  0x72, 
    'Q',  'S',  'c',  'x',  'm',  'l',  'S',  't', 
    'a',  't',  'e',  'M',  'a',  'c',  'h',  'i', 
    'n',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::StateMachineViewerFactory, StateMachineViewerFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
