/********************************************************************************
** Form generated from reading UI file 'bindingtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BINDINGTAB_H
#define UI_BINDINGTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_BindingTab
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *bindingView;

    void setupUi(QWidget *GammaRay__BindingTab)
    {
        if (GammaRay__BindingTab->objectName().isEmpty())
            GammaRay__BindingTab->setObjectName("GammaRay__BindingTab");
        GammaRay__BindingTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__BindingTab);
        verticalLayout->setObjectName("verticalLayout");
        bindingView = new GammaRay::DeferredTreeView(GammaRay__BindingTab);
        bindingView->setObjectName("bindingView");
        bindingView->setContextMenuPolicy(Qt::CustomContextMenu);
        bindingView->setTextElideMode(Qt::ElideLeft);
        bindingView->setUniformRowHeights(true);

        verticalLayout->addWidget(bindingView);


        retranslateUi(GammaRay__BindingTab);

        QMetaObject::connectSlotsByName(GammaRay__BindingTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__BindingTab)
    {
        (void)GammaRay__BindingTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class BindingTab: public Ui_BindingTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_BINDINGTAB_H
