/********************************************************************************
** Form generated from reading UI file 'eventmonitorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EVENTMONITORWIDGET_H
#define UI_EVENTMONITORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_EventMonitorWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tab;
    QHBoxLayout *horizontalLayout;
    QSplitter *mainSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *toolbarLayout;
    QLineEdit *eventSearchLine;
    QToolButton *pauseButton;
    QToolButton *clearButton;
    GammaRay::DeferredTreeView *eventTree;
    QWidget *layoutWidget_2;
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *eventInspector;
    QWidget *tab_2;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *toolbarLayout_2;
    QLineEdit *typeSearchLine;
    QToolButton *recordAllButton;
    QToolButton *recordNoneButton;
    QSpacerItem *horizontalSpacer2;
    QToolButton *showAllButton;
    QToolButton *showNoneButton;
    GammaRay::DeferredTreeView *eventTypeTree;

    void setupUi(QWidget *GammaRay__EventMonitorWidget)
    {
        if (GammaRay__EventMonitorWidget->objectName().isEmpty())
            GammaRay__EventMonitorWidget->setObjectName("GammaRay__EventMonitorWidget");
        GammaRay__EventMonitorWidget->resize(689, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(GammaRay__EventMonitorWidget->sizePolicy().hasHeightForWidth());
        GammaRay__EventMonitorWidget->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(GammaRay__EventMonitorWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        tabWidget = new QTabWidget(GammaRay__EventMonitorWidget);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        horizontalLayout = new QHBoxLayout(tab);
        horizontalLayout->setObjectName("horizontalLayout");
        mainSplitter = new QSplitter(tab);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(mainSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout_4 = new QVBoxLayout(layoutWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        toolbarLayout = new QHBoxLayout();
        toolbarLayout->setObjectName("toolbarLayout");
        toolbarLayout->setContentsMargins(-1, -1, -1, 6);
        eventSearchLine = new QLineEdit(layoutWidget);
        eventSearchLine->setObjectName("eventSearchLine");

        toolbarLayout->addWidget(eventSearchLine);

        pauseButton = new QToolButton(layoutWidget);
        pauseButton->setObjectName("pauseButton");
        pauseButton->setCheckable(true);

        toolbarLayout->addWidget(pauseButton);

        clearButton = new QToolButton(layoutWidget);
        clearButton->setObjectName("clearButton");

        toolbarLayout->addWidget(clearButton);


        verticalLayout_4->addLayout(toolbarLayout);

        eventTree = new GammaRay::DeferredTreeView(layoutWidget);
        eventTree->setObjectName("eventTree");
        eventTree->setContextMenuPolicy(Qt::CustomContextMenu);
        eventTree->setUniformRowHeights(true);

        verticalLayout_4->addWidget(eventTree);

        mainSplitter->addWidget(layoutWidget);
        layoutWidget_2 = new QWidget(mainSplitter);
        layoutWidget_2->setObjectName("layoutWidget_2");
        verticalLayout = new QVBoxLayout(layoutWidget_2);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        eventInspector = new GammaRay::DeferredTreeView(layoutWidget_2);
        eventInspector->setObjectName("eventInspector");
        eventInspector->setContextMenuPolicy(Qt::CustomContextMenu);
        eventInspector->setSelectionMode(QAbstractItemView::ContiguousSelection);

        verticalLayout->addWidget(eventInspector);

        mainSplitter->addWidget(layoutWidget_2);

        horizontalLayout->addWidget(mainSplitter);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        horizontalLayout_2 = new QHBoxLayout(tab_2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        toolbarLayout_2 = new QHBoxLayout();
        toolbarLayout_2->setObjectName("toolbarLayout_2");
        toolbarLayout_2->setContentsMargins(-1, -1, -1, 6);
        typeSearchLine = new QLineEdit(tab_2);
        typeSearchLine->setObjectName("typeSearchLine");

        toolbarLayout_2->addWidget(typeSearchLine);

        recordAllButton = new QToolButton(tab_2);
        recordAllButton->setObjectName("recordAllButton");

        toolbarLayout_2->addWidget(recordAllButton);

        recordNoneButton = new QToolButton(tab_2);
        recordNoneButton->setObjectName("recordNoneButton");

        toolbarLayout_2->addWidget(recordNoneButton);

        horizontalSpacer2 = new QSpacerItem(50, 0, QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Minimum);

        toolbarLayout_2->addItem(horizontalSpacer2);

        showAllButton = new QToolButton(tab_2);
        showAllButton->setObjectName("showAllButton");

        toolbarLayout_2->addWidget(showAllButton);

        showNoneButton = new QToolButton(tab_2);
        showNoneButton->setObjectName("showNoneButton");

        toolbarLayout_2->addWidget(showNoneButton);


        verticalLayout_2->addLayout(toolbarLayout_2);

        eventTypeTree = new GammaRay::DeferredTreeView(tab_2);
        eventTypeTree->setObjectName("eventTypeTree");
        eventTypeTree->setRootIsDecorated(false);
        eventTypeTree->setUniformRowHeights(true);

        verticalLayout_2->addWidget(eventTypeTree);


        horizontalLayout_2->addLayout(verticalLayout_2);

        tabWidget->addTab(tab_2, QString());

        verticalLayout_3->addWidget(tabWidget);


        retranslateUi(GammaRay__EventMonitorWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__EventMonitorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__EventMonitorWidget)
    {
        pauseButton->setText(QCoreApplication::translate("GammaRay::EventMonitorWidget", "Pause", nullptr));
        clearButton->setText(QCoreApplication::translate("GammaRay::EventMonitorWidget", "Clear History", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("GammaRay::EventMonitorWidget", "Log", nullptr));
        recordAllButton->setText(QCoreApplication::translate("GammaRay::EventMonitorWidget", "Record All", nullptr));
        recordNoneButton->setText(QCoreApplication::translate("GammaRay::EventMonitorWidget", "Record None", nullptr));
        showAllButton->setText(QCoreApplication::translate("GammaRay::EventMonitorWidget", "Show All", nullptr));
        showNoneButton->setText(QCoreApplication::translate("GammaRay::EventMonitorWidget", "Show None", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("GammaRay::EventMonitorWidget", "Types", nullptr));
        (void)GammaRay__EventMonitorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class EventMonitorWidget: public Ui_EventMonitorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_EVENTMONITORWIDGET_H
