/********************************************************************************
** Form generated from reading UI file 'networkwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NETWORKWIDGET_H
#define UI_NETWORKWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "networkconfigurationwidget.h"
#include "networkinterfacewidget.h"
#include "networkreplywidget.h"

namespace GammaRay {

class Ui_NetworkWidget
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *interfaceTab;
    QVBoxLayout *verticalLayout_2;
    GammaRay::NetworkInterfaceWidget *widget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_3;
    GammaRay::NetworkConfigurationWidget *configTab;
    QWidget *replyTab;
    QVBoxLayout *verticalLayout_4;
    QSplitter *splitter;
    GammaRay::NetworkReplyWidget *widget_2;

    void setupUi(QWidget *GammaRay__NetworkWidget)
    {
        if (GammaRay__NetworkWidget->objectName().isEmpty())
            GammaRay__NetworkWidget->setObjectName("GammaRay__NetworkWidget");
        GammaRay__NetworkWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__NetworkWidget);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(GammaRay__NetworkWidget);
        tabWidget->setObjectName("tabWidget");
        interfaceTab = new QWidget();
        interfaceTab->setObjectName("interfaceTab");
        verticalLayout_2 = new QVBoxLayout(interfaceTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        widget = new GammaRay::NetworkInterfaceWidget(interfaceTab);
        widget->setObjectName("widget");

        verticalLayout_2->addWidget(widget);

        tabWidget->addTab(interfaceTab, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_3 = new QVBoxLayout(tab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        configTab = new GammaRay::NetworkConfigurationWidget(tab);
        configTab->setObjectName("configTab");

        verticalLayout_3->addWidget(configTab);

        tabWidget->addTab(tab, QString());
        replyTab = new QWidget();
        replyTab->setObjectName("replyTab");
        verticalLayout_4 = new QVBoxLayout(replyTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        splitter = new QSplitter(replyTab);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        widget_2 = new GammaRay::NetworkReplyWidget(splitter);
        widget_2->setObjectName("widget_2");
        splitter->addWidget(widget_2);

        verticalLayout_4->addWidget(splitter);

        tabWidget->addTab(replyTab, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(GammaRay__NetworkWidget);

        tabWidget->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(GammaRay__NetworkWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__NetworkWidget)
    {
        tabWidget->setTabText(tabWidget->indexOf(interfaceTab), QCoreApplication::translate("GammaRay::NetworkWidget", "Interfaces", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("GammaRay::NetworkWidget", "Configurations", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(replyTab), QCoreApplication::translate("GammaRay::NetworkWidget", "Operations", nullptr));
        (void)GammaRay__NetworkWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class NetworkWidget: public Ui_NetworkWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_NETWORKWIDGET_H
