/****************************************************************************
** Meta object code from reading C++ file 'positioninginterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/positioning/positioninginterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'positioninginterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::PositioningInterface::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::PositioningInterface",
        "positionInfoChanged",
        "",
        "positioningOverrideAvailableChanged",
        "positioningOverrideEnabledChanged",
        "positionInfoOverrideChanged",
        "setPositionInfo",
        "QGeoPositionInfo",
        "info",
        "positionInfo",
        "positioningOverrideAvailable",
        "positioningOverrideEnabled",
        "positionInfoOverride"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'positionInfoChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positioningOverrideAvailableChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positioningOverrideEnabledChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionInfoOverrideChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPositionInfo'
        QtMocHelpers::SlotData<void(const QGeoPositionInfo &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'positionInfo'
        QtMocHelpers::PropertyData<QGeoPositionInfo>(9, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'positioningOverrideAvailable'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'positioningOverrideEnabled'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'positionInfoOverride'
        QtMocHelpers::PropertyData<QGeoPositionInfo>(12, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PositioningInterface, qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::PositioningInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t>.metaTypes,
    nullptr
} };

void GammaRay::PositioningInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PositioningInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->positionInfoChanged(); break;
        case 1: _t->positioningOverrideAvailableChanged(); break;
        case 2: _t->positioningOverrideEnabledChanged(); break;
        case 3: _t->positionInfoOverrideChanged(); break;
        case 4: _t->setPositionInfo((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfo>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoPositionInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PositioningInterface::*)()>(_a, &PositioningInterface::positionInfoChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PositioningInterface::*)()>(_a, &PositioningInterface::positioningOverrideAvailableChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PositioningInterface::*)()>(_a, &PositioningInterface::positioningOverrideEnabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PositioningInterface::*)()>(_a, &PositioningInterface::positionInfoOverrideChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoPositionInfo >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoPositionInfo*>(_v) = _t->positionInfo(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->positioningOverrideAvailable(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->positioningOverrideEnabled(); break;
        case 3: *reinterpret_cast<QGeoPositionInfo*>(_v) = _t->positionInfoOverride(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPositionInfo(*reinterpret_cast<QGeoPositionInfo*>(_v)); break;
        case 1: _t->setPositioningOverrideAvailable(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setPositioningOverrideEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setPositionInfoOverride(*reinterpret_cast<QGeoPositionInfo*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *GammaRay::PositioningInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::PositioningInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay20PositioningInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::PositioningInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::PositioningInterface::positionInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::PositioningInterface::positioningOverrideAvailableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GammaRay::PositioningInterface::positioningOverrideEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GammaRay::PositioningInterface::positionInfoOverrideChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
