/********************************************************************************
** Form generated from reading UI file 'styleelementstatetablepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STYLEELEMENTSTATETABLEPAGE_H
#define UI_STYLEELEMENTSTATETABLEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_StyleElementStateTablePage
{
public:
    QVBoxLayout *verticalLayout;
    QTableView *tableView;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *widthBox;
    QLabel *label_2;
    QSpinBox *heightBox;
    QLabel *label_3;
    QSlider *zoomSlider;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *GammaRay__StyleElementStateTablePage)
    {
        if (GammaRay__StyleElementStateTablePage->objectName().isEmpty())
            GammaRay__StyleElementStateTablePage->setObjectName("GammaRay__StyleElementStateTablePage");
        GammaRay__StyleElementStateTablePage->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__StyleElementStateTablePage);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        tableView = new QTableView(GammaRay__StyleElementStateTablePage);
        tableView->setObjectName("tableView");
        tableView->setSelectionMode(QAbstractItemView::NoSelection);
        tableView->setShowGrid(false);

        verticalLayout->addWidget(tableView);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(GammaRay__StyleElementStateTablePage);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        widthBox = new QSpinBox(GammaRay__StyleElementStateTablePage);
        widthBox->setObjectName("widthBox");
        widthBox->setMaximum(999);

        horizontalLayout->addWidget(widthBox);

        label_2 = new QLabel(GammaRay__StyleElementStateTablePage);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        heightBox = new QSpinBox(GammaRay__StyleElementStateTablePage);
        heightBox->setObjectName("heightBox");
        heightBox->setMaximum(999);

        horizontalLayout->addWidget(heightBox);

        label_3 = new QLabel(GammaRay__StyleElementStateTablePage);
        label_3->setObjectName("label_3");

        horizontalLayout->addWidget(label_3);

        zoomSlider = new QSlider(GammaRay__StyleElementStateTablePage);
        zoomSlider->setObjectName("zoomSlider");
        zoomSlider->setMinimum(1);
        zoomSlider->setMaximum(8);
        zoomSlider->setPageStep(1);
        zoomSlider->setOrientation(Qt::Horizontal);

        horizontalLayout->addWidget(zoomSlider);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

#if QT_CONFIG(shortcut)
        label->setBuddy(widthBox);
        label_2->setBuddy(heightBox);
        label_3->setBuddy(zoomSlider);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__StyleElementStateTablePage);

        QMetaObject::connectSlotsByName(GammaRay__StyleElementStateTablePage);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__StyleElementStateTablePage)
    {
        label->setText(QCoreApplication::translate("GammaRay::StyleElementStateTablePage", "Cell &Width:", nullptr));
        widthBox->setSuffix(QCoreApplication::translate("GammaRay::StyleElementStateTablePage", "px", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::StyleElementStateTablePage", "Cell &Height:", nullptr));
        heightBox->setSuffix(QCoreApplication::translate("GammaRay::StyleElementStateTablePage", "px", nullptr));
        label_3->setText(QCoreApplication::translate("GammaRay::StyleElementStateTablePage", "&Zoom:", nullptr));
        (void)GammaRay__StyleElementStateTablePage;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class StyleElementStateTablePage: public Ui_StyleElementStateTablePage {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_STYLEELEMENTSTATETABLEPAGE_H
