/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QWidget>
#include "sidepane.h"

namespace GammaRay {

class Ui_MainWindow
{
public:
    QAction *actionRetractProbe;
    QAction *actionQuit;
    QAction *actionAboutGammaRay;
    QAction *actionAboutQt;
    QAction *actionAboutKDAB;
    QAction *actionPlugins;
    QAction *actionMessageStatistics;
    QAction *actionHelp;
    QAction *actionContribute;
    QWidget *centralwidget;
    QHBoxLayout *horizontalLayout;
    SidePane *toolSelector;
    QStackedWidget *toolStack;
    QMenuBar *menuBar;
    QMenu *menu_GammaRay;
    QMenu *menu_Help;
    QMenu *menu_Diagnostics;
    QMenu *actionsMenu;
    QMenu *menuSettings;
    QToolBar *mainToolBar;
    QStatusBar *statusBar;

    void setupUi(QMainWindow *GammaRay__MainWindow)
    {
        if (GammaRay__MainWindow->objectName().isEmpty())
            GammaRay__MainWindow->setObjectName("GammaRay__MainWindow");
        GammaRay__MainWindow->resize(829, 600);
        actionRetractProbe = new QAction(GammaRay__MainWindow);
        actionRetractProbe->setObjectName("actionRetractProbe");
        actionRetractProbe->setMenuRole(QAction::ApplicationSpecificRole);
        actionQuit = new QAction(GammaRay__MainWindow);
        actionQuit->setObjectName("actionQuit");
        actionQuit->setMenuRole(QAction::QuitRole);
        actionAboutGammaRay = new QAction(GammaRay__MainWindow);
        actionAboutGammaRay->setObjectName("actionAboutGammaRay");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/gammaray/GammaRay-32x32.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionAboutGammaRay->setIcon(icon);
        actionAboutGammaRay->setMenuRole(QAction::AboutRole);
        actionAboutQt = new QAction(GammaRay__MainWindow);
        actionAboutQt->setObjectName("actionAboutQt");
        actionAboutQt->setMenuRole(QAction::AboutQtRole);
        actionAboutKDAB = new QAction(GammaRay__MainWindow);
        actionAboutKDAB->setObjectName("actionAboutKDAB");
        actionAboutKDAB->setMenuRole(QAction::NoRole);
        actionPlugins = new QAction(GammaRay__MainWindow);
        actionPlugins->setObjectName("actionPlugins");
        actionMessageStatistics = new QAction(GammaRay__MainWindow);
        actionMessageStatistics->setObjectName("actionMessageStatistics");
        actionHelp = new QAction(GammaRay__MainWindow);
        actionHelp->setObjectName("actionHelp");
        actionContribute = new QAction(GammaRay__MainWindow);
        actionContribute->setObjectName("actionContribute");
        actionContribute->setEnabled(false);
        centralwidget = new QWidget(GammaRay__MainWindow);
        centralwidget->setObjectName("centralwidget");
        horizontalLayout = new QHBoxLayout(centralwidget);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        toolSelector = new SidePane(centralwidget);
        toolSelector->setObjectName("toolSelector");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(toolSelector->sizePolicy().hasHeightForWidth());
        toolSelector->setSizePolicy(sizePolicy);
        toolSelector->setMinimumSize(QSize(180, 0));
        toolSelector->setContextMenuPolicy(Qt::CustomContextMenu);
        toolSelector->setFrameShape(QFrame::NoFrame);
        toolSelector->setFrameShadow(QFrame::Plain);
        toolSelector->setTextElideMode(Qt::ElideRight);
        toolSelector->setFlow(QListView::TopToBottom);
        toolSelector->setProperty("isWrapping", QVariant(false));
        toolSelector->setResizeMode(QListView::Fixed);
        toolSelector->setViewMode(QListView::ListMode);
        toolSelector->setUniformItemSizes(true);
        toolSelector->setWordWrap(false);
        toolSelector->setSelectionRectVisible(true);

        horizontalLayout->addWidget(toolSelector);

        toolStack = new QStackedWidget(centralwidget);
        toolStack->setObjectName("toolStack");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(toolStack->sizePolicy().hasHeightForWidth());
        toolStack->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(toolStack);

        GammaRay__MainWindow->setCentralWidget(centralwidget);
        menuBar = new QMenuBar(GammaRay__MainWindow);
        menuBar->setObjectName("menuBar");
        menuBar->setGeometry(QRect(0, 0, 829, 30));
        menu_GammaRay = new QMenu(menuBar);
        menu_GammaRay->setObjectName("menu_GammaRay");
        menu_Help = new QMenu(menuBar);
        menu_Help->setObjectName("menu_Help");
        menu_Diagnostics = new QMenu(menu_Help);
        menu_Diagnostics->setObjectName("menu_Diagnostics");
        actionsMenu = new QMenu(menuBar);
        actionsMenu->setObjectName("actionsMenu");
        menuSettings = new QMenu(menuBar);
        menuSettings->setObjectName("menuSettings");
        GammaRay__MainWindow->setMenuBar(menuBar);
        mainToolBar = new QToolBar(GammaRay__MainWindow);
        mainToolBar->setObjectName("mainToolBar");
        mainToolBar->setMovable(false);
        mainToolBar->setAllowedAreas(Qt::BottomToolBarArea|Qt::TopToolBarArea);
        mainToolBar->setFloatable(false);
        GammaRay__MainWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, mainToolBar);
        statusBar = new QStatusBar(GammaRay__MainWindow);
        statusBar->setObjectName("statusBar");
        GammaRay__MainWindow->setStatusBar(statusBar);

        menuBar->addAction(menu_GammaRay->menuAction());
        menuBar->addAction(actionsMenu->menuAction());
        menuBar->addAction(menuSettings->menuAction());
        menuBar->addAction(menu_Help->menuAction());
        menu_GammaRay->addAction(actionRetractProbe);
        menu_GammaRay->addSeparator();
        menu_GammaRay->addAction(actionQuit);
        menu_Help->addAction(actionHelp);
        menu_Help->addAction(actionContribute);
        menu_Help->addSeparator();
        menu_Help->addAction(menu_Diagnostics->menuAction());
        menu_Help->addSeparator();
        menu_Help->addAction(actionAboutGammaRay);
        menu_Help->addAction(actionAboutKDAB);
        menu_Help->addAction(actionAboutQt);
        menu_Diagnostics->addAction(actionPlugins);
        menu_Diagnostics->addAction(actionMessageStatistics);

        retranslateUi(GammaRay__MainWindow);

        QMetaObject::connectSlotsByName(GammaRay__MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *GammaRay__MainWindow)
    {
        actionRetractProbe->setText(QCoreApplication::translate("GammaRay::MainWindow", "&Detach", nullptr));
#if QT_CONFIG(tooltip)
        actionRetractProbe->setToolTip(QCoreApplication::translate("GammaRay::MainWindow", "Retract probe from application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionQuit->setText(QCoreApplication::translate("GammaRay::MainWindow", "&Quit", nullptr));
#if QT_CONFIG(tooltip)
        actionQuit->setToolTip(QCoreApplication::translate("GammaRay::MainWindow", "Quit probed application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAboutGammaRay->setText(QCoreApplication::translate("GammaRay::MainWindow", "About &GammaRay", nullptr));
        actionAboutQt->setText(QCoreApplication::translate("GammaRay::MainWindow", "About &Qt", nullptr));
        actionAboutKDAB->setText(QCoreApplication::translate("GammaRay::MainWindow", "About &KDAB", nullptr));
        actionPlugins->setText(QCoreApplication::translate("GammaRay::MainWindow", "&Plugins...", nullptr));
        actionMessageStatistics->setText(QCoreApplication::translate("GammaRay::MainWindow", "&Message Statistics...", nullptr));
#if QT_CONFIG(tooltip)
        actionMessageStatistics->setToolTip(QCoreApplication::translate("GammaRay::MainWindow", "Show GammaRay communication statistics.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionHelp->setText(QCoreApplication::translate("GammaRay::MainWindow", "&Help...", nullptr));
#if QT_CONFIG(tooltip)
        actionHelp->setToolTip(QCoreApplication::translate("GammaRay::MainWindow", "Show help browser.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionContribute->setText(QCoreApplication::translate("GammaRay::MainWindow", "&Contribute...", nullptr));
        menu_GammaRay->setTitle(QCoreApplication::translate("GammaRay::MainWindow", "&GammaRay", nullptr));
        menu_Help->setTitle(QCoreApplication::translate("GammaRay::MainWindow", "&Help", nullptr));
        menu_Diagnostics->setTitle(QCoreApplication::translate("GammaRay::MainWindow", "&Diagnostics", nullptr));
        actionsMenu->setTitle(QCoreApplication::translate("GammaRay::MainWindow", "&Actions", nullptr));
        menuSettings->setTitle(QCoreApplication::translate("GammaRay::MainWindow", "Setti&ngs", nullptr));
        mainToolBar->setWindowTitle(QCoreApplication::translate("GammaRay::MainWindow", "Main Toolbar", nullptr));
        (void)GammaRay__MainWindow;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_MAINWINDOW_H
