/****************************************************************************
** Meta object code from reading C++ file 'eventmonitor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/eventmonitor/eventmonitor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'eventmonitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::EventPropagationListener::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::EventPropagationListener"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EventPropagationListener, qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::EventPropagationListener::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t>.metaTypes,
    nullptr
} };

void GammaRay::EventPropagationListener::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EventPropagationListener *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::EventPropagationListener::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::EventPropagationListener::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24EventPropagationListenerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::EventPropagationListener::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay12EventMonitorE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::EventMonitor::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay12EventMonitorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::EventMonitor",
        "clearHistory",
        "",
        "recordAll",
        "recordNone",
        "showAll",
        "showNone",
        "addEvent",
        "GammaRay::EventData",
        "event",
        "eventSelected",
        "QItemSelection",
        "selection"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'clearHistory'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'recordAll'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'recordNone'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showAll'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showNone'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addEvent'
        QtMocHelpers::SlotData<void(const GammaRay::EventData &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'eventSelected'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EventMonitor, qt_meta_tag_ZN8GammaRay12EventMonitorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::EventMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<EventMonitorInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay12EventMonitorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay12EventMonitorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay12EventMonitorE_t>.metaTypes,
    nullptr
} };

void GammaRay::EventMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EventMonitor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearHistory(); break;
        case 1: _t->recordAll(); break;
        case 2: _t->recordNone(); break;
        case 3: _t->showAll(); break;
        case 4: _t->showNone(); break;
        case 5: _t->addEvent((*reinterpret_cast<std::add_pointer_t<GammaRay::EventData>>(_a[1]))); break;
        case 6: _t->eventSelected((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *GammaRay::EventMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::EventMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay12EventMonitorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.EventMonitorInterface"))
        return static_cast< GammaRay::EventMonitorInterface*>(this);
    return EventMonitorInterface::qt_metacast(_clname);
}

int GammaRay::EventMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EventMonitorInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::EventMonitorFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::EventMonitorFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EventMonitorFactory, qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::EventMonitorFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::EventMonitorFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EventMonitorFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::EventMonitorFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::EventMonitorFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay19EventMonitorFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QObject,EventMonitor>"))
        return static_cast< StandardToolFactory<QObject,EventMonitor>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::EventMonitorFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_EventMonitorFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x73,  'E',  'v',  'e',  'n',  't',  'M', 
    'o',  'n',  'i',  't',  'o',  'r',  'F',  'a', 
    'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa3,  0x62,  'i',  'd',  0x75,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'e', 
    'v',  'e',  'n',  't',  'm',  'o',  'n',  'i', 
    't',  'o',  'r',  0x64,  'n',  'a',  'm',  'e', 
    0x66,  'E',  'v',  'e',  'n',  't',  's',  0x65, 
    't',  'y',  'p',  'e',  's',  0x81,  0x67,  'Q', 
    'O',  'b',  'j',  'e',  'c',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::EventMonitorFactory, EventMonitorFactory, qt_pluginMetaDataV2_EventMonitorFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_EventMonitorFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x73,  'E',  'v',  'e',  'n',  't',  'M', 
    'o',  'n',  'i',  't',  'o',  'r',  'F',  'a', 
    'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa3,  0x62,  'i',  'd',  0x75,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'e', 
    'v',  'e',  'n',  't',  'm',  'o',  'n',  'i', 
    't',  'o',  'r',  0x64,  'n',  'a',  'm',  'e', 
    0x66,  'E',  'v',  'e',  'n',  't',  's',  0x65, 
    't',  'y',  'p',  'e',  's',  0x81,  0x67,  'Q', 
    'O',  'b',  'j',  'e',  'c',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::EventMonitorFactory, EventMonitorFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
