/********************************************************************************
** Form generated from reading UI file 'resourcebrowserwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESOURCEBROWSERWIDGET_H
#define UI_RESOURCEBROWSERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/codeeditor/codeeditor.h>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ResourceBrowserWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QSplitter *mainSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLineEdit *searchLine;
    GammaRay::DeferredTreeView *treeView;
    QStackedWidget *stackedWidget;
    QWidget *contentTextPage;
    QHBoxLayout *horizontalLayout_3;
    GammaRay::CodeEditor *textBrowser;
    QWidget *contentLabelPage;
    QVBoxLayout *verticalLayout_2;
    QScrollArea *scrollArea;
    QLabel *resourceLabel;

    void setupUi(QWidget *GammaRay__ResourceBrowserWidget)
    {
        if (GammaRay__ResourceBrowserWidget->objectName().isEmpty())
            GammaRay__ResourceBrowserWidget->setObjectName("GammaRay__ResourceBrowserWidget");
        GammaRay__ResourceBrowserWidget->resize(1036, 846);
        horizontalLayout = new QHBoxLayout(GammaRay__ResourceBrowserWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        mainSplitter = new QSplitter(GammaRay__ResourceBrowserWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(mainSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        searchLine = new QLineEdit(layoutWidget);
        searchLine->setObjectName("searchLine");

        verticalLayout->addWidget(searchLine);

        treeView = new GammaRay::DeferredTreeView(layoutWidget);
        treeView->setObjectName("treeView");

        verticalLayout->addWidget(treeView);

        mainSplitter->addWidget(layoutWidget);
        stackedWidget = new QStackedWidget(mainSplitter);
        stackedWidget->setObjectName("stackedWidget");
        contentTextPage = new QWidget();
        contentTextPage->setObjectName("contentTextPage");
        horizontalLayout_3 = new QHBoxLayout(contentTextPage);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        textBrowser = new GammaRay::CodeEditor(contentTextPage);
        textBrowser->setObjectName("textBrowser");
        textBrowser->setTabChangesFocus(true);
        textBrowser->setLineWrapMode(QPlainTextEdit::NoWrap);
        textBrowser->setReadOnly(true);
        textBrowser->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        horizontalLayout_3->addWidget(textBrowser);

        stackedWidget->addWidget(contentTextPage);
        contentLabelPage = new QWidget();
        contentLabelPage->setObjectName("contentLabelPage");
        verticalLayout_2 = new QVBoxLayout(contentLabelPage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(contentLabelPage);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollArea->setAlignment(Qt::AlignCenter);
        resourceLabel = new QLabel();
        resourceLabel->setObjectName("resourceLabel");
        resourceLabel->setGeometry(QRect(0, 0, 527, 846));
        resourceLabel->setAlignment(Qt::AlignCenter);
        scrollArea->setWidget(resourceLabel);

        verticalLayout_2->addWidget(scrollArea);

        stackedWidget->addWidget(contentLabelPage);
        mainSplitter->addWidget(stackedWidget);

        horizontalLayout->addWidget(mainSplitter);


        retranslateUi(GammaRay__ResourceBrowserWidget);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(GammaRay__ResourceBrowserWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ResourceBrowserWidget)
    {
        resourceLabel->setText(QString());
        (void)GammaRay__ResourceBrowserWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ResourceBrowserWidget: public Ui_ResourceBrowserWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_RESOURCEBROWSERWIDGET_H
