/****************************************************************************
** Meta object code from reading C++ file 'modelinspector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/modelinspector/modelinspector.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'modelinspector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay14ModelInspectorE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::ModelInspector::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay14ModelInspectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::ModelInspector",
        "modelSelected",
        "",
        "QItemSelection",
        "selected",
        "cellSelectionChanged",
        "selection",
        "selectionModelSelected",
        "objectSelected",
        "object",
        "objectCreated"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'modelSelected'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'cellSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Slot 'selectionModelSelected'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'objectSelected'
        QtMocHelpers::SlotData<void(QObject *)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
        // Slot 'objectCreated'
        QtMocHelpers::SlotData<void(QObject *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ModelInspector, qt_meta_tag_ZN8GammaRay14ModelInspectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::ModelInspector::staticMetaObject = { {
    QMetaObject::SuperData::link<ModelInspectorInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14ModelInspectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14ModelInspectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay14ModelInspectorE_t>.metaTypes,
    nullptr
} };

void GammaRay::ModelInspector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ModelInspector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modelSelected((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 1: _t->cellSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 2: _t->selectionModelSelected((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 3: _t->objectSelected((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 4: _t->objectCreated((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *GammaRay::ModelInspector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::ModelInspector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14ModelInspectorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ModelInspectorInterface"))
        return static_cast< GammaRay::ModelInspectorInterface*>(this);
    return ModelInspectorInterface::qt_metacast(_clname);
}

int GammaRay::ModelInspector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ModelInspectorInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::ModelInspectorFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::ModelInspectorFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ModelInspectorFactory, qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::ModelInspectorFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::ModelInspectorFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ModelInspectorFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::ModelInspectorFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::ModelInspectorFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21ModelInspectorFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QAbstractItemModel,ModelInspector>"))
        return static_cast< StandardToolFactory<QAbstractItemModel,ModelInspector>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::ModelInspectorFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_ModelInspectorFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x75,  'M',  'o',  'd',  'e',  'l',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x77,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'm', 
    'o',  'd',  'e',  'l',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x66,  'M',  'o',  'd',  'e',  'l', 
    's',  0x68,  'n',  'a',  'm',  'e',  '[',  'd', 
    'e',  ']',  0x67,  'M',  'o',  'd',  'e',  'l', 
    'l',  'e',  0x6f,  's',  'e',  'l',  'e',  'c', 
    't',  'a',  'b',  'l',  'e',  'T',  'y',  'p', 
    'e',  's',  0x82,  0x72,  'Q',  'A',  'b',  's', 
    't',  'r',  'a',  'c',  't',  'I',  't',  'e', 
    'm',  'M',  'o',  'd',  'e',  'l',  0x73,  'Q', 
    'I',  't',  'e',  'm',  'S',  'e',  'l',  'e', 
    'c',  't',  'i',  'o',  'n',  'M',  'o',  'd', 
    'e',  'l',  0x65,  't',  'y',  'p',  'e',  's', 
    0x82,  0x72,  'Q',  'A',  'b',  's',  't',  'r', 
    'a',  'c',  't',  'I',  't',  'e',  'm',  'M', 
    'o',  'd',  'e',  'l',  0x73,  'Q',  'I',  't', 
    'e',  'm',  'S',  'e',  'l',  'e',  'c',  't', 
    'i',  'o',  'n',  'M',  'o',  'd',  'e',  'l', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::ModelInspectorFactory, ModelInspectorFactory, qt_pluginMetaDataV2_ModelInspectorFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_ModelInspectorFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x75,  'M',  'o',  'd',  'e',  'l',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x77,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'm', 
    'o',  'd',  'e',  'l',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x66,  'M',  'o',  'd',  'e',  'l', 
    's',  0x68,  'n',  'a',  'm',  'e',  '[',  'd', 
    'e',  ']',  0x67,  'M',  'o',  'd',  'e',  'l', 
    'l',  'e',  0x6f,  's',  'e',  'l',  'e',  'c', 
    't',  'a',  'b',  'l',  'e',  'T',  'y',  'p', 
    'e',  's',  0x82,  0x72,  'Q',  'A',  'b',  's', 
    't',  'r',  'a',  'c',  't',  'I',  't',  'e', 
    'm',  'M',  'o',  'd',  'e',  'l',  0x73,  'Q', 
    'I',  't',  'e',  'm',  'S',  'e',  'l',  'e', 
    'c',  't',  'i',  'o',  'n',  'M',  'o',  'd', 
    'e',  'l',  0x65,  't',  'y',  'p',  'e',  's', 
    0x82,  0x72,  'Q',  'A',  'b',  's',  't',  'r', 
    'a',  'c',  't',  'I',  't',  'e',  'm',  'M', 
    'o',  'd',  'e',  'l',  0x73,  'Q',  'I',  't', 
    'e',  'm',  'S',  'e',  'l',  'e',  'c',  't', 
    'i',  'o',  'n',  'M',  'o',  'd',  'e',  'l', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::ModelInspectorFactory, ModelInspectorFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
