/********************************************************************************
** Form generated from reading UI file 'objectinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OBJECTINSPECTORWIDGET_H
#define UI_OBJECTINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "deferredtreeview.h"
#include "favoritesitemview.h"
#include "propertywidget.h"

namespace GammaRay {

class Ui_ObjectInspectorWidget
{
public:
    QVBoxLayout *verticalLayout_5;
    QSplitter *mainSplitter;
    QWidget *layoutWidget_1;
    QVBoxLayout *verticalLayout_2;
    QLineEdit *objectSearchLine;
    GammaRay::ObjectsFavoriteView *favoritesTreeView;
    GammaRay::DeferredTreeView *objectTreeView;
    GammaRay::PropertyWidget *objectPropertyWidget;

    void setupUi(QWidget *GammaRay__ObjectInspectorWidget)
    {
        if (GammaRay__ObjectInspectorWidget->objectName().isEmpty())
            GammaRay__ObjectInspectorWidget->setObjectName("GammaRay__ObjectInspectorWidget");
        GammaRay__ObjectInspectorWidget->resize(400, 300);
        verticalLayout_5 = new QVBoxLayout(GammaRay__ObjectInspectorWidget);
        verticalLayout_5->setObjectName("verticalLayout_5");
        mainSplitter = new QSplitter(GammaRay__ObjectInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        layoutWidget_1 = new QWidget(mainSplitter);
        layoutWidget_1->setObjectName("layoutWidget_1");
        verticalLayout_2 = new QVBoxLayout(layoutWidget_1);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        objectSearchLine = new QLineEdit(layoutWidget_1);
        objectSearchLine->setObjectName("objectSearchLine");

        verticalLayout_2->addWidget(objectSearchLine);

        favoritesTreeView = new GammaRay::ObjectsFavoriteView(layoutWidget_1);
        favoritesTreeView->setObjectName("favoritesTreeView");

        verticalLayout_2->addWidget(favoritesTreeView);

        objectTreeView = new GammaRay::DeferredTreeView(layoutWidget_1);
        objectTreeView->setObjectName("objectTreeView");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(objectTreeView->sizePolicy().hasHeightForWidth());
        objectTreeView->setSizePolicy(sizePolicy);
        objectTreeView->setUniformRowHeights(true);

        verticalLayout_2->addWidget(objectTreeView);

        mainSplitter->addWidget(layoutWidget_1);
        objectPropertyWidget = new GammaRay::PropertyWidget(mainSplitter);
        objectPropertyWidget->setObjectName("objectPropertyWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(objectPropertyWidget->sizePolicy().hasHeightForWidth());
        objectPropertyWidget->setSizePolicy(sizePolicy1);
        mainSplitter->addWidget(objectPropertyWidget);

        verticalLayout_5->addWidget(mainSplitter);


        retranslateUi(GammaRay__ObjectInspectorWidget);

        QMetaObject::connectSlotsByName(GammaRay__ObjectInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ObjectInspectorWidget)
    {
        (void)GammaRay__ObjectInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ObjectInspectorWidget: public Ui_ObjectInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_OBJECTINSPECTORWIDGET_H
