/*
    SPDX-FileCopyrightText: 2017 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: MIT
*/

#ifndef KUSERFEEDBACK_CORE_LOGGING_H
#define KUSERFEEDBACK_CORE_LOGGING_H

#include <qglobal.h>

#include <QLoggingCategory>

namespace KUserFeedback {
Q_DECLARE_LOGGING_CATEGORY(Log)
}

#endif
