/****************************************************************************
** Meta object code from reading C++ file 'remoteviewinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/common/remoteviewinterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'remoteviewinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::RemoteViewInterface::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::RemoteViewInterface",
        "reset",
        "",
        "elementsAtReceived",
        "QList<GammaRay::ObjectId>",
        "ids",
        "bestCandidate",
        "frameUpdated",
        "GammaRay::RemoteViewFrame",
        "frame",
        "requestElementsAt",
        "QPoint",
        "pos",
        "GammaRay::RemoteViewInterface::RequestMode",
        "mode",
        "pickElementId",
        "GammaRay::ObjectId",
        "id",
        "sendKeyEvent",
        "type",
        "key",
        "modifiers",
        "text",
        "autorep",
        "count",
        "sendMouseEvent",
        "localPos",
        "button",
        "buttons",
        "sendWheelEvent",
        "pixelDelta",
        "angleDelta",
        "sendTouchEvent",
        "deviceName",
        "deviceSystemId",
        "touchDeviceType",
        "deviceCaps",
        "touchDeviceMaxTouchPoints",
        "QList<QTouchEvent::TouchPoint>",
        "touchPoints",
        "sendUserViewport",
        "QRectF",
        "userViewport",
        "setViewActive",
        "active",
        "clientViewUpdated",
        "requestCompleteFrame"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'reset'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'elementsAtReceived'
        QtMocHelpers::SignalData<void(const QList<GammaRay::ObjectId> &, int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'frameUpdated'
        QtMocHelpers::SignalData<void(const GammaRay::RemoteViewFrame &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'requestElementsAt'
        QtMocHelpers::SlotData<void(const QPoint &, GammaRay::RemoteViewInterface::RequestMode)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'pickElementId'
        QtMocHelpers::SlotData<void(const GammaRay::ObjectId &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'sendKeyEvent'
        QtMocHelpers::SlotData<void(int, int, int, const QString &, bool, ushort)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 }, { QMetaType::QString, 22 },
            { QMetaType::Bool, 23 }, { QMetaType::UShort, 24 },
        }}),
        // Slot 'sendKeyEvent'
        QtMocHelpers::SlotData<void(int, int, int, const QString &, bool)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 }, { QMetaType::QString, 22 },
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'sendKeyEvent'
        QtMocHelpers::SlotData<void(int, int, int, const QString &)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 }, { QMetaType::QString, 22 },
        }}),
        // Slot 'sendKeyEvent'
        QtMocHelpers::SlotData<void(int, int, int)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'sendMouseEvent'
        QtMocHelpers::SlotData<void(int, const QPoint &, int, int, int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { 0x80000000 | 11, 26 }, { QMetaType::Int, 27 }, { QMetaType::Int, 28 },
            { QMetaType::Int, 21 },
        }}),
        // Slot 'sendWheelEvent'
        QtMocHelpers::SlotData<void(const QPoint &, QPoint, QPoint, int, int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 26 }, { 0x80000000 | 11, 30 }, { 0x80000000 | 11, 31 }, { QMetaType::Int, 28 },
            { QMetaType::Int, 21 },
        }}),
        // Slot 'sendTouchEvent'
        QtMocHelpers::SlotData<void(const QString &, int, int, int, int, int, int, const QList<QTouchEvent::TouchPoint> &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 }, { QMetaType::Int, 34 }, { QMetaType::Int, 19 }, { QMetaType::Int, 35 },
            { QMetaType::Int, 36 }, { QMetaType::Int, 37 }, { QMetaType::Int, 21 }, { 0x80000000 | 38, 39 },
        }}),
        // Slot 'sendUserViewport'
        QtMocHelpers::SlotData<void(const QRectF &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 42 },
        }}),
        // Slot 'setViewActive'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 44 },
        }}),
        // Slot 'clientViewUpdated'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestCompleteFrame'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RemoteViewInterface, qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::RemoteViewInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t>.metaTypes,
    nullptr
} };

void GammaRay::RemoteViewInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteViewInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reset(); break;
        case 1: _t->elementsAtReceived((*reinterpret_cast<std::add_pointer_t<QList<GammaRay::ObjectId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->frameUpdated((*reinterpret_cast<std::add_pointer_t<GammaRay::RemoteViewFrame>>(_a[1]))); break;
        case 3: _t->requestElementsAt((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GammaRay::RemoteViewInterface::RequestMode>>(_a[2]))); break;
        case 4: _t->pickElementId((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectId>>(_a[1]))); break;
        case 5: _t->sendKeyEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[6]))); break;
        case 6: _t->sendKeyEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 7: _t->sendKeyEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 8: _t->sendKeyEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->sendMouseEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 10: _t->sendWheelEvent((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 11: _t->sendTouchEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QList<QTouchEvent::TouchPoint>>>(_a[8]))); break;
        case 12: _t->sendUserViewport((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 13: _t->setViewActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->clientViewUpdated(); break;
        case 15: _t->requestCompleteFrame(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<GammaRay::ObjectId> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::RemoteViewInterface::RequestMode >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectId >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 7:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QTouchEvent::TouchPoint> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RemoteViewInterface::*)()>(_a, &RemoteViewInterface::reset, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewInterface::*)(const QList<GammaRay::ObjectId> & , int )>(_a, &RemoteViewInterface::elementsAtReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewInterface::*)(const GammaRay::RemoteViewFrame & )>(_a, &RemoteViewInterface::frameUpdated, 2))
            return;
    }
}

const QMetaObject *GammaRay::RemoteViewInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::RemoteViewInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay19RemoteViewInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::RemoteViewInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::RemoteViewInterface::reset()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::RemoteViewInterface::elementsAtReceived(const QList<GammaRay::ObjectId> & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void GammaRay::RemoteViewInterface::frameUpdated(const GammaRay::RemoteViewFrame & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
