/********************************************************************************
** Form generated from reading UI file 'widgetinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WIDGETINSPECTORWIDGET_H
#define UI_WIDGETINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>
#include <ui/favoritesitemview.h>
#include <ui/propertywidget.h>

namespace GammaRay {

class Ui_WidgetInspectorWidget
{
public:
    QAction *actionSaveAsImage;
    QAction *actionSaveAsSvg;
    QAction *actionSaveAsUiFile;
    QAction *actionAnalyzePainting;
    QVBoxLayout *verticalLayout_4;
    QSplitter *mainSplitter;
    QWidget *mainLayoutWidget;
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *widgetTreeTab;
    QVBoxLayout *verticalLayout_3;
    QSplitter *previewSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout_2;
    QLineEdit *widgetSearchLine;
    GammaRay::ObjectsFavoriteView *favoritesTreeView;
    GammaRay::DeferredTreeView *widgetTreeView;
    QWidget *widgetPreviewContainer;
    GammaRay::PropertyWidget *widgetPropertyWidget;

    void setupUi(QWidget *GammaRay__WidgetInspectorWidget)
    {
        if (GammaRay__WidgetInspectorWidget->objectName().isEmpty())
            GammaRay__WidgetInspectorWidget->setObjectName("GammaRay__WidgetInspectorWidget");
        GammaRay__WidgetInspectorWidget->resize(732, 528);
        actionSaveAsImage = new QAction(GammaRay__WidgetInspectorWidget);
        actionSaveAsImage->setObjectName("actionSaveAsImage");
        actionSaveAsSvg = new QAction(GammaRay__WidgetInspectorWidget);
        actionSaveAsSvg->setObjectName("actionSaveAsSvg");
        actionSaveAsUiFile = new QAction(GammaRay__WidgetInspectorWidget);
        actionSaveAsUiFile->setObjectName("actionSaveAsUiFile");
        actionAnalyzePainting = new QAction(GammaRay__WidgetInspectorWidget);
        actionAnalyzePainting->setObjectName("actionAnalyzePainting");
        verticalLayout_4 = new QVBoxLayout(GammaRay__WidgetInspectorWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        mainSplitter = new QSplitter(GammaRay__WidgetInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        mainLayoutWidget = new QWidget(mainSplitter);
        mainLayoutWidget->setObjectName("mainLayoutWidget");
        verticalLayout = new QVBoxLayout(mainLayoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(mainLayoutWidget);
        tabWidget->setObjectName("tabWidget");
        widgetTreeTab = new QWidget();
        widgetTreeTab->setObjectName("widgetTreeTab");
        verticalLayout_3 = new QVBoxLayout(widgetTreeTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        previewSplitter = new QSplitter(widgetTreeTab);
        previewSplitter->setObjectName("previewSplitter");
        previewSplitter->setOrientation(Qt::Vertical);
        layoutWidget = new QWidget(previewSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout_2 = new QVBoxLayout(layoutWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        widgetSearchLine = new QLineEdit(layoutWidget);
        widgetSearchLine->setObjectName("widgetSearchLine");

        verticalLayout_2->addWidget(widgetSearchLine);

        favoritesTreeView = new GammaRay::ObjectsFavoriteView(layoutWidget);
        favoritesTreeView->setObjectName("favoritesTreeView");

        verticalLayout_2->addWidget(favoritesTreeView);

        widgetTreeView = new GammaRay::DeferredTreeView(layoutWidget);
        widgetTreeView->setObjectName("widgetTreeView");
        widgetTreeView->setContextMenuPolicy(Qt::CustomContextMenu);
        widgetTreeView->setUniformRowHeights(true);

        verticalLayout_2->addWidget(widgetTreeView);

        previewSplitter->addWidget(layoutWidget);
        widgetPreviewContainer = new QWidget(previewSplitter);
        widgetPreviewContainer->setObjectName("widgetPreviewContainer");
        previewSplitter->addWidget(widgetPreviewContainer);

        verticalLayout_3->addWidget(previewSplitter);

        tabWidget->addTab(widgetTreeTab, QString());

        verticalLayout->addWidget(tabWidget);

        mainSplitter->addWidget(mainLayoutWidget);
        widgetPropertyWidget = new GammaRay::PropertyWidget(mainSplitter);
        widgetPropertyWidget->setObjectName("widgetPropertyWidget");
        mainSplitter->addWidget(widgetPropertyWidget);

        verticalLayout_4->addWidget(mainSplitter);


        retranslateUi(GammaRay__WidgetInspectorWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__WidgetInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__WidgetInspectorWidget)
    {
        actionSaveAsImage->setText(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Save as &Image...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAsImage->setToolTip(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Save currently selected widget as image.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveAsSvg->setText(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Save as &SVG...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAsSvg->setToolTip(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Renders the currently selected image as SVG vector graphic.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveAsUiFile->setText(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Save as &UI file...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAsUiFile->setToolTip(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Saves the currently selected widget as a Qt designer UI file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAnalyzePainting->setText(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "&Analyze Painting...", nullptr));
#if QT_CONFIG(shortcut)
        actionAnalyzePainting->setShortcut(QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "F9", nullptr));
#endif // QT_CONFIG(shortcut)
        tabWidget->setTabText(tabWidget->indexOf(widgetTreeTab), QCoreApplication::translate("GammaRay::WidgetInspectorWidget", "Tree", nullptr));
        (void)GammaRay__WidgetInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class WidgetInspectorWidget: public Ui_WidgetInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_WIDGETINSPECTORWIDGET_H
