/********************************************************************************
** Form generated from reading UI file 'sceneinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCENEINSPECTORWIDGET_H
#define UI_SCENEINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>
#include <ui/propertywidget.h>
#include "graphicssceneview.h"

namespace GammaRay {

class Ui_SceneInspectorWidget
{
public:
    QVBoxLayout *verticalLayout_6;
    QComboBox *sceneComboBox;
    QSplitter *mainSplitter;
    QSplitter *previewSplitter;
    QWidget *layoutWidget_6;
    QVBoxLayout *verticalLayout_4;
    QLineEdit *sceneTreeSearchLine;
    GammaRay::DeferredTreeView *sceneTreeView;
    GammaRay::GraphicsSceneView *graphicsSceneView;
    GammaRay::PropertyWidget *scenePropertyWidget;

    void setupUi(QWidget *GammaRay__SceneInspectorWidget)
    {
        if (GammaRay__SceneInspectorWidget->objectName().isEmpty())
            GammaRay__SceneInspectorWidget->setObjectName("GammaRay__SceneInspectorWidget");
        GammaRay__SceneInspectorWidget->resize(400, 300);
        verticalLayout_6 = new QVBoxLayout(GammaRay__SceneInspectorWidget);
        verticalLayout_6->setObjectName("verticalLayout_6");
        sceneComboBox = new QComboBox(GammaRay__SceneInspectorWidget);
        sceneComboBox->setObjectName("sceneComboBox");

        verticalLayout_6->addWidget(sceneComboBox);

        mainSplitter = new QSplitter(GammaRay__SceneInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        previewSplitter = new QSplitter(mainSplitter);
        previewSplitter->setObjectName("previewSplitter");
        previewSplitter->setOrientation(Qt::Vertical);
        layoutWidget_6 = new QWidget(previewSplitter);
        layoutWidget_6->setObjectName("layoutWidget_6");
        verticalLayout_4 = new QVBoxLayout(layoutWidget_6);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        sceneTreeSearchLine = new QLineEdit(layoutWidget_6);
        sceneTreeSearchLine->setObjectName("sceneTreeSearchLine");

        verticalLayout_4->addWidget(sceneTreeSearchLine);

        sceneTreeView = new GammaRay::DeferredTreeView(layoutWidget_6);
        sceneTreeView->setObjectName("sceneTreeView");
        sceneTreeView->setContextMenuPolicy(Qt::CustomContextMenu);

        verticalLayout_4->addWidget(sceneTreeView);

        previewSplitter->addWidget(layoutWidget_6);
        graphicsSceneView = new GammaRay::GraphicsSceneView(previewSplitter);
        graphicsSceneView->setObjectName("graphicsSceneView");
        previewSplitter->addWidget(graphicsSceneView);
        mainSplitter->addWidget(previewSplitter);
        scenePropertyWidget = new GammaRay::PropertyWidget(mainSplitter);
        scenePropertyWidget->setObjectName("scenePropertyWidget");
        mainSplitter->addWidget(scenePropertyWidget);

        verticalLayout_6->addWidget(mainSplitter);


        retranslateUi(GammaRay__SceneInspectorWidget);

        QMetaObject::connectSlotsByName(GammaRay__SceneInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__SceneInspectorWidget)
    {
        (void)GammaRay__SceneInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class SceneInspectorWidget: public Ui_SceneInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_SCENEINSPECTORWIDGET_H
