/********************************************************************************
** Form generated from reading UI file 'propertyintpaireditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYINTPAIREDITOR_H
#define UI_PROPERTYINTPAIREDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_PropertyIntPairEditor
{
public:
    QHBoxLayout *horizontalLayout;
    QSpinBox *xBox;
    QLabel *label;
    QSpinBox *yBox;

    void setupUi(QWidget *GammaRay__PropertyIntPairEditor)
    {
        if (GammaRay__PropertyIntPairEditor->objectName().isEmpty())
            GammaRay__PropertyIntPairEditor->setObjectName("GammaRay__PropertyIntPairEditor");
        GammaRay__PropertyIntPairEditor->resize(145, 22);
        horizontalLayout = new QHBoxLayout(GammaRay__PropertyIntPairEditor);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        xBox = new QSpinBox(GammaRay__PropertyIntPairEditor);
        xBox->setObjectName("xBox");
        xBox->setFrame(false);
        xBox->setMinimum(-99999);
        xBox->setMaximum(99999);

        horizontalLayout->addWidget(xBox);

        label = new QLabel(GammaRay__PropertyIntPairEditor);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(label);

        yBox = new QSpinBox(GammaRay__PropertyIntPairEditor);
        yBox->setObjectName("yBox");
        yBox->setFrame(false);

        horizontalLayout->addWidget(yBox);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(2, 1);

        retranslateUi(GammaRay__PropertyIntPairEditor);

        QMetaObject::connectSlotsByName(GammaRay__PropertyIntPairEditor);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__PropertyIntPairEditor)
    {
        label->setText(QCoreApplication::translate("GammaRay::PropertyIntPairEditor", "x", nullptr));
        (void)GammaRay__PropertyIntPairEditor;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertyIntPairEditor: public Ui_PropertyIntPairEditor {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTYINTPAIREDITOR_H
