/********************************************************************************
** Form generated from reading UI file 'widgetattributetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WIDGETATTRIBUTETAB_H
#define UI_WIDGETATTRIBUTETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_WidgetAttributeTab
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *attributeView;

    void setupUi(QWidget *GammaRay__WidgetAttributeTab)
    {
        if (GammaRay__WidgetAttributeTab->objectName().isEmpty())
            GammaRay__WidgetAttributeTab->setObjectName("GammaRay__WidgetAttributeTab");
        GammaRay__WidgetAttributeTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__WidgetAttributeTab);
        verticalLayout->setObjectName("verticalLayout");
        attributeView = new GammaRay::DeferredTreeView(GammaRay__WidgetAttributeTab);
        attributeView->setObjectName("attributeView");
        attributeView->setRootIsDecorated(false);
        attributeView->setUniformRowHeights(true);

        verticalLayout->addWidget(attributeView);


        retranslateUi(GammaRay__WidgetAttributeTab);

        QMetaObject::connectSlotsByName(GammaRay__WidgetAttributeTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__WidgetAttributeTab)
    {
        (void)GammaRay__WidgetAttributeTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class WidgetAttributeTab: public Ui_WidgetAttributeTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_WIDGETATTRIBUTETAB_H
