/****************************************************************************
** Meta object code from reading C++ file 'textdocumentinspector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/textdocumentinspector/textdocumentinspector.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textdocumentinspector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::TextDocumentInspector::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::TextDocumentInspector",
        "documentSelected",
        "",
        "QItemSelection",
        "selected",
        "deselected",
        "documentElementSelected",
        "objectSelected",
        "obj"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'documentSelected'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Slot 'documentElementSelected'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Slot 'objectSelected'
        QtMocHelpers::SlotData<void(QObject *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextDocumentInspector, qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::TextDocumentInspector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t>.metaTypes,
    nullptr
} };

void GammaRay::TextDocumentInspector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextDocumentInspector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->documentSelected((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 1: _t->documentElementSelected((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 2: _t->objectSelected((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *GammaRay::TextDocumentInspector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::TextDocumentInspector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21TextDocumentInspectorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::TextDocumentInspector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::TextDocumentInspectorFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::TextDocumentInspectorFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextDocumentInspectorFactory, qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::TextDocumentInspectorFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::TextDocumentInspectorFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextDocumentInspectorFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::TextDocumentInspectorFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::TextDocumentInspectorFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay28TextDocumentInspectorFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QTextDocument,TextDocumentInspector>"))
        return static_cast< StandardToolFactory<QTextDocument,TextDocumentInspector>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::TextDocumentInspectorFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_TextDocumentInspectorFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x78,  0x1c,  'T',  'e',  'x',  't',  'D', 
    'o',  'c',  'u',  'm',  'e',  'n',  't',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x78,  0x1e,  'g', 
    'a',  'm',  'm',  'a',  'r',  'a',  'y',  '_', 
    't',  'e',  'x',  't',  'd',  'o',  'c',  'u', 
    'm',  'e',  'n',  't',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x6e,  'T',  'e',  'x',  't',  ' ', 
    'D',  'o',  'c',  'u',  'm',  'e',  'n',  't', 
    's',  0x68,  'n',  'a',  'm',  'e',  '[',  'd', 
    'e',  ']',  0x6e,  'T',  'e',  'x',  't',  ' ', 
    'D',  'o',  'k',  'u',  'm',  'e',  'n',  't', 
    'e',  0x6f,  's',  'e',  'l',  'e',  'c',  't', 
    'a',  'b',  'l',  'e',  'T',  'y',  'p',  'e', 
    's',  0x83,  0x6d,  'Q',  'T',  'e',  'x',  't', 
    'D',  'o',  'c',  'u',  'm',  'e',  'n',  't', 
    0x6b,  'Q',  'T',  'e',  'x',  't',  'O',  'b', 
    'j',  'e',  'c',  't',  0x78,  0x1b,  'Q',  'A', 
    'b',  's',  't',  'r',  'a',  'c',  't',  'T', 
    'e',  'x',  't',  'D',  'o',  'c',  'u',  'm', 
    'e',  'n',  't',  'L',  'a',  'y',  'o',  'u', 
    't',  0x65,  't',  'y',  'p',  'e',  's',  0x81, 
    0x6d,  'Q',  'T',  'e',  'x',  't',  'D',  'o', 
    'c',  'u',  'm',  'e',  'n',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::TextDocumentInspectorFactory, TextDocumentInspectorFactory, qt_pluginMetaDataV2_TextDocumentInspectorFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_TextDocumentInspectorFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x78,  0x1c,  'T',  'e',  'x',  't',  'D', 
    'o',  'c',  'u',  'm',  'e',  'n',  't',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x78,  0x1e,  'g', 
    'a',  'm',  'm',  'a',  'r',  'a',  'y',  '_', 
    't',  'e',  'x',  't',  'd',  'o',  'c',  'u', 
    'm',  'e',  'n',  't',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x6e,  'T',  'e',  'x',  't',  ' ', 
    'D',  'o',  'c',  'u',  'm',  'e',  'n',  't', 
    's',  0x68,  'n',  'a',  'm',  'e',  '[',  'd', 
    'e',  ']',  0x6e,  'T',  'e',  'x',  't',  ' ', 
    'D',  'o',  'k',  'u',  'm',  'e',  'n',  't', 
    'e',  0x6f,  's',  'e',  'l',  'e',  'c',  't', 
    'a',  'b',  'l',  'e',  'T',  'y',  'p',  'e', 
    's',  0x83,  0x6d,  'Q',  'T',  'e',  'x',  't', 
    'D',  'o',  'c',  'u',  'm',  'e',  'n',  't', 
    0x6b,  'Q',  'T',  'e',  'x',  't',  'O',  'b', 
    'j',  'e',  'c',  't',  0x78,  0x1b,  'Q',  'A', 
    'b',  's',  't',  'r',  'a',  'c',  't',  'T', 
    'e',  'x',  't',  'D',  'o',  'c',  'u',  'm', 
    'e',  'n',  't',  'L',  'a',  'y',  'o',  'u', 
    't',  0x65,  't',  'y',  'p',  'e',  's',  0x81, 
    0x6d,  'Q',  'T',  'e',  'x',  't',  'D',  'o', 
    'c',  'u',  'm',  'e',  'n',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::TextDocumentInspectorFactory, TextDocumentInspectorFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
