/********************************************************************************
** Form generated from reading UI file 'applicationattributetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_APPLICATIONATTRIBUTETAB_H
#define UI_APPLICATIONATTRIBUTETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ApplicationAttributeTab
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *attributeView;

    void setupUi(QWidget *GammaRay__ApplicationAttributeTab)
    {
        if (GammaRay__ApplicationAttributeTab->objectName().isEmpty())
            GammaRay__ApplicationAttributeTab->setObjectName("GammaRay__ApplicationAttributeTab");
        GammaRay__ApplicationAttributeTab->resize(399, 300);
        verticalLayout = new QVBoxLayout(GammaRay__ApplicationAttributeTab);
        verticalLayout->setObjectName("verticalLayout");
        attributeView = new GammaRay::DeferredTreeView(GammaRay__ApplicationAttributeTab);
        attributeView->setObjectName("attributeView");
        attributeView->setRootIsDecorated(false);
        attributeView->setUniformRowHeights(true);

        verticalLayout->addWidget(attributeView);


        retranslateUi(GammaRay__ApplicationAttributeTab);

        QMetaObject::connectSlotsByName(GammaRay__ApplicationAttributeTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ApplicationAttributeTab)
    {
        (void)GammaRay__ApplicationAttributeTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ApplicationAttributeTab: public Ui_ApplicationAttributeTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_APPLICATIONATTRIBUTETAB_H
