/********************************************************************************
** Form generated from reading UI file 'problemreporterwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROBLEMREPORTERWIDGET_H
#define UI_PROBLEMREPORTERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ProblemReporterWidget
{
public:
    QVBoxLayout *toplevelLayout;
    QSplitter *splitter;
    QWidget *layoutLeftWidget;
    QVBoxLayout *verticalLeftLayout;
    QLabel *label;
    QListView *problemfilterwidget;
    QPushButton *scanButton;
    QWidget *layoutRightWidget;
    QVBoxLayout *verticalRightLayout;
    QLineEdit *searchLine;
    GammaRay::DeferredTreeView *problemView;
    QProgressBar *progressBar;

    void setupUi(QWidget *GammaRay__ProblemReporterWidget)
    {
        if (GammaRay__ProblemReporterWidget->objectName().isEmpty())
            GammaRay__ProblemReporterWidget->setObjectName("GammaRay__ProblemReporterWidget");
        GammaRay__ProblemReporterWidget->resize(400, 300);
        toplevelLayout = new QVBoxLayout(GammaRay__ProblemReporterWidget);
        toplevelLayout->setObjectName("toplevelLayout");
        splitter = new QSplitter(GammaRay__ProblemReporterWidget);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        layoutLeftWidget = new QWidget(splitter);
        layoutLeftWidget->setObjectName("layoutLeftWidget");
        verticalLeftLayout = new QVBoxLayout(layoutLeftWidget);
        verticalLeftLayout->setContentsMargins(0, 0, 0, 0);
        verticalLeftLayout->setObjectName("verticalLeftLayout");
        verticalLeftLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(layoutLeftWidget);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        verticalLeftLayout->addWidget(label);

        problemfilterwidget = new QListView(layoutLeftWidget);
        problemfilterwidget->setObjectName("problemfilterwidget");
        problemfilterwidget->setFrameShape(QFrame::NoFrame);
        problemfilterwidget->setFrameShadow(QFrame::Plain);
        problemfilterwidget->setSelectionMode(QAbstractItemView::NoSelection);
        problemfilterwidget->setResizeMode(QListView::Adjust);
        problemfilterwidget->setWordWrap(true);

        verticalLeftLayout->addWidget(problemfilterwidget);

        scanButton = new QPushButton(layoutLeftWidget);
        scanButton->setObjectName("scanButton");

        verticalLeftLayout->addWidget(scanButton);

        splitter->addWidget(layoutLeftWidget);
        layoutRightWidget = new QWidget(splitter);
        layoutRightWidget->setObjectName("layoutRightWidget");
        verticalRightLayout = new QVBoxLayout(layoutRightWidget);
        verticalRightLayout->setContentsMargins(0, 0, 0, 0);
        verticalRightLayout->setObjectName("verticalRightLayout");
        verticalRightLayout->setContentsMargins(0, 0, 0, 0);
        searchLine = new QLineEdit(layoutRightWidget);
        searchLine->setObjectName("searchLine");

        verticalRightLayout->addWidget(searchLine);

        problemView = new GammaRay::DeferredTreeView(layoutRightWidget);
        problemView->setObjectName("problemView");
        problemView->setContextMenuPolicy(Qt::CustomContextMenu);
        problemView->setRootIsDecorated(false);

        verticalRightLayout->addWidget(problemView);

        progressBar = new QProgressBar(layoutRightWidget);
        progressBar->setObjectName("progressBar");
        progressBar->setMaximum(0);
        progressBar->setValue(0);

        verticalRightLayout->addWidget(progressBar);

        splitter->addWidget(layoutRightWidget);

        toplevelLayout->addWidget(splitter);


        retranslateUi(GammaRay__ProblemReporterWidget);

        QMetaObject::connectSlotsByName(GammaRay__ProblemReporterWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ProblemReporterWidget)
    {
        label->setText(QCoreApplication::translate("GammaRay::ProblemReporterWidget", "Available Problem Checkers", nullptr));
        scanButton->setText(QCoreApplication::translate("GammaRay::ProblemReporterWidget", "Start Scan for Problems", nullptr));
        (void)GammaRay__ProblemReporterWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ProblemReporterWidget: public Ui_ProblemReporterWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROBLEMREPORTERWIDGET_H
