/********************************************************************************
** Form generated from reading UI file 'sysinfowidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SYSINFOWIDGET_H
#define UI_SYSINFOWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "standardpathswidget.h"

namespace GammaRay {

class Ui_SysInfoWidget
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QTreeView *sysInfoView;
    QLabel *label_2;
    QTreeView *libInfoView;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_4;
    GammaRay::StandardPathsWidget *widget;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_3;
    QTreeView *envView;

    void setupUi(QWidget *GammaRay__SysInfoWidget)
    {
        if (GammaRay__SysInfoWidget->objectName().isEmpty())
            GammaRay__SysInfoWidget->setObjectName("GammaRay__SysInfoWidget");
        GammaRay__SysInfoWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__SysInfoWidget);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(GammaRay__SysInfoWidget);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(tab);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        sysInfoView = new QTreeView(tab);
        sysInfoView->setObjectName("sysInfoView");
        sysInfoView->setRootIsDecorated(false);
        sysInfoView->setUniformRowHeights(true);

        verticalLayout_2->addWidget(sysInfoView);

        label_2 = new QLabel(tab);
        label_2->setObjectName("label_2");

        verticalLayout_2->addWidget(label_2);

        libInfoView = new QTreeView(tab);
        libInfoView->setObjectName("libInfoView");
        libInfoView->setRootIsDecorated(false);
        libInfoView->setUniformRowHeights(true);

        verticalLayout_2->addWidget(libInfoView);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_4 = new QVBoxLayout(tab_2);
        verticalLayout_4->setObjectName("verticalLayout_4");
        widget = new GammaRay::StandardPathsWidget(tab_2);
        widget->setObjectName("widget");

        verticalLayout_4->addWidget(widget);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_3 = new QVBoxLayout(tab_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        envView = new QTreeView(tab_3);
        envView->setObjectName("envView");
        envView->setRootIsDecorated(false);
        envView->setUniformRowHeights(true);

        verticalLayout_3->addWidget(envView);

        tabWidget->addTab(tab_3, QString());

        verticalLayout->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        label->setBuddy(sysInfoView);
        label_2->setBuddy(libInfoView);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__SysInfoWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__SysInfoWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__SysInfoWidget)
    {
        GammaRay__SysInfoWidget->setWindowTitle(QCoreApplication::translate("GammaRay::SysInfoWidget", "Form", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::SysInfoWidget", "System information:", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::SysInfoWidget", "Library information:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("GammaRay::SysInfoWidget", "Qt", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("GammaRay::SysInfoWidget", "Standard Paths", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("GammaRay::SysInfoWidget", "Environment", nullptr));
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class SysInfoWidget: public Ui_SysInfoWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_SYSINFOWIDGET_H
