/****************************************************************************
** Meta object code from reading C++ file 'quickinspector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/quickinspector/quickinspector.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quickinspector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::RenderModeRequest::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::RenderModeRequest",
        "aboutToCleanSceneGraph",
        "",
        "sceneGraphCleanedUp",
        "finished",
        "apply",
        "preFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToCleanSceneGraph'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sceneGraphCleanedUp'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'apply'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'preFinished'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RenderModeRequest, qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::RenderModeRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t>.metaTypes,
    nullptr
} };

void GammaRay::RenderModeRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RenderModeRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToCleanSceneGraph(); break;
        case 1: _t->sceneGraphCleanedUp(); break;
        case 2: _t->finished(); break;
        case 3: _t->apply(); break;
        case 4: _t->preFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RenderModeRequest::*)()>(_a, &RenderModeRequest::aboutToCleanSceneGraph, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RenderModeRequest::*)()>(_a, &RenderModeRequest::sceneGraphCleanedUp, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RenderModeRequest::*)()>(_a, &RenderModeRequest::finished, 2))
            return;
    }
}

const QMetaObject *GammaRay::RenderModeRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::RenderModeRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay17RenderModeRequestE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::RenderModeRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::RenderModeRequest::aboutToCleanSceneGraph()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::RenderModeRequest::sceneGraphCleanedUp()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GammaRay::RenderModeRequest::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN8GammaRay14QuickInspectorE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::QuickInspector::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay14QuickInspectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::QuickInspector",
        "elementsAtReceived",
        "",
        "GammaRay::ObjectIds",
        "ids",
        "bestCandidate",
        "selectWindow",
        "index",
        "setCustomRenderMode",
        "GammaRay::QuickInspectorInterface::RenderMode",
        "customRenderMode",
        "checkFeatures",
        "setOverlaySettings",
        "GammaRay::QuickDecorationsSettings",
        "settings",
        "checkOverlaySettings",
        "requestElementsAt",
        "QPoint",
        "pos",
        "GammaRay::RemoteViewInterface::RequestMode",
        "mode",
        "pickElementId",
        "GammaRay::ObjectId",
        "id",
        "sendRenderedScene",
        "GammaRay::GrabbedFrame",
        "grabbedFrame",
        "analyzePainting",
        "checkSlowMode",
        "setSlowMode",
        "slow",
        "slotGrabWindow",
        "itemSelectionChanged",
        "QItemSelection",
        "selection",
        "sgSelectionChanged",
        "sgNodeDeleted",
        "QSGNode*",
        "node",
        "qObjectSelected",
        "object",
        "nonQObjectSelected",
        "typeName",
        "objectCreated",
        "recreateOverlay",
        "aboutToCleanSceneGraph",
        "sceneGraphCleanedUp"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'elementsAtReceived'
        QtMocHelpers::SignalData<void(const GammaRay::ObjectIds &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'selectWindow'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setCustomRenderMode'
        QtMocHelpers::SlotData<void(GammaRay::QuickInspectorInterface::RenderMode)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'checkFeatures'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOverlaySettings'
        QtMocHelpers::SlotData<void(const GammaRay::QuickDecorationsSettings &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'checkOverlaySettings'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestElementsAt'
        QtMocHelpers::SlotData<void(const QPoint &, GammaRay::RemoteViewInterface::RequestMode)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'pickElementId'
        QtMocHelpers::SlotData<void(const GammaRay::ObjectId &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'sendRenderedScene'
        QtMocHelpers::SlotData<void(const GammaRay::GrabbedFrame &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'analyzePainting'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkSlowMode'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSlowMode'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Slot 'slotGrabWindow'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'itemSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'sgSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'sgNodeDeleted'
        QtMocHelpers::SlotData<void(QSGNode *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'qObjectSelected'
        QtMocHelpers::SlotData<void(QObject *)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 40 },
        }}),
        // Slot 'nonQObjectSelected'
        QtMocHelpers::SlotData<void(void *, const QString &)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::VoidStar, 40 }, { QMetaType::QString, 42 },
        }}),
        // Slot 'objectCreated'
        QtMocHelpers::SlotData<void(QObject *)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 40 },
        }}),
        // Slot 'recreateOverlay'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aboutToCleanSceneGraph'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'sceneGraphCleanedUp'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QuickInspector, qt_meta_tag_ZN8GammaRay14QuickInspectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::QuickInspector::staticMetaObject = { {
    QMetaObject::SuperData::link<QuickInspectorInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14QuickInspectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14QuickInspectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay14QuickInspectorE_t>.metaTypes,
    nullptr
} };

void GammaRay::QuickInspector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QuickInspector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->elementsAtReceived((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectIds>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->selectWindow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setCustomRenderMode((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickInspectorInterface::RenderMode>>(_a[1]))); break;
        case 3: _t->checkFeatures(); break;
        case 4: _t->setOverlaySettings((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickDecorationsSettings>>(_a[1]))); break;
        case 5: _t->checkOverlaySettings(); break;
        case 6: _t->requestElementsAt((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GammaRay::RemoteViewInterface::RequestMode>>(_a[2]))); break;
        case 7: _t->pickElementId((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectId>>(_a[1]))); break;
        case 8: _t->sendRenderedScene((*reinterpret_cast<std::add_pointer_t<GammaRay::GrabbedFrame>>(_a[1]))); break;
        case 9: _t->analyzePainting(); break;
        case 10: _t->checkSlowMode(); break;
        case 11: _t->setSlowMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->slotGrabWindow(); break;
        case 13: _t->itemSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 14: _t->sgSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 15: _t->sgNodeDeleted((*reinterpret_cast<std::add_pointer_t<QSGNode*>>(_a[1]))); break;
        case 16: _t->qObjectSelected((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 17: _t->nonQObjectSelected((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->objectCreated((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 19: _t->recreateOverlay(); break;
        case 20: _t->aboutToCleanSceneGraph(); break;
        case 21: _t->sceneGraphCleanedUp(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectIds >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::QuickInspectorInterface::RenderMode >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::RemoteViewInterface::RequestMode >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QuickInspector::*)(const GammaRay::ObjectIds & , int )>(_a, &QuickInspector::elementsAtReceived, 0))
            return;
    }
}

const QMetaObject *GammaRay::QuickInspector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::QuickInspector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay14QuickInspectorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.QuickInspectorInterface/1.0"))
        return static_cast< GammaRay::QuickInspectorInterface*>(this);
    return QuickInspectorInterface::qt_metacast(_clname);
}

int GammaRay::QuickInspector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QuickInspectorInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::QuickInspector::elementsAtReceived(const GammaRay::ObjectIds & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::QuickInspectorFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::QuickInspectorFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QuickInspectorFactory, qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::QuickInspectorFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::QuickInspectorFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QuickInspectorFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::QuickInspectorFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::QuickInspectorFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21QuickInspectorFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QQuickWindow,QuickInspector>"))
        return static_cast< StandardToolFactory<QQuickWindow,QuickInspector>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::QuickInspectorFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_QuickInspectorFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x75,  'Q',  'u',  'i',  'c',  'k',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x77,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'q', 
    'u',  'i',  'c',  'k',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x6c,  'Q',  'u',  'i',  'c',  'k', 
    ' ',  'S',  'c',  'e',  'n',  'e',  's',  0x68, 
    'n',  'a',  'm',  'e',  '[',  'd',  'e',  ']', 
    0x6c,  'Q',  'u',  'i',  'c',  'k',  ' ',  'S', 
    'z',  'e',  'n',  'e',  'n',  0x6f,  's',  'e', 
    'l',  'e',  'c',  't',  'a',  'b',  'l',  'e', 
    'T',  'y',  'p',  'e',  's',  0x83,  0x6a,  'Q', 
    'Q',  'u',  'i',  'c',  'k',  'I',  't',  'e', 
    'm',  0x67,  'Q',  'S',  'G',  'N',  'o',  'd', 
    'e',  0x6c,  'Q',  'Q',  'u',  'i',  'c',  'k', 
    'W',  'i',  'n',  'd',  'o',  'w',  0x65,  't', 
    'y',  'p',  'e',  's',  0x83,  0x6c,  'Q',  'Q', 
    'u',  'i',  'c',  'k',  'W',  'i',  'n',  'd', 
    'o',  'w',  0x6a,  'Q',  'Q',  'u',  'i',  'c', 
    'k',  'I',  't',  'e',  'm',  0x67,  'Q',  'S', 
    'G',  'N',  'o',  'd',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::QuickInspectorFactory, QuickInspectorFactory, qt_pluginMetaDataV2_QuickInspectorFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_QuickInspectorFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x75,  'Q',  'u',  'i',  'c',  'k',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x77,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'q', 
    'u',  'i',  'c',  'k',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x6c,  'Q',  'u',  'i',  'c',  'k', 
    ' ',  'S',  'c',  'e',  'n',  'e',  's',  0x68, 
    'n',  'a',  'm',  'e',  '[',  'd',  'e',  ']', 
    0x6c,  'Q',  'u',  'i',  'c',  'k',  ' ',  'S', 
    'z',  'e',  'n',  'e',  'n',  0x6f,  's',  'e', 
    'l',  'e',  'c',  't',  'a',  'b',  'l',  'e', 
    'T',  'y',  'p',  'e',  's',  0x83,  0x6a,  'Q', 
    'Q',  'u',  'i',  'c',  'k',  'I',  't',  'e', 
    'm',  0x67,  'Q',  'S',  'G',  'N',  'o',  'd', 
    'e',  0x6c,  'Q',  'Q',  'u',  'i',  'c',  'k', 
    'W',  'i',  'n',  'd',  'o',  'w',  0x65,  't', 
    'y',  'p',  'e',  's',  0x83,  0x6c,  'Q',  'Q', 
    'u',  'i',  'c',  'k',  'W',  'i',  'n',  'd', 
    'o',  'w',  0x6a,  'Q',  'Q',  'u',  'i',  'c', 
    'k',  'I',  't',  'e',  'm',  0x67,  'Q',  'S', 
    'G',  'N',  'o',  'd',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::QuickInspectorFactory, QuickInspectorFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
