/****************************************************************************
** Meta object code from reading C++ file 'widgetinspectorwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/widgetinspector/widgetinspectorwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'widgetinspectorwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::WidgetInspectorWidget::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::WidgetInspectorWidget",
        "widgetSelected",
        "",
        "QItemSelection",
        "selection",
        "widgetTreeContextMenu",
        "QPoint",
        "pos",
        "saveAsImage",
        "saveAsSvg",
        "saveAsUiFile",
        "analyzePainting",
        "updateActions",
        "propertyWidgetTabsChanged",
        "saveTargetState",
        "QSettings*",
        "settings",
        "restoreTargetState"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'widgetSelected'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'widgetTreeContextMenu'
        QtMocHelpers::SlotData<void(QPoint)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'saveAsImage'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAsSvg'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAsUiFile'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'analyzePainting'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'propertyWidgetTabsChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'saveTargetState'
        QtMocHelpers::MethodData<void(QSettings *) const>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Method 'restoreTargetState'
        QtMocHelpers::MethodData<void(QSettings *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WidgetInspectorWidget, qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::WidgetInspectorWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t>.metaTypes,
    nullptr
} };

void GammaRay::WidgetInspectorWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WidgetInspectorWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->widgetSelected((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 1: _t->widgetTreeContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->saveAsImage(); break;
        case 3: _t->saveAsSvg(); break;
        case 4: _t->saveAsUiFile(); break;
        case 5: _t->analyzePainting(); break;
        case 6: _t->updateActions(); break;
        case 7: _t->propertyWidgetTabsChanged(); break;
        case 8: _t->saveTargetState((*reinterpret_cast<std::add_pointer_t<QSettings*>>(_a[1]))); break;
        case 9: _t->restoreTargetState((*reinterpret_cast<std::add_pointer_t<QSettings*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
}

const QMetaObject *GammaRay::WidgetInspectorWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::WidgetInspectorWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int GammaRay::WidgetInspectorWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::WidgetInspectorUiFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::WidgetInspectorUiFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WidgetInspectorUiFactory, qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::WidgetInspectorUiFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::WidgetInspectorUiFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WidgetInspectorUiFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::WidgetInspectorUiFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::WidgetInspectorUiFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24WidgetInspectorUiFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolUiFactory<WidgetInspectorWidget>"))
        return static_cast< StandardToolUiFactory<WidgetInspectorWidget>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolUiFactory/1.0"))
        return static_cast< GammaRay::ToolUiFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::WidgetInspectorUiFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_WidgetInspectorUiFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1f,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'U',  'i',  'F',  'a',  'c',  't',  'o', 
    'r',  'y', 
    // "className"
    0x03,  0x78,  0x18,  'W',  'i',  'd',  'g',  'e', 
    't',  'I',  'n',  's',  'p',  'e',  'c',  't', 
    'o',  'r',  'U',  'i',  'F',  'a',  'c',  't', 
    'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa4,  0x62,  'i',  'd',  0x78,  0x18,  'g', 
    'a',  'm',  'm',  'a',  'r',  'a',  'y',  '_', 
    'w',  'i',  'd',  'g',  'e',  't',  'i',  'n', 
    's',  'p',  'e',  'c',  't',  'o',  'r',  0x64, 
    'n',  'a',  'm',  'e',  0x67,  'W',  'i',  'd', 
    'g',  'e',  't',  's',  0x6f,  's',  'e',  'l', 
    'e',  'c',  't',  'a',  'b',  'l',  'e',  'T', 
    'y',  'p',  'e',  's',  0x82,  0x67,  'Q',  'W', 
    'i',  'd',  'g',  'e',  't',  0x67,  'Q',  'L', 
    'a',  'y',  'o',  'u',  't',  0x65,  't',  'y', 
    'p',  'e',  's',  0x82,  0x67,  'Q',  'W',  'i', 
    'd',  'g',  'e',  't',  0x6c,  'Q',  'A',  'p', 
    'p',  'l',  'i',  'c',  'a',  't',  'i',  'o', 
    'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::WidgetInspectorUiFactory, WidgetInspectorUiFactory, qt_pluginMetaDataV2_WidgetInspectorUiFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_WidgetInspectorUiFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1f,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'U',  'i',  'F',  'a',  'c',  't',  'o', 
    'r',  'y', 
    // "className"
    0x03,  0x78,  0x18,  'W',  'i',  'd',  'g',  'e', 
    't',  'I',  'n',  's',  'p',  'e',  'c',  't', 
    'o',  'r',  'U',  'i',  'F',  'a',  'c',  't', 
    'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa4,  0x62,  'i',  'd',  0x78,  0x18,  'g', 
    'a',  'm',  'm',  'a',  'r',  'a',  'y',  '_', 
    'w',  'i',  'd',  'g',  'e',  't',  'i',  'n', 
    's',  'p',  'e',  'c',  't',  'o',  'r',  0x64, 
    'n',  'a',  'm',  'e',  0x67,  'W',  'i',  'd', 
    'g',  'e',  't',  's',  0x6f,  's',  'e',  'l', 
    'e',  'c',  't',  'a',  'b',  'l',  'e',  'T', 
    'y',  'p',  'e',  's',  0x82,  0x67,  'Q',  'W', 
    'i',  'd',  'g',  'e',  't',  0x67,  'Q',  'L', 
    'a',  'y',  'o',  'u',  't',  0x65,  't',  'y', 
    'p',  'e',  's',  0x82,  0x67,  'Q',  'W',  'i', 
    'd',  'g',  'e',  't',  0x6c,  'Q',  'A',  'p', 
    'p',  'l',  'i',  'c',  'a',  't',  'i',  'o', 
    'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::WidgetInspectorUiFactory, WidgetInspectorUiFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
