/********************************************************************************
** Form generated from reading UI file 'signalmonitorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SIGNALMONITORWIDGET_H
#define UI_SIGNALMONITORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "signalhistoryview.h"
#include "signalmonitorwidget.h"

namespace GammaRay {

class Ui_SignalMonitorWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *toolbarLayout;
    QLineEdit *objectSearchLine;
    QToolButton *pauseButton;
    QSpacerItem *toolbarSpacer;
    QLabel *intervalScaleLabel;
    QSlider *intervalScale;
    GammaRay::SignalHistoryFavoritesView *favoritesObjectsTreeView;
    GammaRay::SignalHistoryView *objectTreeView;
    QHBoxLayout *eventScrollBarLayout;
    QScrollBar *eventScrollBar;

    void setupUi(QWidget *GammaRay__SignalMonitorWidget)
    {
        if (GammaRay__SignalMonitorWidget->objectName().isEmpty())
            GammaRay__SignalMonitorWidget->setObjectName("GammaRay__SignalMonitorWidget");
        GammaRay__SignalMonitorWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__SignalMonitorWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        toolbarLayout = new QHBoxLayout();
        toolbarLayout->setObjectName("toolbarLayout");
        toolbarLayout->setContentsMargins(-1, -1, -1, 6);
        objectSearchLine = new QLineEdit(GammaRay__SignalMonitorWidget);
        objectSearchLine->setObjectName("objectSearchLine");

        toolbarLayout->addWidget(objectSearchLine);

        pauseButton = new QToolButton(GammaRay__SignalMonitorWidget);
        pauseButton->setObjectName("pauseButton");
        pauseButton->setCheckable(true);

        toolbarLayout->addWidget(pauseButton);

        toolbarSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        toolbarLayout->addItem(toolbarSpacer);

        intervalScaleLabel = new QLabel(GammaRay__SignalMonitorWidget);
        intervalScaleLabel->setObjectName("intervalScaleLabel");

        toolbarLayout->addWidget(intervalScaleLabel);

        intervalScale = new QSlider(GammaRay__SignalMonitorWidget);
        intervalScale->setObjectName("intervalScale");
        intervalScale->setMinimum(-100);
        intervalScale->setMaximum(100);
        intervalScale->setValue(0);
        intervalScale->setOrientation(Qt::Horizontal);

        toolbarLayout->addWidget(intervalScale);


        verticalLayout->addLayout(toolbarLayout);

        favoritesObjectsTreeView = new GammaRay::SignalHistoryFavoritesView(GammaRay__SignalMonitorWidget);
        favoritesObjectsTreeView->setObjectName("favoritesObjectsTreeView");

        verticalLayout->addWidget(favoritesObjectsTreeView);

        objectTreeView = new GammaRay::SignalHistoryView(GammaRay__SignalMonitorWidget);
        objectTreeView->setObjectName("objectTreeView");
        objectTreeView->setContextMenuPolicy(Qt::CustomContextMenu);
        objectTreeView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        objectTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
        objectTreeView->setRootIsDecorated(false);
        objectTreeView->setUniformRowHeights(true);

        verticalLayout->addWidget(objectTreeView);

        eventScrollBarLayout = new QHBoxLayout();
        eventScrollBarLayout->setObjectName("eventScrollBarLayout");
        eventScrollBar = new QScrollBar(GammaRay__SignalMonitorWidget);
        eventScrollBar->setObjectName("eventScrollBar");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(eventScrollBar->sizePolicy().hasHeightForWidth());
        eventScrollBar->setSizePolicy(sizePolicy);
        eventScrollBar->setTracking(true);
        eventScrollBar->setOrientation(Qt::Horizontal);

        eventScrollBarLayout->addWidget(eventScrollBar);


        verticalLayout->addLayout(eventScrollBarLayout);


        retranslateUi(GammaRay__SignalMonitorWidget);

        QMetaObject::connectSlotsByName(GammaRay__SignalMonitorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__SignalMonitorWidget)
    {
        pauseButton->setText(QCoreApplication::translate("GammaRay::SignalMonitorWidget", "Pause", nullptr));
        intervalScaleLabel->setText(QCoreApplication::translate("GammaRay::SignalMonitorWidget", "Zoom Level:", nullptr));
        (void)GammaRay__SignalMonitorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class SignalMonitorWidget: public Ui_SignalMonitorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_SIGNALMONITORWIDGET_H
