/****************************************************************************
** Meta object code from reading C++ file 'quickinspectorinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/quickinspector/quickinspectorinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quickinspectorinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::QuickInspectorInterface::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::QuickInspectorInterface",
        "features",
        "",
        "QFlags<GammaRay::QuickInspectorInterface::Feature>",
        "serverSideDecorationChanged",
        "enabled",
        "overlaySettings",
        "GammaRay::QuickDecorationsSettings",
        "settings",
        "slowModeChanged",
        "slow",
        "selectWindow",
        "index",
        "setCustomRenderMode",
        "GammaRay::QuickInspectorInterface::RenderMode",
        "customRenderMode",
        "checkFeatures",
        "setOverlaySettings",
        "checkOverlaySettings",
        "analyzePainting",
        "checkSlowMode",
        "setSlowMode",
        "serverSideDecoration",
        "RenderMode",
        "NormalRendering",
        "VisualizeClipping",
        "VisualizeOverdraw",
        "VisualizeBatches",
        "VisualizeChanges",
        "VisualizeTraces"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'features'
        QtMocHelpers::SignalData<void(QFlags<GammaRay::QuickInspectorInterface::Feature>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 1 },
        }}),
        // Signal 'serverSideDecorationChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'overlaySettings'
        QtMocHelpers::SignalData<void(const GammaRay::QuickDecorationsSettings &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'slowModeChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'selectWindow'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setCustomRenderMode'
        QtMocHelpers::SlotData<void(GammaRay::QuickInspectorInterface::RenderMode)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'checkFeatures'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOverlaySettings'
        QtMocHelpers::SlotData<void(const GammaRay::QuickDecorationsSettings &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'checkOverlaySettings'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'analyzePainting'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkSlowMode'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSlowMode'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'serverSideDecoration'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RenderMode'
        QtMocHelpers::EnumData<enum RenderMode>(23, 23, QMC::EnumFlags{}).add({
            {   24, RenderMode::NormalRendering },
            {   25, RenderMode::VisualizeClipping },
            {   26, RenderMode::VisualizeOverdraw },
            {   27, RenderMode::VisualizeBatches },
            {   28, RenderMode::VisualizeChanges },
            {   29, RenderMode::VisualizeTraces },
        }),
    };
    return QtMocHelpers::metaObjectData<QuickInspectorInterface, qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::QuickInspectorInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t>.metaTypes,
    nullptr
} };

void GammaRay::QuickInspectorInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QuickInspectorInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->features((*reinterpret_cast<std::add_pointer_t<QFlags<GammaRay::QuickInspectorInterface::Feature>>>(_a[1]))); break;
        case 1: _t->serverSideDecorationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->overlaySettings((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickDecorationsSettings>>(_a[1]))); break;
        case 3: _t->slowModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->selectWindow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setCustomRenderMode((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickInspectorInterface::RenderMode>>(_a[1]))); break;
        case 6: _t->checkFeatures(); break;
        case 7: _t->setOverlaySettings((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickDecorationsSettings>>(_a[1]))); break;
        case 8: _t->checkOverlaySettings(); break;
        case 9: _t->analyzePainting(); break;
        case 10: _t->checkSlowMode(); break;
        case 11: _t->setSlowMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::QuickInspectorInterface::RenderMode >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QuickInspectorInterface::*)(QFlags<GammaRay::QuickInspectorInterface::Feature> )>(_a, &QuickInspectorInterface::features, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QuickInspectorInterface::*)(bool )>(_a, &QuickInspectorInterface::serverSideDecorationChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QuickInspectorInterface::*)(const GammaRay::QuickDecorationsSettings & )>(_a, &QuickInspectorInterface::overlaySettings, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QuickInspectorInterface::*)(bool )>(_a, &QuickInspectorInterface::slowModeChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->serverSideDecorationEnabled(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServerSideDecorationsEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *GammaRay::QuickInspectorInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::QuickInspectorInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23QuickInspectorInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::QuickInspectorInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::QuickInspectorInterface::features(QFlags<GammaRay::QuickInspectorInterface::Feature> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GammaRay::QuickInspectorInterface::serverSideDecorationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GammaRay::QuickInspectorInterface::overlaySettings(const GammaRay::QuickDecorationsSettings & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void GammaRay::QuickInspectorInterface::slowModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
