/********************************************************************************
** Form generated from reading UI file 'standardpathswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STANDARDPATHSWIDGET_H
#define UI_STANDARDPATHSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_StandardPathsWidget
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::DeferredTreeView *pathView;

    void setupUi(QWidget *GammaRay__StandardPathsWidget)
    {
        if (GammaRay__StandardPathsWidget->objectName().isEmpty())
            GammaRay__StandardPathsWidget->setObjectName("GammaRay__StandardPathsWidget");
        GammaRay__StandardPathsWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__StandardPathsWidget);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        pathView = new GammaRay::DeferredTreeView(GammaRay__StandardPathsWidget);
        pathView->setObjectName("pathView");
        pathView->setAlternatingRowColors(true);
        pathView->setRootIsDecorated(false);

        verticalLayout->addWidget(pathView);


        retranslateUi(GammaRay__StandardPathsWidget);

        QMetaObject::connectSlotsByName(GammaRay__StandardPathsWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__StandardPathsWidget)
    {
        (void)GammaRay__StandardPathsWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class StandardPathsWidget: public Ui_StandardPathsWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_STANDARDPATHSWIDGET_H
