/****************************************************************************
** Meta object code from reading C++ file 'quickinspectorwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/quickinspector/quickinspectorwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quickinspectorwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::QuickInspectorWidget::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::QuickInspectorWidget",
        "itemSelectionChanged",
        "",
        "QItemSelection",
        "selection",
        "sgSelectionChanged",
        "setFeatures",
        "GammaRay::QuickInspectorInterface::Features",
        "features",
        "setOverlaySettings",
        "GammaRay::QuickDecorationsSettings",
        "settings",
        "setSlowMode",
        "slow",
        "itemModelDataChanged",
        "QModelIndex",
        "topLeft",
        "bottomRight",
        "QList<int>",
        "roles",
        "itemContextMenu",
        "QPoint",
        "pos",
        "stateReceived",
        "GammaRay::QuickInspectorWidget::StateFlag",
        "flag",
        "resetState",
        "saveState",
        "saveAsImage",
        "updateActions",
        "saveTargetState",
        "QSettings*",
        "restoreTargetState"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'itemSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'sgSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setFeatures'
        QtMocHelpers::SlotData<void(GammaRay::QuickInspectorInterface::Features)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setOverlaySettings'
        QtMocHelpers::SlotData<void(const GammaRay::QuickDecorationsSettings &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setSlowMode'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'itemModelDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &, const QVector<int> &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'itemContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'stateReceived'
        QtMocHelpers::SlotData<void(GammaRay::QuickInspectorWidget::StateFlag)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'resetState'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveState'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAsImage'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'saveTargetState'
        QtMocHelpers::MethodData<void(QSettings *) const>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 11 },
        }}),
        // Method 'restoreTargetState'
        QtMocHelpers::MethodData<void(QSettings *)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QuickInspectorWidget, qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::QuickInspectorWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t>.metaTypes,
    nullptr
} };

void GammaRay::QuickInspectorWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QuickInspectorWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 1: _t->sgSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 2: _t->setFeatures((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickInspectorInterface::Features>>(_a[1]))); break;
        case 3: _t->setOverlaySettings((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickDecorationsSettings>>(_a[1]))); break;
        case 4: _t->setSlowMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->itemModelDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 6: _t->itemContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 7: _t->stateReceived((*reinterpret_cast<std::add_pointer_t<GammaRay::QuickInspectorWidget::StateFlag>>(_a[1]))); break;
        case 8: _t->resetState(); break;
        case 9: _t->saveState(); break;
        case 10: _t->saveAsImage(); break;
        case 11: _t->updateActions(); break;
        case 12: _t->saveTargetState((*reinterpret_cast<std::add_pointer_t<QSettings*>>(_a[1]))); break;
        case 13: _t->restoreTargetState((*reinterpret_cast<std::add_pointer_t<QSettings*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::QuickInspectorInterface::Features >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::QuickInspectorWidget::StateFlag >(); break;
            }
            break;
        }
    }
}

const QMetaObject *GammaRay::QuickInspectorWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::QuickInspectorWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay20QuickInspectorWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int GammaRay::QuickInspectorWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::QuickInspectorUiFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::QuickInspectorUiFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QuickInspectorUiFactory, qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::QuickInspectorUiFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::QuickInspectorUiFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QuickInspectorUiFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::QuickInspectorUiFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::QuickInspectorUiFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23QuickInspectorUiFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolUiFactory<QuickInspectorWidget>"))
        return static_cast< StandardToolUiFactory<QuickInspectorWidget>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolUiFactory/1.0"))
        return static_cast< GammaRay::ToolUiFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::QuickInspectorUiFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_QuickInspectorUiFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1f,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'U',  'i',  'F',  'a',  'c',  't',  'o', 
    'r',  'y', 
    // "className"
    0x03,  0x77,  'Q',  'u',  'i',  'c',  'k',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'U',  'i',  'F',  'a',  'c',  't',  'o',  'r', 
    'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x77,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'q', 
    'u',  'i',  'c',  'k',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x6c,  'Q',  'u',  'i',  'c',  'k', 
    ' ',  'S',  'c',  'e',  'n',  'e',  's',  0x68, 
    'n',  'a',  'm',  'e',  '[',  'd',  'e',  ']', 
    0x6c,  'Q',  'u',  'i',  'c',  'k',  ' ',  'S', 
    'z',  'e',  'n',  'e',  'n',  0x6f,  's',  'e', 
    'l',  'e',  'c',  't',  'a',  'b',  'l',  'e', 
    'T',  'y',  'p',  'e',  's',  0x83,  0x6a,  'Q', 
    'Q',  'u',  'i',  'c',  'k',  'I',  't',  'e', 
    'm',  0x67,  'Q',  'S',  'G',  'N',  'o',  'd', 
    'e',  0x6c,  'Q',  'Q',  'u',  'i',  'c',  'k', 
    'W',  'i',  'n',  'd',  'o',  'w',  0x65,  't', 
    'y',  'p',  'e',  's',  0x83,  0x6c,  'Q',  'Q', 
    'u',  'i',  'c',  'k',  'W',  'i',  'n',  'd', 
    'o',  'w',  0x6a,  'Q',  'Q',  'u',  'i',  'c', 
    'k',  'I',  't',  'e',  'm',  0x67,  'Q',  'S', 
    'G',  'N',  'o',  'd',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::QuickInspectorUiFactory, QuickInspectorUiFactory, qt_pluginMetaDataV2_QuickInspectorUiFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_QuickInspectorUiFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1f,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'U',  'i',  'F',  'a',  'c',  't',  'o', 
    'r',  'y', 
    // "className"
    0x03,  0x77,  'Q',  'u',  'i',  'c',  'k',  'I', 
    'n',  's',  'p',  'e',  'c',  't',  'o',  'r', 
    'U',  'i',  'F',  'a',  'c',  't',  'o',  'r', 
    'y', 
    // "MetaData"
    0x04,  0xa5,  0x62,  'i',  'd',  0x77,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'q', 
    'u',  'i',  'c',  'k',  'i',  'n',  's',  'p', 
    'e',  'c',  't',  'o',  'r',  0x64,  'n',  'a', 
    'm',  'e',  0x6c,  'Q',  'u',  'i',  'c',  'k', 
    ' ',  'S',  'c',  'e',  'n',  'e',  's',  0x68, 
    'n',  'a',  'm',  'e',  '[',  'd',  'e',  ']', 
    0x6c,  'Q',  'u',  'i',  'c',  'k',  ' ',  'S', 
    'z',  'e',  'n',  'e',  'n',  0x6f,  's',  'e', 
    'l',  'e',  'c',  't',  'a',  'b',  'l',  'e', 
    'T',  'y',  'p',  'e',  's',  0x83,  0x6a,  'Q', 
    'Q',  'u',  'i',  'c',  'k',  'I',  't',  'e', 
    'm',  0x67,  'Q',  'S',  'G',  'N',  'o',  'd', 
    'e',  0x6c,  'Q',  'Q',  'u',  'i',  'c',  'k', 
    'W',  'i',  'n',  'd',  'o',  'w',  0x65,  't', 
    'y',  'p',  'e',  's',  0x83,  0x6c,  'Q',  'Q', 
    'u',  'i',  'c',  'k',  'W',  'i',  'n',  'd', 
    'o',  'w',  0x6a,  'Q',  'Q',  'u',  'i',  'c', 
    'k',  'I',  't',  'e',  'm',  0x67,  'Q',  'S', 
    'G',  'N',  'o',  'd',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::QuickInspectorUiFactory, QuickInspectorUiFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
