/********************************************************************************
** Form generated from reading UI file 'propertyrecteditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYRECTEDITOR_H
#define UI_PROPERTYRECTEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/propertyeditor/propertydoublepaireditor.h"
#include "ui/propertyeditor/propertyintpaireditor.h"

namespace GammaRay {

class Ui_PropertyRectEditorDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *stackedWidget;
    QWidget *intPage;
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout_2;
    QLabel *label_3;
    QLabel *label_4;
    GammaRay::PropertyPointEditor *pointWidget;
    GammaRay::PropertySizeEditor *sizeWidget;
    QWidget *floatPage;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *label;
    QLabel *label_2;
    GammaRay::PropertyPointFEditor *pointFWidget;
    GammaRay::PropertySizeFEditor *sizeFWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GammaRay__PropertyRectEditorDialog)
    {
        if (GammaRay__PropertyRectEditorDialog->objectName().isEmpty())
            GammaRay__PropertyRectEditorDialog->setObjectName("GammaRay__PropertyRectEditorDialog");
        GammaRay__PropertyRectEditorDialog->resize(400, 140);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(GammaRay__PropertyRectEditorDialog->sizePolicy().hasHeightForWidth());
        GammaRay__PropertyRectEditorDialog->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(GammaRay__PropertyRectEditorDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        stackedWidget = new QStackedWidget(GammaRay__PropertyRectEditorDialog);
        stackedWidget->setObjectName("stackedWidget");
        intPage = new QWidget();
        intPage->setObjectName("intPage");
        verticalLayout = new QVBoxLayout(intPage);
        verticalLayout->setObjectName("verticalLayout");
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        label_3 = new QLabel(intPage);
        label_3->setObjectName("label_3");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_3);

        label_4 = new QLabel(intPage);
        label_4->setObjectName("label_4");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label_4);

        pointWidget = new GammaRay::PropertyPointEditor(intPage);
        pointWidget->setObjectName("pointWidget");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, pointWidget);

        sizeWidget = new GammaRay::PropertySizeEditor(intPage);
        sizeWidget->setObjectName("sizeWidget");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, sizeWidget);


        verticalLayout->addLayout(formLayout_2);

        stackedWidget->addWidget(intPage);
        floatPage = new QWidget();
        floatPage->setObjectName("floatPage");
        verticalLayout_2 = new QVBoxLayout(floatPage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(floatPage);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        label_2 = new QLabel(floatPage);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        pointFWidget = new GammaRay::PropertyPointFEditor(floatPage);
        pointFWidget->setObjectName("pointFWidget");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, pointFWidget);

        sizeFWidget = new GammaRay::PropertySizeFEditor(floatPage);
        sizeFWidget->setObjectName("sizeFWidget");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, sizeFWidget);


        verticalLayout_2->addLayout(formLayout);

        stackedWidget->addWidget(floatPage);

        verticalLayout_3->addWidget(stackedWidget);

        buttonBox = new QDialogButtonBox(GammaRay__PropertyRectEditorDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(buttonBox);


        retranslateUi(GammaRay__PropertyRectEditorDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, GammaRay__PropertyRectEditorDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GammaRay__PropertyRectEditorDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(GammaRay__PropertyRectEditorDialog);
    } // setupUi

    void retranslateUi(QDialog *GammaRay__PropertyRectEditorDialog)
    {
        label_3->setText(QCoreApplication::translate("GammaRay::PropertyRectEditorDialog", "TopLeft:", nullptr));
        label_4->setText(QCoreApplication::translate("GammaRay::PropertyRectEditorDialog", "Size", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::PropertyRectEditorDialog", "TopLeft:", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::PropertyRectEditorDialog", "Size:", nullptr));
        (void)GammaRay__PropertyRectEditorDialog;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertyRectEditorDialog: public Ui_PropertyRectEditorDialog {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTYRECTEDITOR_H
