/*
  splashscreen.h

  This file is part of GammaRay, the Qt application inspection and manipulation tool.

  SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
  Author: Milian Wolff <milian.wolff@kdab.com>

  SPDX-License-Identifier: GPL-2.0-or-later

  Contact KDAB at <info@kdab.com> for commercial licensing options.
*/

#ifndef GAMMARAY_SPLASHSCREEN_H
#define GAMMARAY_SPLASHSCREEN_H

#include "gammaray_ui_export.h"

namespace GammaRay {
GAMMARAY_UI_EXPORT void showSplashScreen();
GAMMARAY_UI_EXPORT void hideSplashScreen();
}

#endif // SPLASHSCREEN_H
