/****************************************************************************
** Meta object code from reading C++ file 'clientconnectionmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/client/clientconnectionmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'clientconnectionmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::ClientConnectionManager::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::ClientConnectionManager",
        "ready",
        "",
        "persistentConnectionError",
        "msg",
        "disconnected",
        "processTrackerBackendChanged",
        "GammaRay::ProcessTrackerBackend*",
        "backend",
        "processTrackerInfoChanged",
        "GammaRay::ProcessTrackerInfo",
        "info",
        "disconnectFromHost",
        "createMainWindow",
        "QMainWindow*",
        "handlePersistentConnectionError",
        "doConnectToHost",
        "transientConnectionError",
        "delayedHideSplashScreen",
        "hideSplashScreen",
        "targetQuitRequested",
        "updateProcessTrackerState",
        "clientConnected",
        "clientDisconnected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'persistentConnectionError'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'processTrackerBackendChanged'
        QtMocHelpers::SignalData<void(GammaRay::ProcessTrackerBackend *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'processTrackerInfoChanged'
        QtMocHelpers::SignalData<void(const GammaRay::ProcessTrackerInfo &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'disconnectFromHost'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createMainWindow'
        QtMocHelpers::SlotData<QMainWindow *()>(13, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'handlePersistentConnectionError'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'doConnectToHost'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'transientConnectionError'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'delayedHideSplashScreen'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hideSplashScreen'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'targetQuitRequested'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateProcessTrackerState'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clientConnected'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clientDisconnected'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ClientConnectionManager, qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::ClientConnectionManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t>.metaTypes,
    nullptr
} };

void GammaRay::ClientConnectionManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ClientConnectionManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->persistentConnectionError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->disconnected(); break;
        case 3: _t->processTrackerBackendChanged((*reinterpret_cast<std::add_pointer_t<GammaRay::ProcessTrackerBackend*>>(_a[1]))); break;
        case 4: _t->processTrackerInfoChanged((*reinterpret_cast<std::add_pointer_t<GammaRay::ProcessTrackerInfo>>(_a[1]))); break;
        case 5: _t->disconnectFromHost(); break;
        case 6: { QMainWindow* _r = _t->createMainWindow();
            if (_a[0]) *reinterpret_cast<QMainWindow**>(_a[0]) = std::move(_r); }  break;
        case 7: _t->handlePersistentConnectionError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->doConnectToHost(); break;
        case 9: _t->transientConnectionError(); break;
        case 10: _t->delayedHideSplashScreen(); break;
        case 11: _t->hideSplashScreen(); break;
        case 12: _t->targetQuitRequested(); break;
        case 13: _t->updateProcessTrackerState(); break;
        case 14: _t->clientConnected(); break;
        case 15: _t->clientDisconnected(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ClientConnectionManager::*)()>(_a, &ClientConnectionManager::ready, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientConnectionManager::*)(const QString & )>(_a, &ClientConnectionManager::persistentConnectionError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientConnectionManager::*)()>(_a, &ClientConnectionManager::disconnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientConnectionManager::*)(GammaRay::ProcessTrackerBackend * )>(_a, &ClientConnectionManager::processTrackerBackendChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ClientConnectionManager::*)(const GammaRay::ProcessTrackerInfo & )>(_a, &ClientConnectionManager::processTrackerInfoChanged, 4))
            return;
    }
}

const QMetaObject *GammaRay::ClientConnectionManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::ClientConnectionManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay23ClientConnectionManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::ClientConnectionManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::ClientConnectionManager::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::ClientConnectionManager::persistentConnectionError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GammaRay::ClientConnectionManager::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GammaRay::ClientConnectionManager::processTrackerBackendChanged(GammaRay::ProcessTrackerBackend * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void GammaRay::ClientConnectionManager::processTrackerInfoChanged(const GammaRay::ProcessTrackerInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
