/********************************************************************************
** Form generated from reading UI file 'positioningwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_POSITIONINGWIDGET_H
#define UI_POSITIONINGWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_PositioningWidget
{
public:
    QAction *actionLoadNMEA;
    QAction *actionCenterOn;
    QVBoxLayout *topLayout;
    QCheckBox *overrideBox;
    QGridLayout *gridLayout;
    QLabel *label;
    QDoubleSpinBox *latitude;
    QLabel *label_2;
    QDoubleSpinBox *longitude;
    QLabel *label_5;
    QDoubleSpinBox *horizontalSpeed;
    QLabel *label_4;
    QSpinBox *horizontalAccuracy;
    QLabel *label_10;
    QDateTimeEdit *timestamp;
    QSpacerItem *horizontalSpacer;
    QLabel *label_3;
    QSpinBox *altitude;
    QLabel *label_6;
    QDoubleSpinBox *verticalSpeed;
    QLabel *label_7;
    QSpinBox *verticalAccuracy;
    QLabel *label_8;
    QSpinBox *direction;
    QLabel *label_9;
    QSpinBox *magneticVariation;

    void setupUi(QWidget *GammaRay__PositioningWidget)
    {
        if (GammaRay__PositioningWidget->objectName().isEmpty())
            GammaRay__PositioningWidget->setObjectName("GammaRay__PositioningWidget");
        GammaRay__PositioningWidget->resize(1131, 150);
        actionLoadNMEA = new QAction(GammaRay__PositioningWidget);
        actionLoadNMEA->setObjectName("actionLoadNMEA");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("document-open")));
        actionLoadNMEA->setIcon(icon);
        actionCenterOn = new QAction(GammaRay__PositioningWidget);
        actionCenterOn->setObjectName("actionCenterOn");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("crosshairs")));
        actionCenterOn->setIcon(icon1);
        topLayout = new QVBoxLayout(GammaRay__PositioningWidget);
        topLayout->setObjectName("topLayout");
        overrideBox = new QCheckBox(GammaRay__PositioningWidget);
        overrideBox->setObjectName("overrideBox");

        topLayout->addWidget(overrideBox);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(GammaRay__PositioningWidget);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        latitude = new QDoubleSpinBox(GammaRay__PositioningWidget);
        latitude->setObjectName("latitude");
        latitude->setDecimals(6);
        latitude->setMinimum(-90.000000000000000);
        latitude->setMaximum(90.000000000000000);
        latitude->setSingleStep(0.500000000000000);

        gridLayout->addWidget(latitude, 0, 1, 1, 1);

        label_2 = new QLabel(GammaRay__PositioningWidget);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 2, 1, 1);

        longitude = new QDoubleSpinBox(GammaRay__PositioningWidget);
        longitude->setObjectName("longitude");
        longitude->setWrapping(true);
        longitude->setDecimals(6);
        longitude->setMinimum(-180.000000000000000);
        longitude->setMaximum(180.000000000000000);
        longitude->setSingleStep(0.500000000000000);

        gridLayout->addWidget(longitude, 0, 3, 1, 1);

        label_5 = new QLabel(GammaRay__PositioningWidget);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 0, 4, 1, 1);

        horizontalSpeed = new QDoubleSpinBox(GammaRay__PositioningWidget);
        horizontalSpeed->setObjectName("horizontalSpeed");
        horizontalSpeed->setMaximum(100.000000000000000);
        horizontalSpeed->setSingleStep(0.500000000000000);

        gridLayout->addWidget(horizontalSpeed, 0, 5, 1, 1);

        label_4 = new QLabel(GammaRay__PositioningWidget);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 0, 6, 1, 1);

        horizontalAccuracy = new QSpinBox(GammaRay__PositioningWidget);
        horizontalAccuracy->setObjectName("horizontalAccuracy");
        horizontalAccuracy->setMaximum(100000);
        horizontalAccuracy->setSingleStep(10);
        horizontalAccuracy->setValue(10);

        gridLayout->addWidget(horizontalAccuracy, 0, 7, 1, 1);

        label_10 = new QLabel(GammaRay__PositioningWidget);
        label_10->setObjectName("label_10");

        gridLayout->addWidget(label_10, 0, 8, 1, 1);

        timestamp = new QDateTimeEdit(GammaRay__PositioningWidget);
        timestamp->setObjectName("timestamp");

        gridLayout->addWidget(timestamp, 0, 9, 1, 1);

        horizontalSpacer = new QSpacerItem(17, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 10, 1, 1);

        label_3 = new QLabel(GammaRay__PositioningWidget);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        altitude = new QSpinBox(GammaRay__PositioningWidget);
        altitude->setObjectName("altitude");
        altitude->setMinimum(-500);
        altitude->setMaximum(10000);
        altitude->setSingleStep(50);

        gridLayout->addWidget(altitude, 1, 1, 1, 1);

        label_6 = new QLabel(GammaRay__PositioningWidget);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 1, 4, 1, 1);

        verticalSpeed = new QDoubleSpinBox(GammaRay__PositioningWidget);
        verticalSpeed->setObjectName("verticalSpeed");
        verticalSpeed->setMaximum(100.000000000000000);
        verticalSpeed->setSingleStep(0.500000000000000);

        gridLayout->addWidget(verticalSpeed, 1, 5, 1, 1);

        label_7 = new QLabel(GammaRay__PositioningWidget);
        label_7->setObjectName("label_7");

        gridLayout->addWidget(label_7, 1, 6, 1, 1);

        verticalAccuracy = new QSpinBox(GammaRay__PositioningWidget);
        verticalAccuracy->setObjectName("verticalAccuracy");
        verticalAccuracy->setMaximum(1000);
        verticalAccuracy->setSingleStep(10);
        verticalAccuracy->setValue(10);

        gridLayout->addWidget(verticalAccuracy, 1, 7, 1, 1);

        label_8 = new QLabel(GammaRay__PositioningWidget);
        label_8->setObjectName("label_8");

        gridLayout->addWidget(label_8, 2, 0, 1, 1);

        direction = new QSpinBox(GammaRay__PositioningWidget);
        direction->setObjectName("direction");
        direction->setWrapping(true);
        direction->setMaximum(360);
        direction->setSingleStep(5);

        gridLayout->addWidget(direction, 2, 1, 1, 1);

        label_9 = new QLabel(GammaRay__PositioningWidget);
        label_9->setObjectName("label_9");

        gridLayout->addWidget(label_9, 2, 6, 1, 1);

        magneticVariation = new QSpinBox(GammaRay__PositioningWidget);
        magneticVariation->setObjectName("magneticVariation");
        magneticVariation->setMaximum(360);

        gridLayout->addWidget(magneticVariation, 2, 7, 1, 1);


        topLayout->addLayout(gridLayout);

#if QT_CONFIG(shortcut)
        label->setBuddy(latitude);
        label_2->setBuddy(longitude);
        label_5->setBuddy(horizontalSpeed);
        label_4->setBuddy(horizontalAccuracy);
        label_10->setBuddy(timestamp);
        label_3->setBuddy(altitude);
        label_6->setBuddy(verticalSpeed);
        label_7->setBuddy(verticalAccuracy);
        label_8->setBuddy(direction);
        label_9->setBuddy(magneticVariation);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__PositioningWidget);

        QMetaObject::connectSlotsByName(GammaRay__PositioningWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__PositioningWidget)
    {
        actionLoadNMEA->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Load NMEA file...", nullptr));
#if QT_CONFIG(tooltip)
        actionLoadNMEA->setToolTip(QCoreApplication::translate("GammaRay::PositioningWidget", "Load a GPS recording from an NMEA file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCenterOn->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Center on position", nullptr));
#if QT_CONFIG(tooltip)
        actionCenterOn->setToolTip(QCoreApplication::translate("GammaRay::PositioningWidget", "Center map on current position.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionCenterOn->setShortcut(QCoreApplication::translate("GammaRay::PositioningWidget", "F3", nullptr));
#endif // QT_CONFIG(shortcut)
        overrideBox->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Override", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "&Latitude:", nullptr));
        latitude->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "\302\260", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "&Longitude:", nullptr));
        longitude->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "\302\260", nullptr));
        label_5->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Speed:", nullptr));
        horizontalSpeed->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "m/s", nullptr));
        label_4->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Acc&uracy:", nullptr));
        horizontalAccuracy->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "m", nullptr));
        label_10->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Timestamp:", nullptr));
        timestamp->setDisplayFormat(QCoreApplication::translate("GammaRay::PositioningWidget", "yyyy-MM-dd hh:mm:ss.zzz t", nullptr));
        label_3->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Al&titude:", nullptr));
        altitude->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "m", nullptr));
        label_6->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Speed:", nullptr));
        verticalSpeed->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "m/s", nullptr));
        label_7->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Accurac&y:", nullptr));
        verticalAccuracy->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "m", nullptr));
        label_8->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "&Direction:", nullptr));
        direction->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "\302\260", nullptr));
        label_9->setText(QCoreApplication::translate("GammaRay::PositioningWidget", "Ma&gnetic variation:", nullptr));
        magneticVariation->setSuffix(QCoreApplication::translate("GammaRay::PositioningWidget", "\302\260", nullptr));
        (void)GammaRay__PositioningWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PositioningWidget: public Ui_PositioningWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_POSITIONINGWIDGET_H
