/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // warning.png
  0x0,0x0,0x2,0x2d,
  0x89,
  0x50,0x4e,0x47,0xd,0xa,0x1a,0xa,0x0,0x0,0x0,0xd,0x49,0x48,0x44,0x52,0x0,
  0x0,0x0,0x10,0x0,0x0,0x0,0x10,0x8,0x6,0x0,0x0,0x0,0x1f,0xf3,0xff,0x61,
  0x0,0x0,0x0,0x4,0x73,0x42,0x49,0x54,0x8,0x8,0x8,0x8,0x7c,0x8,0x64,0x88,
  0x0,0x0,0x0,0x9,0x70,0x48,0x59,0x73,0x0,0x0,0xd,0xd7,0x0,0x0,0xd,0xd7,
  0x1,0x42,0x28,0x9b,0x78,0x0,0x0,0x0,0x19,0x74,0x45,0x58,0x74,0x53,0x6f,0x66,
  0x74,0x77,0x61,0x72,0x65,0x0,0x77,0x77,0x77,0x2e,0x69,0x6e,0x6b,0x73,0x63,0x61,
  0x70,0x65,0x2e,0x6f,0x72,0x67,0x9b,0xee,0x3c,0x1a,0x0,0x0,0x1,0xaa,0x49,0x44,
  0x41,0x54,0x38,0x8d,0xa5,0x93,0x3d,0x88,0x13,0x51,0x14,0x85,0xcf,0x7d,0x88,0x45,
  0xb0,0xb,0x9a,0x59,0x56,0x6c,0x26,0x22,0x6e,0xa1,0x30,0x2f,0x21,0xc8,0x24,0x29,
  0xc6,0x45,0xb4,0x1a,0x1b,0x7,0xb6,0x11,0xb6,0x10,0xac,0x22,0x88,0x8d,0xad,0xa5,
  0x65,0xb0,0x10,0x41,0xc4,0x3f,0x90,0x14,0xbb,0x8c,0xcb,0x14,0x6,0x4c,0x17,0xb0,
  0x98,0xb1,0x92,0xad,0x92,0x4e,0x8,0x5a,0xc8,0x66,0x60,0xe4,0x21,0xcc,0x1e,0x1b,
  0x53,0x38,0x99,0xe8,0x8a,0x7,0x5e,0xf3,0xb8,0xf7,0x3b,0xf7,0xbc,0x1f,0x21,0x29,
  0xf8,0xf,0x1d,0x2b,0xdb,0xc,0x82,0x40,0x4d,0x26,0x93,0x5d,0x11,0xb9,0xa,0x80,
  0x24,0xdf,0xb5,0xdb,0xed,0x1b,0xfd,0x7e,0xff,0x47,0xb1,0x56,0x95,0x1,0xa6,0xd3,
  0xe9,0x23,0xa5,0xd4,0x5,0x0,0x27,0x2d,0xcb,0xb2,0x94,0x52,0xe7,0xc6,0xe3,0xf1,
  0xd3,0xb2,0x5a,0x29,0x46,0x70,0x5d,0xf7,0x84,0x31,0xe6,0xab,0x88,0x34,0xe2,0x38,
  0xde,0x7,0x0,0xcf,0xf3,0xd6,0xe6,0xf3,0xf9,0xc4,0xb2,0xac,0xf5,0x28,0x8a,0xe,
  0xfe,0x18,0xc1,0x18,0xf3,0x2,0xc0,0x87,0x38,0x8e,0xf7,0x7d,0xdf,0x3f,0x4,0x80,
  0xd1,0x68,0xa4,0x1a,0x8d,0xc6,0xfb,0xd9,0x6c,0xf6,0x1c,0x80,0xbf,0x32,0x42,0xb7,
  0xdb,0x5d,0x7,0x70,0xd,0xc0,0x56,0x11,0x5c,0xab,0xd5,0x6e,0x2a,0xa5,0x36,0x3d,
  0xcf,0x5b,0x5b,0x9,0xc8,0xb2,0xec,0xd,0x80,0xb7,0x49,0x92,0x7c,0x29,0x2,0xa2,
  0x28,0x3a,0x20,0xb9,0x93,0xa6,0xe9,0xab,0x52,0x40,0xab,0xd5,0xba,0x8,0x40,0x57,
  0xab,0xd5,0xed,0x62,0xf3,0x42,0xb6,0x6d,0xdf,0x22,0xd9,0x72,0x5d,0xf7,0xec,0x12,
  0x20,0xcf,0xf3,0xd7,0x0,0x9e,0xc,0x87,0xc3,0xef,0xab,0x0,0x83,0xc1,0xc0,0x90,
  0x7c,0x66,0x8c,0x79,0xf9,0x1b,0xc0,0x71,0x9c,0x2b,0x24,0xcf,0xd8,0xb6,0x7d,0x77,
  0x55,0xf3,0x42,0xf5,0x7a,0xfd,0xe,0x80,0x8d,0x5f,0x13,0x3,0x24,0x45,0x6b,0xfd,
  0x59,0x6b,0x7d,0x8f,0xa4,0x1c,0x65,0x69,0xad,0x1f,0x38,0x8e,0xf3,0x89,0xa4,0x48,
  0xb3,0xd9,0xdc,0xce,0xf3,0xfc,0x61,0x92,0x24,0xa7,0x8a,0x6e,0x8b,0x6b,0xc,0xc3,
  0x70,0xe9,0xc1,0x69,0xad,0xbf,0x91,0xbc,0xae,0xf2,0x3c,0xbf,0x2d,0x22,0x8f,0xff,
  0x36,0x7a,0xd9,0x91,0x28,0xa5,0xee,0x2b,0x0,0x29,0xc9,0xcb,0x41,0x10,0x2c,0xb9,
  0x84,0x61,0xa8,0xca,0xdc,0x7b,0xbd,0xde,0x71,0x0,0x97,0x48,0xa6,0xd2,0xe9,0x74,
  0x4e,0x67,0x59,0xb6,0x27,0x22,0xe7,0x49,0x1e,0x1e,0xd1,0x5d,0x1,0xf8,0x58,0xa9,
  0x54,0xfc,0xa5,0xbf,0xf0,0xaf,0xfa,0x9,0x52,0x5d,0xc7,0xde,0xb3,0x4b,0xc3,0x65,
  0x0,0x0,0x0,0x0,0x49,0x45,0x4e,0x44,0xae,0x42,0x60,0x82,
  
};

static const unsigned char qt_resource_name[] = {
  // launcher
  0x0,0x8,
  0x8,0xc4,0x90,0x82,
  0x0,0x6c,
  0x0,0x61,0x0,0x75,0x0,0x6e,0x0,0x63,0x0,0x68,0x0,0x65,0x0,0x72,
    // warning.png
  0x0,0xb,
  0x0,0xb5,0x45,0xe7,
  0x0,0x77,
  0x0,0x61,0x0,0x72,0x0,0x6e,0x0,0x69,0x0,0x6e,0x0,0x67,0x0,0x2e,0x0,0x70,0x0,0x6e,0x0,0x67,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9d,0x1b,0x43,0x71,0x50,
  // :/launcher
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9d,0x1b,0x43,0x71,0x50,
  // :/launcher/warning.png
  0x0,0x0,0x0,0x16,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9d,0x1b,0x43,0x71,0x50,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_ui)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_ui)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_ui)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_ui)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_ui)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_ui)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
