/********************************************************************************
** Form generated from reading UI file 'launcherwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAUNCHERWINDOW_H
#define UI_LAUNCHERWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/aboutwidget.h>
#include "attachdialog.h"
#include "connectpage.h"
#include "launchpage.h"
#include "promolabel.h"
#include "selftestpage.h"

namespace GammaRay {

class Ui_LauncherWindow
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    GammaRay::AttachDialog *attachPage;
    GammaRay::LaunchPage *launchPage;
    GammaRay::ConnectPage *connectPage;
    GammaRay::SelfTestPage *selfTestPage;
    GammaRay::AboutWidget *aboutPage;
    QWidget *widget;
    QHBoxLayout *horizontalLayout;
    GammaRay::PromoLabel *promoLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GammaRay__LauncherWindow)
    {
        if (GammaRay__LauncherWindow->objectName().isEmpty())
            GammaRay__LauncherWindow->setObjectName("GammaRay__LauncherWindow");
        GammaRay__LauncherWindow->resize(748, 482);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/gammaray/GammaRay-128x128.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        GammaRay__LauncherWindow->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(GammaRay__LauncherWindow);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(GammaRay__LauncherWindow);
        tabWidget->setObjectName("tabWidget");
        attachPage = new GammaRay::AttachDialog();
        attachPage->setObjectName("attachPage");
        tabWidget->addTab(attachPage, QString());
        launchPage = new GammaRay::LaunchPage();
        launchPage->setObjectName("launchPage");
        tabWidget->addTab(launchPage, QString());
        connectPage = new GammaRay::ConnectPage();
        connectPage->setObjectName("connectPage");
        tabWidget->addTab(connectPage, QString());
        selfTestPage = new GammaRay::SelfTestPage();
        selfTestPage->setObjectName("selfTestPage");
        tabWidget->addTab(selfTestPage, QString());
        aboutPage = new GammaRay::AboutWidget();
        aboutPage->setObjectName("aboutPage");
        tabWidget->addTab(aboutPage, QString());

        verticalLayout->addWidget(tabWidget);

        widget = new QWidget(GammaRay__LauncherWindow);
        widget->setObjectName("widget");
        horizontalLayout = new QHBoxLayout(widget);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        promoLabel = new GammaRay::PromoLabel(widget);
        promoLabel->setObjectName("promoLabel");

        horizontalLayout->addWidget(promoLabel);

        buttonBox = new QDialogButtonBox(widget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addWidget(widget);


        retranslateUi(GammaRay__LauncherWindow);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, GammaRay__LauncherWindow, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GammaRay__LauncherWindow, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(3);


        QMetaObject::connectSlotsByName(GammaRay__LauncherWindow);
    } // setupUi

    void retranslateUi(QDialog *GammaRay__LauncherWindow)
    {
        tabWidget->setTabText(tabWidget->indexOf(attachPage), QCoreApplication::translate("GammaRay::LauncherWindow", "Attach", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(launchPage), QCoreApplication::translate("GammaRay::LauncherWindow", "Launch", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(connectPage), QCoreApplication::translate("GammaRay::LauncherWindow", "Connect", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(selfTestPage), QCoreApplication::translate("GammaRay::LauncherWindow", "Self Test", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(aboutPage), QCoreApplication::translate("GammaRay::LauncherWindow", "About", nullptr));
        (void)GammaRay__LauncherWindow;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class LauncherWindow: public Ui_LauncherWindow {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_LAUNCHERWINDOW_H
