/****************************************************************************
** Meta object code from reading C++ file 'statemachineviewerinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/statemachineviewer/statemachineviewerinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'statemachineviewerinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::StateMachineViewerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::StateMachineViewerInterface",
        "statusChanged",
        "",
        "haveStateMachine",
        "running",
        "message",
        "aboutToRepopulateGraph",
        "graphRepopulated",
        "stateConfigurationChanged",
        "GammaRay::StateMachineConfiguration",
        "config",
        "maximumDepthChanged",
        "depth",
        "transitionTriggered",
        "GammaRay::TransitionId",
        "transition",
        "label",
        "stateAdded",
        "GammaRay::StateId",
        "state",
        "parent",
        "hasChildren",
        "GammaRay::StateType",
        "type",
        "connectToInitial",
        "stateEntered",
        "stateExited",
        "transitionAdded",
        "source",
        "target",
        "selectStateMachine",
        "index",
        "toggleRunning",
        "repopulateGraph"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(bool, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'message'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'aboutToRepopulateGraph'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'graphRepopulated'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateConfigurationChanged'
        QtMocHelpers::SignalData<void(const GammaRay::StateMachineConfiguration &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'maximumDepthChanged'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'transitionTriggered'
        QtMocHelpers::SignalData<void(GammaRay::TransitionId, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 },
        }}),
        // Signal 'stateAdded'
        QtMocHelpers::SignalData<void(GammaRay::StateId, GammaRay::StateId, bool, const QString &, GammaRay::StateType, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 18, 20 }, { QMetaType::Bool, 21 }, { QMetaType::QString, 16 },
            { 0x80000000 | 22, 23 }, { QMetaType::Bool, 24 },
        }}),
        // Signal 'stateEntered'
        QtMocHelpers::SignalData<void(GammaRay::StateId)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'stateExited'
        QtMocHelpers::SignalData<void(GammaRay::StateId)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'transitionAdded'
        QtMocHelpers::SignalData<void(GammaRay::TransitionId, GammaRay::StateId, GammaRay::StateId, const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 19 }, { 0x80000000 | 18, 28 }, { 0x80000000 | 18, 29 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'selectStateMachine'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Slot 'toggleRunning'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'repopulateGraph'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateMachineViewerInterface, qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::StateMachineViewerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t>.metaTypes,
    nullptr
} };

void GammaRay::StateMachineViewerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateMachineViewerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->message((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->aboutToRepopulateGraph(); break;
        case 3: _t->graphRepopulated(); break;
        case 4: _t->stateConfigurationChanged((*reinterpret_cast<std::add_pointer_t<GammaRay::StateMachineConfiguration>>(_a[1]))); break;
        case 5: _t->maximumDepthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->transitionTriggered((*reinterpret_cast<std::add_pointer_t<GammaRay::TransitionId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->stateAdded((*reinterpret_cast<std::add_pointer_t<GammaRay::StateId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GammaRay::StateId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<GammaRay::StateType>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 8: _t->stateEntered((*reinterpret_cast<std::add_pointer_t<GammaRay::StateId>>(_a[1]))); break;
        case 9: _t->stateExited((*reinterpret_cast<std::add_pointer_t<GammaRay::StateId>>(_a[1]))); break;
        case 10: _t->transitionAdded((*reinterpret_cast<std::add_pointer_t<GammaRay::TransitionId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GammaRay::StateId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<GammaRay::StateId>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 11: _t->selectStateMachine((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->toggleRunning(); break;
        case 13: _t->repopulateGraph(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateMachineConfiguration >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::TransitionId >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateId >(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateType >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateId >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateId >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::StateId >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::TransitionId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(bool , bool )>(_a, &StateMachineViewerInterface::statusChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(const QString & )>(_a, &StateMachineViewerInterface::message, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)()>(_a, &StateMachineViewerInterface::aboutToRepopulateGraph, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)()>(_a, &StateMachineViewerInterface::graphRepopulated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(const GammaRay::StateMachineConfiguration & )>(_a, &StateMachineViewerInterface::stateConfigurationChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(int )>(_a, &StateMachineViewerInterface::maximumDepthChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(GammaRay::TransitionId , const QString & )>(_a, &StateMachineViewerInterface::transitionTriggered, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(GammaRay::StateId , GammaRay::StateId , bool , const QString & , GammaRay::StateType , bool )>(_a, &StateMachineViewerInterface::stateAdded, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(GammaRay::StateId )>(_a, &StateMachineViewerInterface::stateEntered, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(GammaRay::StateId )>(_a, &StateMachineViewerInterface::stateExited, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateMachineViewerInterface::*)(GammaRay::TransitionId , GammaRay::StateId , GammaRay::StateId , const QString & )>(_a, &StateMachineViewerInterface::transitionAdded, 10))
            return;
    }
}

const QMetaObject *GammaRay::StateMachineViewerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::StateMachineViewerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay27StateMachineViewerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::StateMachineViewerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::StateMachineViewerInterface::statusChanged(bool _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void GammaRay::StateMachineViewerInterface::message(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GammaRay::StateMachineViewerInterface::aboutToRepopulateGraph()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GammaRay::StateMachineViewerInterface::graphRepopulated()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GammaRay::StateMachineViewerInterface::stateConfigurationChanged(const GammaRay::StateMachineConfiguration & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void GammaRay::StateMachineViewerInterface::maximumDepthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void GammaRay::StateMachineViewerInterface::transitionTriggered(GammaRay::TransitionId _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void GammaRay::StateMachineViewerInterface::stateAdded(GammaRay::StateId _t1, GammaRay::StateId _t2, bool _t3, const QString & _t4, GammaRay::StateType _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 8
void GammaRay::StateMachineViewerInterface::stateEntered(GammaRay::StateId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void GammaRay::StateMachineViewerInterface::stateExited(GammaRay::StateId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void GammaRay::StateMachineViewerInterface::transitionAdded(GammaRay::TransitionId _t1, GammaRay::StateId _t2, GammaRay::StateId _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
