/********************************************************************************
** Form generated from reading UI file 'propertymatrixdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYMATRIXDIALOG_H
#define UI_PROPERTYMATRIXDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>

namespace GammaRay {

class Ui_PropertyMatrixDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTableView *tableView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GammaRay__PropertyMatrixDialog)
    {
        if (GammaRay__PropertyMatrixDialog->objectName().isEmpty())
            GammaRay__PropertyMatrixDialog->setObjectName("GammaRay__PropertyMatrixDialog");
        GammaRay__PropertyMatrixDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__PropertyMatrixDialog);
        verticalLayout->setObjectName("verticalLayout");
        tableView = new QTableView(GammaRay__PropertyMatrixDialog);
        tableView->setObjectName("tableView");

        verticalLayout->addWidget(tableView);

        buttonBox = new QDialogButtonBox(GammaRay__PropertyMatrixDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(GammaRay__PropertyMatrixDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, GammaRay__PropertyMatrixDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GammaRay__PropertyMatrixDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(GammaRay__PropertyMatrixDialog);
    } // setupUi

    void retranslateUi(QDialog *GammaRay__PropertyMatrixDialog)
    {
        (void)GammaRay__PropertyMatrixDialog;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertyMatrixDialog: public Ui_PropertyMatrixDialog {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTYMATRIXDIALOG_H
