/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/ui/mainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::MainWindowUIStateManager::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::MainWindowUIStateManager"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindowUIStateManager, qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::MainWindowUIStateManager::staticMetaObject = { {
    QMetaObject::SuperData::link<UIStateManager::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t>.metaTypes,
    nullptr
} };

void GammaRay::MainWindowUIStateManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindowUIStateManager *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::MainWindowUIStateManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::MainWindowUIStateManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay24MainWindowUIStateManagerE_t>.strings))
        return static_cast<void*>(this);
    return UIStateManager::qt_metacast(_clname);
}

int GammaRay::MainWindowUIStateManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UIStateManager::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::MainWindow",
        "targetQuitRequested",
        "",
        "help",
        "configureFeedback",
        "about",
        "aboutPlugins",
        "aboutKDAB",
        "showMessageStatistics",
        "toolSelected",
        "selectTool",
        "id",
        "toolContextMenu",
        "QPoint",
        "pos",
        "quitHost",
        "detachProbe",
        "navigateToCode",
        "QUrl",
        "url",
        "lineNumber",
        "columnNumber",
        "logTransmissionRate",
        "bytesRead",
        "bytesWritten",
        "setCodeNavigationIDE",
        "QAction*",
        "action",
        "saveTargetState",
        "QSettings*",
        "settings",
        "restoreTargetState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'targetQuitRequested'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'help'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'configureFeedback'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'about'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aboutPlugins'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aboutKDAB'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showMessageStatistics'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toolSelected'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectTool'
        QtMocHelpers::SlotData<bool(const QString &)>(10, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'toolContextMenu'
        QtMocHelpers::SlotData<void(QPoint)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'quitHost'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'detachProbe'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'navigateToCode'
        QtMocHelpers::SlotData<void(const QUrl &, int, int)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'logTransmissionRate'
        QtMocHelpers::SlotData<void(quint64, quint64)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::ULongLong, 23 }, { QMetaType::ULongLong, 24 },
        }}),
        // Slot 'setCodeNavigationIDE'
        QtMocHelpers::SlotData<void(QAction *)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'saveTargetState'
        QtMocHelpers::MethodData<void(QSettings *) const>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Method 'restoreTargetState'
        QtMocHelpers::MethodData<void(QSettings *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN8GammaRay10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay10MainWindowE_t>.metaTypes,
    nullptr
} };

void GammaRay::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->targetQuitRequested(); break;
        case 1: _t->help(); break;
        case 2: _t->configureFeedback(); break;
        case 3: _t->about(); break;
        case 4: _t->aboutPlugins(); break;
        case 5: _t->aboutKDAB(); break;
        case 6: _t->showMessageStatistics(); break;
        case 7: _t->toolSelected(); break;
        case 8: { bool _r = _t->selectTool((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->toolContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 10: _t->quitHost(); break;
        case 11: _t->detachProbe(); break;
        case 12: _t->navigateToCode((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 13: _t->logTransmissionRate((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 14: _t->setCodeNavigationIDE((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 15: _t->saveTargetState((*reinterpret_cast<std::add_pointer_t<QSettings*>>(_a[1]))); break;
        case 16: _t->restoreTargetState((*reinterpret_cast<std::add_pointer_t<QSettings*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::targetQuitRequested, 0))
            return;
    }
}

const QMetaObject *GammaRay::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int GammaRay::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::MainWindow::targetQuitRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
