/********************************************************************************
** Form generated from reading UI file 'propertiestab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESTAB_H
#define UI_PROPERTIESTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_PropertiesTab
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *propertySearchLine;
    GammaRay::DeferredTreeView *propertyView;
    QWidget *horizontalWidget;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QLineEdit *newPropertyName;
    QWidget *newPropertyBar;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QComboBox *newPropertyType;
    QLabel *newPropertyValueLabel;
    QPushButton *newPropertyButton;

    void setupUi(QWidget *GammaRay__PropertiesTab)
    {
        if (GammaRay__PropertiesTab->objectName().isEmpty())
            GammaRay__PropertiesTab->setObjectName("GammaRay__PropertiesTab");
        GammaRay__PropertiesTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__PropertiesTab);
        verticalLayout->setObjectName("verticalLayout");
        propertySearchLine = new QLineEdit(GammaRay__PropertiesTab);
        propertySearchLine->setObjectName("propertySearchLine");

        verticalLayout->addWidget(propertySearchLine);

        propertyView = new GammaRay::DeferredTreeView(GammaRay__PropertiesTab);
        propertyView->setObjectName("propertyView");
        propertyView->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
        propertyView->setRootIsDecorated(false);

        verticalLayout->addWidget(propertyView);

        horizontalWidget = new QWidget(GammaRay__PropertiesTab);
        horizontalWidget->setObjectName("horizontalWidget");
        horizontalLayout_2 = new QHBoxLayout(horizontalWidget);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(horizontalWidget);
        label->setObjectName("label");
        label->setWordWrap(true);

        horizontalLayout_2->addWidget(label);

        newPropertyName = new QLineEdit(horizontalWidget);
        newPropertyName->setObjectName("newPropertyName");

        horizontalLayout_2->addWidget(newPropertyName);


        verticalLayout->addWidget(horizontalWidget);

        newPropertyBar = new QWidget(GammaRay__PropertiesTab);
        newPropertyBar->setObjectName("newPropertyBar");
        horizontalLayout = new QHBoxLayout(newPropertyBar);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(newPropertyBar);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(label_2);

        newPropertyType = new QComboBox(newPropertyBar);
        newPropertyType->setObjectName("newPropertyType");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(newPropertyType->sizePolicy().hasHeightForWidth());
        newPropertyType->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(newPropertyType);

        newPropertyValueLabel = new QLabel(newPropertyBar);
        newPropertyValueLabel->setObjectName("newPropertyValueLabel");
        sizePolicy.setHeightForWidth(newPropertyValueLabel->sizePolicy().hasHeightForWidth());
        newPropertyValueLabel->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(newPropertyValueLabel);


        verticalLayout->addWidget(newPropertyBar);

        newPropertyButton = new QPushButton(GammaRay__PropertiesTab);
        newPropertyButton->setObjectName("newPropertyButton");

        verticalLayout->addWidget(newPropertyButton);

#if QT_CONFIG(shortcut)
        label->setBuddy(newPropertyName);
        label_2->setBuddy(newPropertyType);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__PropertiesTab);

        QMetaObject::connectSlotsByName(GammaRay__PropertiesTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__PropertiesTab)
    {
        label->setText(QCoreApplication::translate("GammaRay::PropertiesTab", "&New Dynamic Property:", nullptr));
        newPropertyName->setPlaceholderText(QCoreApplication::translate("GammaRay::PropertiesTab", "Name", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::PropertiesTab", "&Type:", nullptr));
        newPropertyValueLabel->setText(QCoreApplication::translate("GammaRay::PropertiesTab", "&Value:", nullptr));
        newPropertyButton->setText(QCoreApplication::translate("GammaRay::PropertiesTab", "Add", nullptr));
        (void)GammaRay__PropertiesTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertiesTab: public Ui_PropertiesTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTIESTAB_H
