/********************************************************************************
** Form generated from reading UI file 'methodstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_METHODSTAB_H
#define UI_METHODSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_MethodsTab
{
public:
    QVBoxLayout *verticalLayout_5;
    QLineEdit *methodSearchLine;
    GammaRay::DeferredTreeView *methodView;
    GammaRay::DeferredTreeView *methodLog;

    void setupUi(QWidget *GammaRay__MethodsTab)
    {
        if (GammaRay__MethodsTab->objectName().isEmpty())
            GammaRay__MethodsTab->setObjectName("GammaRay__MethodsTab");
        GammaRay__MethodsTab->resize(400, 300);
        verticalLayout_5 = new QVBoxLayout(GammaRay__MethodsTab);
        verticalLayout_5->setObjectName("verticalLayout_5");
        methodSearchLine = new QLineEdit(GammaRay__MethodsTab);
        methodSearchLine->setObjectName("methodSearchLine");

        verticalLayout_5->addWidget(methodSearchLine);

        methodView = new GammaRay::DeferredTreeView(GammaRay__MethodsTab);
        methodView->setObjectName("methodView");
        methodView->setContextMenuPolicy(Qt::CustomContextMenu);
        methodView->setRootIsDecorated(false);

        verticalLayout_5->addWidget(methodView);

        methodLog = new GammaRay::DeferredTreeView(GammaRay__MethodsTab);
        methodLog->setObjectName("methodLog");
        methodLog->setRootIsDecorated(false);
        methodLog->setUniformRowHeights(true);
        methodLog->header()->setVisible(false);

        verticalLayout_5->addWidget(methodLog);

        verticalLayout_5->setStretch(1, 4);
        verticalLayout_5->setStretch(2, 1);

        retranslateUi(GammaRay__MethodsTab);

        QMetaObject::connectSlotsByName(GammaRay__MethodsTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__MethodsTab)
    {
        (void)GammaRay__MethodsTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class MethodsTab: public Ui_MethodsTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_METHODSTAB_H
