/****************************************************************************
** Meta object code from reading C++ file 'bluetooth.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/bluetooth/bluetooth.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetooth.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay9BluetoothE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::Bluetooth::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay9BluetoothE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::Bluetooth"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Bluetooth, qt_meta_tag_ZN8GammaRay9BluetoothE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::Bluetooth::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay9BluetoothE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay9BluetoothE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay9BluetoothE_t>.metaTypes,
    nullptr
} };

void GammaRay::Bluetooth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Bluetooth *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::Bluetooth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::Bluetooth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay9BluetoothE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::Bluetooth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::BluetoothFactory::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::BluetoothFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BluetoothFactory, qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::BluetoothFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t>.metaTypes,
    nullptr
} };

void GammaRay::BluetoothFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BluetoothFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *GammaRay::BluetoothFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::BluetoothFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16BluetoothFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StandardToolFactory<QObject,Bluetooth>"))
        return static_cast< StandardToolFactory<QObject,Bluetooth>*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.ToolFactory/1.0"))
        return static_cast< GammaRay::ToolFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::BluetoothFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace GammaRay;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_BluetoothFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x70,  'B',  'l',  'u',  'e',  't',  'o', 
    'o',  't',  'h',  'F',  'a',  'c',  't',  'o', 
    'r',  'y', 
    // "MetaData"
    0x04,  0xa4,  0x66,  'h',  'i',  'd',  'd',  'e', 
    'n',  0xf5,  0x62,  'i',  'd',  0x72,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'b', 
    'l',  'u',  'e',  't',  'o',  'o',  't',  'h', 
    0x64,  'n',  'a',  'm',  'e',  0x69,  'B',  'l', 
    'u',  'e',  't',  'o',  'o',  't',  'h',  0x65, 
    't',  'y',  'p',  'e',  's',  0x84,  0x78,  0x1e, 
    'Q',  'B',  'l',  'u',  'e',  't',  'o',  'o', 
    't',  'h',  'D',  'e',  'v',  'i',  'c',  'e', 
    'D',  'i',  's',  'c',  'o',  'v',  'e',  'r', 
    'y',  'A',  'g',  'e',  'n',  't',  0x75,  'Q', 
    'B',  'l',  'u',  'e',  't',  'o',  'o',  't', 
    'h',  'L',  'o',  'c',  'a',  'l',  'D',  'e', 
    'v',  'i',  'c',  'e',  0x78,  0x1f,  'Q',  'B', 
    'l',  'u',  'e',  't',  'o',  'o',  't',  'h', 
    'S',  'e',  'r',  'v',  'i',  'c',  'e',  'D', 
    'i',  's',  'c',  'o',  'v',  'e',  'r',  'y', 
    'A',  'g',  'e',  'n',  't',  0x70,  'Q',  'B', 
    'l',  'u',  'e',  't',  'o',  'o',  't',  'h', 
    'S',  'o',  'c',  'k',  'e',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(GammaRay::BluetoothFactory, BluetoothFactory, qt_pluginMetaDataV2_BluetoothFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_BluetoothFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1d,  'c',  'o',  'm',  '.',  'k', 
    'd',  'a',  'b',  '.',  'G',  'a',  'm',  'm', 
    'a',  'R',  'a',  'y',  '.',  'T',  'o',  'o', 
    'l',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x70,  'B',  'l',  'u',  'e',  't',  'o', 
    'o',  't',  'h',  'F',  'a',  'c',  't',  'o', 
    'r',  'y', 
    // "MetaData"
    0x04,  0xa4,  0x66,  'h',  'i',  'd',  'd',  'e', 
    'n',  0xf5,  0x62,  'i',  'd',  0x72,  'g',  'a', 
    'm',  'm',  'a',  'r',  'a',  'y',  '_',  'b', 
    'l',  'u',  'e',  't',  'o',  'o',  't',  'h', 
    0x64,  'n',  'a',  'm',  'e',  0x69,  'B',  'l', 
    'u',  'e',  't',  'o',  'o',  't',  'h',  0x65, 
    't',  'y',  'p',  'e',  's',  0x84,  0x78,  0x1e, 
    'Q',  'B',  'l',  'u',  'e',  't',  'o',  'o', 
    't',  'h',  'D',  'e',  'v',  'i',  'c',  'e', 
    'D',  'i',  's',  'c',  'o',  'v',  'e',  'r', 
    'y',  'A',  'g',  'e',  'n',  't',  0x75,  'Q', 
    'B',  'l',  'u',  'e',  't',  'o',  'o',  't', 
    'h',  'L',  'o',  'c',  'a',  'l',  'D',  'e', 
    'v',  'i',  'c',  'e',  0x78,  0x1f,  'Q',  'B', 
    'l',  'u',  'e',  't',  'o',  'o',  't',  'h', 
    'S',  'e',  'r',  'v',  'i',  'c',  'e',  'D', 
    'i',  's',  'c',  'o',  'v',  'e',  'r',  'y', 
    'A',  'g',  'e',  'n',  't',  0x70,  'Q',  'B', 
    'l',  'u',  'e',  't',  'o',  'o',  't',  'h', 
    'S',  'o',  'c',  'k',  'e',  't', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(GammaRay::BluetoothFactory, BluetoothFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
