/********************************************************************************
** Form generated from reading UI file 'sggeometrytab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SGGEOMETRYTAB_H
#define UI_SGGEOMETRYTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <plugins/quickinspector/geometryextension/sgwireframewidget.h>

namespace GammaRay {

class Ui_SGGeometryTab
{
public:
    QVBoxLayout *verticalLayout1;
    QSplitter *splitter_6;
    QWidget *verticalLayoutWidget1;
    QVBoxLayout *verticalLayout2;
    QLabel *label;
    QTableView *tableView;
    QWidget *verticalLayoutWidget2;
    QVBoxLayout *verticalLayout3;
    QLabel *label_2;
    SGWireframeWidget *wireframeWidget;

    void setupUi(QWidget *GammaRay__SGGeometryTab)
    {
        if (GammaRay__SGGeometryTab->objectName().isEmpty())
            GammaRay__SGGeometryTab->setObjectName("GammaRay__SGGeometryTab");
        GammaRay__SGGeometryTab->resize(561, 362);
        verticalLayout1 = new QVBoxLayout(GammaRay__SGGeometryTab);
        verticalLayout1->setObjectName("verticalLayout1");
        splitter_6 = new QSplitter(GammaRay__SGGeometryTab);
        splitter_6->setObjectName("splitter_6");
        splitter_6->setOrientation(Qt::Horizontal);
        verticalLayoutWidget1 = new QWidget(splitter_6);
        verticalLayoutWidget1->setObjectName("verticalLayoutWidget1");
        verticalLayout2 = new QVBoxLayout(verticalLayoutWidget1);
        verticalLayout2->setObjectName("verticalLayout2");
        verticalLayout2->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(verticalLayoutWidget1);
        label->setObjectName("label");

        verticalLayout2->addWidget(label);

        tableView = new QTableView(verticalLayoutWidget1);
        tableView->setObjectName("tableView");

        verticalLayout2->addWidget(tableView);

        splitter_6->addWidget(verticalLayoutWidget1);
        verticalLayoutWidget2 = new QWidget(splitter_6);
        verticalLayoutWidget2->setObjectName("verticalLayoutWidget2");
        verticalLayout3 = new QVBoxLayout(verticalLayoutWidget2);
        verticalLayout3->setObjectName("verticalLayout3");
        verticalLayout3->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(verticalLayoutWidget2);
        label_2->setObjectName("label_2");

        verticalLayout3->addWidget(label_2);

        wireframeWidget = new SGWireframeWidget(verticalLayoutWidget2);
        wireframeWidget->setObjectName("wireframeWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(wireframeWidget->sizePolicy().hasHeightForWidth());
        wireframeWidget->setSizePolicy(sizePolicy);

        verticalLayout3->addWidget(wireframeWidget);

        splitter_6->addWidget(verticalLayoutWidget2);

        verticalLayout1->addWidget(splitter_6);


        retranslateUi(GammaRay__SGGeometryTab);

        QMetaObject::connectSlotsByName(GammaRay__SGGeometryTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__SGGeometryTab)
    {
        label->setText(QCoreApplication::translate("GammaRay::SGGeometryTab", "Raw Vertex Data", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::SGGeometryTab", "Preview", nullptr));
        (void)GammaRay__SGGeometryTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class SGGeometryTab: public Ui_SGGeometryTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_SGGEOMETRYTAB_H
