/********************************************************************************
** Form generated from reading UI file 'methodinvocationdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_METHODINVOCATIONDIALOG_H
#define UI_METHODINVOCATIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_MethodInvocationDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *connectionTypeComboBox;
    GammaRay::DeferredTreeView *argumentView;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *GammaRay__MethodInvocationDialog)
    {
        if (GammaRay__MethodInvocationDialog->objectName().isEmpty())
            GammaRay__MethodInvocationDialog->setObjectName("GammaRay__MethodInvocationDialog");
        GammaRay__MethodInvocationDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__MethodInvocationDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(GammaRay__MethodInvocationDialog);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        connectionTypeComboBox = new QComboBox(GammaRay__MethodInvocationDialog);
        connectionTypeComboBox->setObjectName("connectionTypeComboBox");

        horizontalLayout->addWidget(connectionTypeComboBox);


        verticalLayout->addLayout(horizontalLayout);

        argumentView = new GammaRay::DeferredTreeView(GammaRay__MethodInvocationDialog);
        argumentView->setObjectName("argumentView");
        argumentView->setRootIsDecorated(false);
        argumentView->header()->setStretchLastSection(false);

        verticalLayout->addWidget(argumentView);

        buttonBox = new QDialogButtonBox(GammaRay__MethodInvocationDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(connectionTypeComboBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__MethodInvocationDialog);

        QMetaObject::connectSlotsByName(GammaRay__MethodInvocationDialog);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__MethodInvocationDialog)
    {
        label->setText(QCoreApplication::translate("GammaRay::MethodInvocationDialog", "&Connection type:", nullptr));
        (void)GammaRay__MethodInvocationDialog;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class MethodInvocationDialog: public Ui_MethodInvocationDialog {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_METHODINVOCATIONDIALOG_H
