/********************************************************************************
** Form generated from reading UI file 'propertytexteditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYTEXTEDITOR_H
#define UI_PROPERTYTEXTEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <ui/codeeditor/codeeditor.h>

namespace GammaRay {

class Ui_PropertyTextEditorDialog
{
public:
    QVBoxLayout *verticalLayout;
    GammaRay::CodeEditor *plainTextEdit;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *modeButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GammaRay__PropertyTextEditorDialog)
    {
        if (GammaRay__PropertyTextEditorDialog->objectName().isEmpty())
            GammaRay__PropertyTextEditorDialog->setObjectName("GammaRay__PropertyTextEditorDialog");
        GammaRay__PropertyTextEditorDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__PropertyTextEditorDialog);
        verticalLayout->setObjectName("verticalLayout");
        plainTextEdit = new GammaRay::CodeEditor(GammaRay__PropertyTextEditorDialog);
        plainTextEdit->setObjectName("plainTextEdit");
        plainTextEdit->setLineWrapMode(QPlainTextEdit::NoWrap);

        verticalLayout->addWidget(plainTextEdit);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        modeButton = new QPushButton(GammaRay__PropertyTextEditorDialog);
        modeButton->setObjectName("modeButton");

        horizontalLayout_2->addWidget(modeButton);

        buttonBox = new QDialogButtonBox(GammaRay__PropertyTextEditorDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Save);

        horizontalLayout_2->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(GammaRay__PropertyTextEditorDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, GammaRay__PropertyTextEditorDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GammaRay__PropertyTextEditorDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(GammaRay__PropertyTextEditorDialog);
    } // setupUi

    void retranslateUi(QDialog *GammaRay__PropertyTextEditorDialog)
    {
        modeButton->setText(QCoreApplication::translate("GammaRay::PropertyTextEditorDialog", "Switch to Hex Mode", nullptr));
#if QT_CONFIG(tooltip)
        buttonBox->setToolTip(QCoreApplication::translate("GammaRay::PropertyTextEditorDialog", "Unsaved changes will be lost", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)GammaRay__PropertyTextEditorDialog;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertyTextEditorDialog: public Ui_PropertyTextEditorDialog {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTYTEXTEDITOR_H
