/********************************************************************************
** Form generated from reading UI file 'networkconfigurationwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NETWORKCONFIGURATIONWIDGET_H
#define UI_NETWORKCONFIGURATIONWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_NetworkConfigurationWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *searchLine;
    QTreeView *networkConfigView;

    void setupUi(QWidget *GammaRay__NetworkConfigurationWidget)
    {
        if (GammaRay__NetworkConfigurationWidget->objectName().isEmpty())
            GammaRay__NetworkConfigurationWidget->setObjectName("GammaRay__NetworkConfigurationWidget");
        GammaRay__NetworkConfigurationWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__NetworkConfigurationWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        searchLine = new QLineEdit(GammaRay__NetworkConfigurationWidget);
        searchLine->setObjectName("searchLine");

        verticalLayout->addWidget(searchLine);

        networkConfigView = new QTreeView(GammaRay__NetworkConfigurationWidget);
        networkConfigView->setObjectName("networkConfigView");
        networkConfigView->setRootIsDecorated(false);
        networkConfigView->setUniformRowHeights(true);

        verticalLayout->addWidget(networkConfigView);


        retranslateUi(GammaRay__NetworkConfigurationWidget);

        QMetaObject::connectSlotsByName(GammaRay__NetworkConfigurationWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__NetworkConfigurationWidget)
    {
        (void)GammaRay__NetworkConfigurationWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class NetworkConfigurationWidget: public Ui_NetworkConfigurationWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_NETWORKCONFIGURATIONWIDGET_H
