/********************************************************************************
** Form generated from reading UI file 'cookietab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COOKIETAB_H
#define UI_COOKIETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_CookieTab
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *cookieJarView;

    void setupUi(QWidget *GammaRay__CookieTab)
    {
        if (GammaRay__CookieTab->objectName().isEmpty())
            GammaRay__CookieTab->setObjectName("GammaRay__CookieTab");
        GammaRay__CookieTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__CookieTab);
        verticalLayout->setObjectName("verticalLayout");
        cookieJarView = new QTreeView(GammaRay__CookieTab);
        cookieJarView->setObjectName("cookieJarView");
        cookieJarView->setRootIsDecorated(false);
        cookieJarView->setUniformRowHeights(true);

        verticalLayout->addWidget(cookieJarView);


        retranslateUi(GammaRay__CookieTab);

        QMetaObject::connectSlotsByName(GammaRay__CookieTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__CookieTab)
    {
        (void)GammaRay__CookieTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class CookieTab: public Ui_CookieTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_COOKIETAB_H
