/********************************************************************************
** Form generated from reading UI file 'materialtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MATERIALTAB_H
#define UI_MATERIALTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/codeeditor/codeeditor.h>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_MaterialTab
{
public:
    QHBoxLayout *horizontalLayout_2;
    QSplitter *splitter;
    GammaRay::DeferredTreeView *materialPropertyView;
    QWidget *widget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *shaderList;
    GammaRay::CodeEditor *shaderEdit;

    void setupUi(QWidget *GammaRay__MaterialTab)
    {
        if (GammaRay__MaterialTab->objectName().isEmpty())
            GammaRay__MaterialTab->setObjectName("GammaRay__MaterialTab");
        GammaRay__MaterialTab->resize(727, 544);
        horizontalLayout_2 = new QHBoxLayout(GammaRay__MaterialTab);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        splitter = new QSplitter(GammaRay__MaterialTab);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        splitter->setChildrenCollapsible(false);
        materialPropertyView = new GammaRay::DeferredTreeView(splitter);
        materialPropertyView->setObjectName("materialPropertyView");
        materialPropertyView->setContextMenuPolicy(Qt::CustomContextMenu);
        splitter->addWidget(materialPropertyView);
        widget = new QWidget(splitter);
        widget->setObjectName("widget");
        verticalLayout = new QVBoxLayout(widget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(widget);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        shaderList = new QComboBox(widget);
        shaderList->setObjectName("shaderList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(shaderList->sizePolicy().hasHeightForWidth());
        shaderList->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(shaderList);


        verticalLayout->addLayout(horizontalLayout);

        shaderEdit = new GammaRay::CodeEditor(widget);
        shaderEdit->setObjectName("shaderEdit");
        shaderEdit->setReadOnly(true);

        verticalLayout->addWidget(shaderEdit);

        splitter->addWidget(widget);

        horizontalLayout_2->addWidget(splitter);

#if QT_CONFIG(shortcut)
        label->setBuddy(shaderList);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__MaterialTab);

        QMetaObject::connectSlotsByName(GammaRay__MaterialTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__MaterialTab)
    {
        label->setText(QCoreApplication::translate("GammaRay::MaterialTab", "&Shader:", nullptr));
        (void)GammaRay__MaterialTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class MaterialTab: public Ui_MaterialTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_MATERIALTAB_H
