/********************************************************************************
** Form generated from reading UI file 'qmlcontexttab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QMLCONTEXTTAB_H
#define UI_QMLCONTEXTTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_QmlContextTab
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    GammaRay::DeferredTreeView *contextView;
    GammaRay::DeferredTreeView *contextPropertyView;

    void setupUi(QWidget *GammaRay__QmlContextTab)
    {
        if (GammaRay__QmlContextTab->objectName().isEmpty())
            GammaRay__QmlContextTab->setObjectName("GammaRay__QmlContextTab");
        GammaRay__QmlContextTab->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__QmlContextTab);
        verticalLayout->setObjectName("verticalLayout");
        splitter = new QSplitter(GammaRay__QmlContextTab);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        contextView = new GammaRay::DeferredTreeView(splitter);
        contextView->setObjectName("contextView");
        contextView->setContextMenuPolicy(Qt::CustomContextMenu);
        contextView->setTextElideMode(Qt::ElideMiddle);
        contextView->setRootIsDecorated(false);
        contextView->setUniformRowHeights(true);
        splitter->addWidget(contextView);
        contextPropertyView = new GammaRay::DeferredTreeView(splitter);
        contextPropertyView->setObjectName("contextPropertyView");
        contextPropertyView->setContextMenuPolicy(Qt::CustomContextMenu);
        contextPropertyView->setSortingEnabled(true);
        splitter->addWidget(contextPropertyView);
        contextPropertyView->header()->setProperty("showSortIndicator", QVariant(true));

        verticalLayout->addWidget(splitter);


        retranslateUi(GammaRay__QmlContextTab);

        QMetaObject::connectSlotsByName(GammaRay__QmlContextTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__QmlContextTab)
    {
        (void)GammaRay__QmlContextTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class QmlContextTab: public Ui_QmlContextTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_QMLCONTEXTTAB_H
