/****************************************************************************
** Meta object code from reading C++ file 'probe.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/core/probe.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'probe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay5ProbeE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::Probe::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay5ProbeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::Probe",
        "objectSelected",
        "",
        "object",
        "QPoint",
        "pos",
        "nonQObjectSelected",
        "typeName",
        "objectCreated",
        "obj",
        "objectDestroyed",
        "objectReparented",
        "objectFavorited",
        "objectUnfavorited",
        "aboutToDetach",
        "delayedInit",
        "shutdown",
        "processQueuedObjectChanges",
        "handleObjectDestroyed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'objectSelected'
        QtMocHelpers::SignalData<void(QObject *, const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'nonQObjectSelected'
        QtMocHelpers::SignalData<void(void *, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 3 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'objectCreated'
        QtMocHelpers::SignalData<void(QObject *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
        // Signal 'objectDestroyed'
        QtMocHelpers::SignalData<void(QObject *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
        // Signal 'objectReparented'
        QtMocHelpers::SignalData<void(QObject *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
        // Signal 'objectFavorited'
        QtMocHelpers::SignalData<void(QObject *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
        // Signal 'objectUnfavorited'
        QtMocHelpers::SignalData<void(QObject *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
        // Signal 'aboutToDetach'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'delayedInit'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'processQueuedObjectChanges'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleObjectDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Probe, qt_meta_tag_ZN8GammaRay5ProbeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::Probe::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay5ProbeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay5ProbeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay5ProbeE_t>.metaTypes,
    nullptr
} };

void GammaRay::Probe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Probe *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->objectSelected((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 1: _t->nonQObjectSelected((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->objectCreated((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 3: _t->objectDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 4: _t->objectReparented((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 5: _t->objectFavorited((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 6: _t->objectUnfavorited((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 7: _t->aboutToDetach(); break;
        case 8: _t->delayedInit(); break;
        case 9: _t->shutdown(); break;
        case 10: _t->processQueuedObjectChanges(); break;
        case 11: _t->handleObjectDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(QObject * , const QPoint & )>(_a, &Probe::objectSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(void * , const QString & )>(_a, &Probe::nonQObjectSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(QObject * )>(_a, &Probe::objectCreated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(QObject * )>(_a, &Probe::objectDestroyed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(QObject * )>(_a, &Probe::objectReparented, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(QObject * )>(_a, &Probe::objectFavorited, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)(QObject * )>(_a, &Probe::objectUnfavorited, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Probe::*)()>(_a, &Probe::aboutToDetach, 7))
            return;
    }
}

const QMetaObject *GammaRay::Probe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::Probe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay5ProbeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::Probe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::Probe::objectSelected(QObject * _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void GammaRay::Probe::nonQObjectSelected(void * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void GammaRay::Probe::objectCreated(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void GammaRay::Probe::objectDestroyed(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void GammaRay::Probe::objectReparented(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void GammaRay::Probe::objectFavorited(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void GammaRay::Probe::objectUnfavorited(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void GammaRay::Probe::aboutToDetach()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
