/********************************************************************************
** Form generated from reading UI file 'propertyextendededitor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYEXTENDEDEDITOR_H
#define UI_PROPERTYEXTENDEDEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_PropertyExtendedEditor
{
public:
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineEdit;
    QToolButton *editButton;

    void setupUi(QWidget *GammaRay__PropertyExtendedEditor)
    {
        if (GammaRay__PropertyExtendedEditor->objectName().isEmpty())
            GammaRay__PropertyExtendedEditor->setObjectName("GammaRay__PropertyExtendedEditor");
        GammaRay__PropertyExtendedEditor->resize(392, 28);
        horizontalLayout = new QHBoxLayout(GammaRay__PropertyExtendedEditor);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        lineEdit = new QLineEdit(GammaRay__PropertyExtendedEditor);
        lineEdit->setObjectName("lineEdit");

        horizontalLayout->addWidget(lineEdit);

        editButton = new QToolButton(GammaRay__PropertyExtendedEditor);
        editButton->setObjectName("editButton");

        horizontalLayout->addWidget(editButton);


        retranslateUi(GammaRay__PropertyExtendedEditor);

        QMetaObject::connectSlotsByName(GammaRay__PropertyExtendedEditor);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__PropertyExtendedEditor)
    {
        editButton->setText(QCoreApplication::translate("GammaRay::PropertyExtendedEditor", "...", nullptr));
        (void)GammaRay__PropertyExtendedEditor;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertyExtendedEditor: public Ui_PropertyExtendedEditor {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTYEXTENDEDEDITOR_H
