/********************************************************************************
** Form generated from reading UI file 'styleinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STYLEINSPECTORWIDGET_H
#define UI_STYLEINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>
#include "styleelementstatetablepage.h"

namespace GammaRay {

class Ui_StyleInspectorWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *styleSelector;
    QTabWidget *tabWidget;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_5;
    GammaRay::StyleElementStateTablePage *primitivePage;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_6;
    GammaRay::StyleElementStateTablePage *controlPage;
    QWidget *tab_6;
    QVBoxLayout *verticalLayout_7;
    GammaRay::StyleElementStateTablePage *complexControlPage;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    GammaRay::DeferredTreeView *pixelMetricView;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    GammaRay::DeferredTreeView *standardIconView;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_4;
    GammaRay::DeferredTreeView *standardPaletteView;
    QWidget *tab_7;
    QVBoxLayout *verticalLayout_8;
    GammaRay::DeferredTreeView *styleHintView;

    void setupUi(QWidget *GammaRay__StyleInspectorWidget)
    {
        if (GammaRay__StyleInspectorWidget->objectName().isEmpty())
            GammaRay__StyleInspectorWidget->setObjectName("GammaRay__StyleInspectorWidget");
        GammaRay__StyleInspectorWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__StyleInspectorWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(GammaRay__StyleInspectorWidget);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(label);

        styleSelector = new QComboBox(GammaRay__StyleInspectorWidget);
        styleSelector->setObjectName("styleSelector");

        horizontalLayout->addWidget(styleSelector);


        verticalLayout->addLayout(horizontalLayout);

        tabWidget = new QTabWidget(GammaRay__StyleInspectorWidget);
        tabWidget->setObjectName("tabWidget");
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout_5 = new QVBoxLayout(tab_4);
        verticalLayout_5->setObjectName("verticalLayout_5");
        primitivePage = new GammaRay::StyleElementStateTablePage(tab_4);
        primitivePage->setObjectName("primitivePage");

        verticalLayout_5->addWidget(primitivePage);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName("tab_5");
        verticalLayout_6 = new QVBoxLayout(tab_5);
        verticalLayout_6->setObjectName("verticalLayout_6");
        controlPage = new GammaRay::StyleElementStateTablePage(tab_5);
        controlPage->setObjectName("controlPage");

        verticalLayout_6->addWidget(controlPage);

        tabWidget->addTab(tab_5, QString());
        tab_6 = new QWidget();
        tab_6->setObjectName("tab_6");
        verticalLayout_7 = new QVBoxLayout(tab_6);
        verticalLayout_7->setObjectName("verticalLayout_7");
        complexControlPage = new GammaRay::StyleElementStateTablePage(tab_6);
        complexControlPage->setObjectName("complexControlPage");

        verticalLayout_7->addWidget(complexControlPage);

        tabWidget->addTab(tab_6, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        pixelMetricView = new GammaRay::DeferredTreeView(tab);
        pixelMetricView->setObjectName("pixelMetricView");
        pixelMetricView->setRootIsDecorated(false);
        pixelMetricView->setUniformRowHeights(true);
        pixelMetricView->header()->setStretchLastSection(false);

        verticalLayout_2->addWidget(pixelMetricView);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        standardIconView = new GammaRay::DeferredTreeView(tab_2);
        standardIconView->setObjectName("standardIconView");
        standardIconView->setRootIsDecorated(false);
        standardIconView->setUniformRowHeights(true);
        standardIconView->header()->setStretchLastSection(false);

        verticalLayout_3->addWidget(standardIconView);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_4 = new QVBoxLayout(tab_3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        standardPaletteView = new GammaRay::DeferredTreeView(tab_3);
        standardPaletteView->setObjectName("standardPaletteView");
        standardPaletteView->setRootIsDecorated(false);
        standardPaletteView->setUniformRowHeights(true);
        standardPaletteView->header()->setStretchLastSection(false);

        verticalLayout_4->addWidget(standardPaletteView);

        tabWidget->addTab(tab_3, QString());
        tab_7 = new QWidget();
        tab_7->setObjectName("tab_7");
        verticalLayout_8 = new QVBoxLayout(tab_7);
        verticalLayout_8->setObjectName("verticalLayout_8");
        styleHintView = new GammaRay::DeferredTreeView(tab_7);
        styleHintView->setObjectName("styleHintView");
        styleHintView->setRootIsDecorated(false);
        styleHintView->setUniformRowHeights(true);

        verticalLayout_8->addWidget(styleHintView);

        tabWidget->addTab(tab_7, QString());

        verticalLayout->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        label->setBuddy(styleSelector);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__StyleInspectorWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__StyleInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__StyleInspectorWidget)
    {
        label->setText(QCoreApplication::translate("GammaRay::StyleInspectorWidget", "&Style:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Primitives", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Controls", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_6), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Complex Controls", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Pixel Metric", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Standard Icons", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Standard Palette", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_7), QCoreApplication::translate("GammaRay::StyleInspectorWidget", "Style Hints", nullptr));
        (void)GammaRay__StyleInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class StyleInspectorWidget: public Ui_StyleInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_STYLEINSPECTORWIDGET_H
