/********************************************************************************
** Form generated from reading UI file 'selftestpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SELFTESTPAGE_H
#define UI_SELFTESTPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QListView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_SelfTestPage
{
public:
    QVBoxLayout *verticalLayout;
    QListView *resultView;

    void setupUi(QWidget *GammaRay__SelfTestPage)
    {
        if (GammaRay__SelfTestPage->objectName().isEmpty())
            GammaRay__SelfTestPage->setObjectName("GammaRay__SelfTestPage");
        GammaRay__SelfTestPage->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__SelfTestPage);
        verticalLayout->setObjectName("verticalLayout");
        resultView = new QListView(GammaRay__SelfTestPage);
        resultView->setObjectName("resultView");

        verticalLayout->addWidget(resultView);


        retranslateUi(GammaRay__SelfTestPage);

        QMetaObject::connectSlotsByName(GammaRay__SelfTestPage);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__SelfTestPage)
    {
        (void)GammaRay__SelfTestPage;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class SelfTestPage: public Ui_SelfTestPage {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_SELFTESTPAGE_H
