/****************************************************************************
** Meta object code from reading C++ file 'mapcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/positioning/mapcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mapcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay13MapControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::MapController::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay13MapControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::MapController",
        "centerOnPosition",
        "",
        "sourceChanged",
        "overrideEnabledChanged",
        "overrideCoordinateChanged",
        "overrideHorizontalAccuracyChanged",
        "overrideDirectionChanged",
        "sourceCoordinate",
        "QGeoCoordinate",
        "sourceHorizontalAccuracy",
        "sourceDirection",
        "overrideEnabled",
        "overrideCoordinate",
        "overrideHorizontalAccuracy",
        "overrideDirection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'centerOnPosition'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overrideEnabledChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overrideCoordinateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overrideHorizontalAccuracyChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overrideDirectionChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'sourceCoordinate'
        QtMocHelpers::PropertyData<QGeoCoordinate>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'sourceHorizontalAccuracy'
        QtMocHelpers::PropertyData<double>(10, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'sourceDirection'
        QtMocHelpers::PropertyData<double>(11, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'overrideEnabled'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'overrideCoordinate'
        QtMocHelpers::PropertyData<QGeoCoordinate>(13, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'overrideHorizontalAccuracy'
        QtMocHelpers::PropertyData<double>(14, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'overrideDirection'
        QtMocHelpers::PropertyData<double>(15, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MapController, qt_meta_tag_ZN8GammaRay13MapControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::MapController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay13MapControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay13MapControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay13MapControllerE_t>.metaTypes,
    nullptr
} };

void GammaRay::MapController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MapController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->centerOnPosition(); break;
        case 1: _t->sourceChanged(); break;
        case 2: _t->overrideEnabledChanged(); break;
        case 3: _t->overrideCoordinateChanged(); break;
        case 4: _t->overrideHorizontalAccuracyChanged(); break;
        case 5: _t->overrideDirectionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MapController::*)()>(_a, &MapController::centerOnPosition, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapController::*)()>(_a, &MapController::sourceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapController::*)()>(_a, &MapController::overrideEnabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapController::*)()>(_a, &MapController::overrideCoordinateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapController::*)()>(_a, &MapController::overrideHorizontalAccuracyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapController::*)()>(_a, &MapController::overrideDirectionChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->sourceCoordinate(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->sourceHorizontalAccuracy(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->sourceDirection(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->overrideEnabled(); break;
        case 4: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->overrideCoordinate(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->overrideHorizontalAccuracy(); break;
        case 6: *reinterpret_cast<double*>(_v) = _t->overrideDirection(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSourceCoordinate(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 1: _t->setSourceHorizontalAccuracy(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setSourceDirection(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setOverrideEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setOverrideCoordinate(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 5: _t->setOverrideHorizontalAccuracy(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setOverrideDirection(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *GammaRay::MapController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::MapController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay13MapControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::MapController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::MapController::centerOnPosition()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::MapController::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GammaRay::MapController::overrideEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GammaRay::MapController::overrideCoordinateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GammaRay::MapController::overrideHorizontalAccuracyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void GammaRay::MapController::overrideDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
