/********************************************************************************
** Form generated from reading UI file 'textdocumentinspectorwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TEXTDOCUMENTINSPECTORWIDGET_H
#define UI_TEXTDOCUMENTINSPECTORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>
#include "textdocumentcontentview.h"

namespace GammaRay {

class Ui_TextDocumentInspectorWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QSplitter *mainSplitter;
    QWidget *layoutWidget_1;
    QVBoxLayout *verticalLayout_17;
    QLabel *label_5;
    GammaRay::DeferredTreeView *documentList;
    QSplitter *structureSplitter;
    QWidget *layoutWidget_2;
    QVBoxLayout *verticalLayout_18;
    QLabel *label_6;
    GammaRay::DeferredTreeView *documentTree;
    QWidget *layoutWidget_3;
    QVBoxLayout *verticalLayout_19;
    QLabel *label_4;
    GammaRay::DeferredTreeView *documentFormatView;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout;
    GammaRay::TextDocumentContentView *documentView;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_2;
    QTextBrowser *htmlView;

    void setupUi(QWidget *GammaRay__TextDocumentInspectorWidget)
    {
        if (GammaRay__TextDocumentInspectorWidget->objectName().isEmpty())
            GammaRay__TextDocumentInspectorWidget->setObjectName("GammaRay__TextDocumentInspectorWidget");
        GammaRay__TextDocumentInspectorWidget->resize(762, 631);
        horizontalLayout = new QHBoxLayout(GammaRay__TextDocumentInspectorWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        mainSplitter = new QSplitter(GammaRay__TextDocumentInspectorWidget);
        mainSplitter->setObjectName("mainSplitter");
        mainSplitter->setOrientation(Qt::Horizontal);
        layoutWidget_1 = new QWidget(mainSplitter);
        layoutWidget_1->setObjectName("layoutWidget_1");
        verticalLayout_17 = new QVBoxLayout(layoutWidget_1);
        verticalLayout_17->setObjectName("verticalLayout_17");
        verticalLayout_17->setContentsMargins(0, 0, 0, 0);
        label_5 = new QLabel(layoutWidget_1);
        label_5->setObjectName("label_5");

        verticalLayout_17->addWidget(label_5);

        documentList = new GammaRay::DeferredTreeView(layoutWidget_1);
        documentList->setObjectName("documentList");
        documentList->setContextMenuPolicy(Qt::CustomContextMenu);
        documentList->setRootIsDecorated(false);
        documentList->setUniformRowHeights(true);
        documentList->header()->setStretchLastSection(false);

        verticalLayout_17->addWidget(documentList);

        mainSplitter->addWidget(layoutWidget_1);
        structureSplitter = new QSplitter(mainSplitter);
        structureSplitter->setObjectName("structureSplitter");
        structureSplitter->setOrientation(Qt::Vertical);
        layoutWidget_2 = new QWidget(structureSplitter);
        layoutWidget_2->setObjectName("layoutWidget_2");
        verticalLayout_18 = new QVBoxLayout(layoutWidget_2);
        verticalLayout_18->setObjectName("verticalLayout_18");
        verticalLayout_18->setContentsMargins(0, 0, 0, 0);
        label_6 = new QLabel(layoutWidget_2);
        label_6->setObjectName("label_6");

        verticalLayout_18->addWidget(label_6);

        documentTree = new GammaRay::DeferredTreeView(layoutWidget_2);
        documentTree->setObjectName("documentTree");
        documentTree->setUniformRowHeights(true);
        documentTree->header()->setStretchLastSection(false);

        verticalLayout_18->addWidget(documentTree);

        structureSplitter->addWidget(layoutWidget_2);
        layoutWidget_3 = new QWidget(structureSplitter);
        layoutWidget_3->setObjectName("layoutWidget_3");
        verticalLayout_19 = new QVBoxLayout(layoutWidget_3);
        verticalLayout_19->setObjectName("verticalLayout_19");
        verticalLayout_19->setContentsMargins(0, 0, 0, 0);
        label_4 = new QLabel(layoutWidget_3);
        label_4->setObjectName("label_4");

        verticalLayout_19->addWidget(label_4);

        documentFormatView = new GammaRay::DeferredTreeView(layoutWidget_3);
        documentFormatView->setObjectName("documentFormatView");
        documentFormatView->setRootIsDecorated(false);
        documentFormatView->setUniformRowHeights(true);
        documentFormatView->header()->setStretchLastSection(false);

        verticalLayout_19->addWidget(documentFormatView);

        structureSplitter->addWidget(layoutWidget_3);
        mainSplitter->addWidget(structureSplitter);
        tabWidget = new QTabWidget(mainSplitter);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout = new QVBoxLayout(tab);
        verticalLayout->setObjectName("verticalLayout");
        documentView = new GammaRay::TextDocumentContentView(tab);
        documentView->setObjectName("documentView");

        verticalLayout->addWidget(documentView);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_2 = new QVBoxLayout(tab_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        htmlView = new QTextBrowser(tab_2);
        htmlView->setObjectName("htmlView");

        verticalLayout_2->addWidget(htmlView);

        tabWidget->addTab(tab_2, QString());
        mainSplitter->addWidget(tabWidget);

        horizontalLayout->addWidget(mainSplitter);


        retranslateUi(GammaRay__TextDocumentInspectorWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__TextDocumentInspectorWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__TextDocumentInspectorWidget)
    {
        label_5->setText(QCoreApplication::translate("GammaRay::TextDocumentInspectorWidget", "All documents:", nullptr));
        label_6->setText(QCoreApplication::translate("GammaRay::TextDocumentInspectorWidget", "Document structure:", nullptr));
        label_4->setText(QCoreApplication::translate("GammaRay::TextDocumentInspectorWidget", "Element format:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("GammaRay::TextDocumentInspectorWidget", "Content", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("GammaRay::TextDocumentInspectorWidget", "HTML", nullptr));
        (void)GammaRay__TextDocumentInspectorWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class TextDocumentInspectorWidget: public Ui_TextDocumentInspectorWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_TEXTDOCUMENTINSPECTORWIDGET_H
