/********************************************************************************
** Form generated from reading UI file 'graphicssceneview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRAPHICSSCENEVIEW_H
#define UI_GRAPHICSSCENEVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "graphicsview.h"

namespace GammaRay {

class Ui_GraphicsSceneView
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLabel *sceneCoordLabel;
    QLabel *label_3;
    QLabel *itemCoordLabel;
    GammaRay::GraphicsView *graphicsView;

    void setupUi(QWidget *GammaRay__GraphicsSceneView)
    {
        if (GammaRay__GraphicsSceneView->objectName().isEmpty())
            GammaRay__GraphicsSceneView->setObjectName("GammaRay__GraphicsSceneView");
        GammaRay__GraphicsSceneView->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__GraphicsSceneView);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(GammaRay__GraphicsSceneView);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        sceneCoordLabel = new QLabel(GammaRay__GraphicsSceneView);
        sceneCoordLabel->setObjectName("sceneCoordLabel");

        horizontalLayout->addWidget(sceneCoordLabel);

        label_3 = new QLabel(GammaRay__GraphicsSceneView);
        label_3->setObjectName("label_3");

        horizontalLayout->addWidget(label_3);

        itemCoordLabel = new QLabel(GammaRay__GraphicsSceneView);
        itemCoordLabel->setObjectName("itemCoordLabel");

        horizontalLayout->addWidget(itemCoordLabel);


        verticalLayout->addLayout(horizontalLayout);

        graphicsView = new GammaRay::GraphicsView(GammaRay__GraphicsSceneView);
        graphicsView->setObjectName("graphicsView");
        graphicsView->setInteractive(false);
        graphicsView->setDragMode(QGraphicsView::ScrollHandDrag);

        verticalLayout->addWidget(graphicsView);


        retranslateUi(GammaRay__GraphicsSceneView);

        QMetaObject::connectSlotsByName(GammaRay__GraphicsSceneView);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__GraphicsSceneView)
    {
        label->setText(QCoreApplication::translate("GammaRay::GraphicsSceneView", "Scene Coordinates:", nullptr));
        sceneCoordLabel->setText(QCoreApplication::translate("GammaRay::GraphicsSceneView", "0x0", nullptr));
        label_3->setText(QCoreApplication::translate("GammaRay::GraphicsSceneView", "Item Coordinates:", nullptr));
        itemCoordLabel->setText(QCoreApplication::translate("GammaRay::GraphicsSceneView", "0x0", nullptr));
        (void)GammaRay__GraphicsSceneView;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class GraphicsSceneView: public Ui_GraphicsSceneView {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_GRAPHICSSCENEVIEW_H
