/********************************************************************************
** Form generated from reading UI file 'kjobtrackerwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KJOBTRACKERWIDGET_H
#define UI_KJOBTRACKERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_KJobTrackerWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *searchLine;
    GammaRay::DeferredTreeView *jobView;

    void setupUi(QWidget *GammaRay__KJobTrackerWidget)
    {
        if (GammaRay__KJobTrackerWidget->objectName().isEmpty())
            GammaRay__KJobTrackerWidget->setObjectName("GammaRay__KJobTrackerWidget");
        GammaRay__KJobTrackerWidget->resize(400, 300);
        verticalLayout = new QVBoxLayout(GammaRay__KJobTrackerWidget);
        verticalLayout->setObjectName("verticalLayout");
        searchLine = new QLineEdit(GammaRay__KJobTrackerWidget);
        searchLine->setObjectName("searchLine");

        verticalLayout->addWidget(searchLine);

        jobView = new GammaRay::DeferredTreeView(GammaRay__KJobTrackerWidget);
        jobView->setObjectName("jobView");
        jobView->setRootIsDecorated(false);
        jobView->setUniformRowHeights(true);
        jobView->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(jobView);


        retranslateUi(GammaRay__KJobTrackerWidget);

        QMetaObject::connectSlotsByName(GammaRay__KJobTrackerWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__KJobTrackerWidget)
    {
        (void)GammaRay__KJobTrackerWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class KJobTrackerWidget: public Ui_KJobTrackerWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_KJOBTRACKERWIDGET_H
