/****************************************************************************
** Meta object code from reading C++ file 'remoteviewserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/core/remoteviewserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'remoteviewserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::RemoteViewServer::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::RemoteViewServer",
        "elementsAtRequested",
        "",
        "QPoint",
        "pos",
        "GammaRay::RemoteViewInterface::RequestMode",
        "mode",
        "doPickElementId",
        "GammaRay::ObjectId",
        "id",
        "requestUpdate",
        "sourceChanged",
        "requestCompleteFrame",
        "clientConnectedChanged",
        "connected",
        "requestUpdateTimeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'elementsAtRequested'
        QtMocHelpers::SignalData<void(const QPoint &, GammaRay::RemoteViewInterface::RequestMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'doPickElementId'
        QtMocHelpers::SignalData<void(const GammaRay::ObjectId &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'requestUpdate'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sourceChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestCompleteFrame'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clientConnectedChanged'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'requestUpdateTimeout'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RemoteViewServer, qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::RemoteViewServer::staticMetaObject = { {
    QMetaObject::SuperData::link<RemoteViewInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t>.metaTypes,
    nullptr
} };

void GammaRay::RemoteViewServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteViewServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->elementsAtRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GammaRay::RemoteViewInterface::RequestMode>>(_a[2]))); break;
        case 1: _t->doPickElementId((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectId>>(_a[1]))); break;
        case 2: _t->requestUpdate(); break;
        case 3: _t->sourceChanged(); break;
        case 4: _t->requestCompleteFrame(); break;
        case 5: _t->clientConnectedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->requestUpdateTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::RemoteViewInterface::RequestMode >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RemoteViewServer::*)(const QPoint & , GammaRay::RemoteViewInterface::RequestMode )>(_a, &RemoteViewServer::elementsAtRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewServer::*)(const GammaRay::ObjectId & )>(_a, &RemoteViewServer::doPickElementId, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteViewServer::*)()>(_a, &RemoteViewServer::requestUpdate, 2))
            return;
    }
}

const QMetaObject *GammaRay::RemoteViewServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::RemoteViewServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16RemoteViewServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.RemoteViewInterface/1.0"))
        return static_cast< GammaRay::RemoteViewInterface*>(this);
    return RemoteViewInterface::qt_metacast(_clname);
}

int GammaRay::RemoteViewServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RemoteViewInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::RemoteViewServer::elementsAtRequested(const QPoint & _t1, GammaRay::RemoteViewInterface::RequestMode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void GammaRay::RemoteViewServer::doPickElementId(const GammaRay::ObjectId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GammaRay::RemoteViewServer::requestUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
