/********************************************************************************
** Form generated from reading UI file 'paintanalyzerwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAINTANALYZERWIDGET_H
#define UI_PAINTANALYZERWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>
#include <ui/paintanalyzerreplayview.h>

namespace GammaRay {

class Ui_PaintAnalyzerWidget
{
public:
    QAction *actionShowClipArea;
    QHBoxLayout *horizontalLayout_2;
    QSplitter *paintAnalyzerSplitter;
    QSplitter *argumentSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLineEdit *commandSearchLine;
    GammaRay::DeferredTreeView *commandView;
    QTabWidget *detailsTabWidget;
    QWidget *argumentTab;
    QVBoxLayout *verticalLayout_2;
    QTreeView *argumentView;
    QWidget *stackTraceTab;
    QVBoxLayout *verticalLayout_3;
    QTreeView *stackTraceView;
    QWidget *replayLayoutWidget;
    QVBoxLayout *replayContainer;
    GammaRay::PaintAnalyzerReplayView *replayWidget;

    void setupUi(QWidget *GammaRay__PaintAnalyzerWidget)
    {
        if (GammaRay__PaintAnalyzerWidget->objectName().isEmpty())
            GammaRay__PaintAnalyzerWidget->setObjectName("GammaRay__PaintAnalyzerWidget");
        GammaRay__PaintAnalyzerWidget->resize(591, 378);
        actionShowClipArea = new QAction(GammaRay__PaintAnalyzerWidget);
        actionShowClipArea->setObjectName("actionShowClipArea");
        actionShowClipArea->setCheckable(true);
        horizontalLayout_2 = new QHBoxLayout(GammaRay__PaintAnalyzerWidget);
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        paintAnalyzerSplitter = new QSplitter(GammaRay__PaintAnalyzerWidget);
        paintAnalyzerSplitter->setObjectName("paintAnalyzerSplitter");
        paintAnalyzerSplitter->setOrientation(Qt::Horizontal);
        argumentSplitter = new QSplitter(paintAnalyzerSplitter);
        argumentSplitter->setObjectName("argumentSplitter");
        argumentSplitter->setOrientation(Qt::Vertical);
        layoutWidget = new QWidget(argumentSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        commandSearchLine = new QLineEdit(layoutWidget);
        commandSearchLine->setObjectName("commandSearchLine");

        verticalLayout->addWidget(commandSearchLine);

        commandView = new GammaRay::DeferredTreeView(layoutWidget);
        commandView->setObjectName("commandView");
        commandView->setContextMenuPolicy(Qt::CustomContextMenu);
        commandView->setIndentation(10);
        commandView->setSortingEnabled(false);

        verticalLayout->addWidget(commandView);

        argumentSplitter->addWidget(layoutWidget);
        detailsTabWidget = new QTabWidget(argumentSplitter);
        detailsTabWidget->setObjectName("detailsTabWidget");
        argumentTab = new QWidget();
        argumentTab->setObjectName("argumentTab");
        verticalLayout_2 = new QVBoxLayout(argumentTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        argumentView = new QTreeView(argumentTab);
        argumentView->setObjectName("argumentView");
        argumentView->setIndentation(10);

        verticalLayout_2->addWidget(argumentView);

        detailsTabWidget->addTab(argumentTab, QString());
        stackTraceTab = new QWidget();
        stackTraceTab->setObjectName("stackTraceTab");
        verticalLayout_3 = new QVBoxLayout(stackTraceTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        stackTraceView = new QTreeView(stackTraceTab);
        stackTraceView->setObjectName("stackTraceView");
        stackTraceView->setContextMenuPolicy(Qt::CustomContextMenu);
        stackTraceView->setRootIsDecorated(false);
        stackTraceView->setUniformRowHeights(true);

        verticalLayout_3->addWidget(stackTraceView);

        detailsTabWidget->addTab(stackTraceTab, QString());
        argumentSplitter->addWidget(detailsTabWidget);
        paintAnalyzerSplitter->addWidget(argumentSplitter);
        replayLayoutWidget = new QWidget(paintAnalyzerSplitter);
        replayLayoutWidget->setObjectName("replayLayoutWidget");
        replayContainer = new QVBoxLayout(replayLayoutWidget);
        replayContainer->setObjectName("replayContainer");
        replayContainer->setContentsMargins(0, 0, 0, 0);
        replayWidget = new GammaRay::PaintAnalyzerReplayView(replayLayoutWidget);
        replayWidget->setObjectName("replayWidget");

        replayContainer->addWidget(replayWidget);

        paintAnalyzerSplitter->addWidget(replayLayoutWidget);

        horizontalLayout_2->addWidget(paintAnalyzerSplitter);


        retranslateUi(GammaRay__PaintAnalyzerWidget);

        detailsTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__PaintAnalyzerWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__PaintAnalyzerWidget)
    {
        actionShowClipArea->setText(QCoreApplication::translate("GammaRay::PaintAnalyzerWidget", "Visualize Clip Area", nullptr));
#if QT_CONFIG(tooltip)
        actionShowClipArea->setToolTip(QCoreApplication::translate("GammaRay::PaintAnalyzerWidget", "Highlight current clipping area.", nullptr));
#endif // QT_CONFIG(tooltip)
        detailsTabWidget->setTabText(detailsTabWidget->indexOf(argumentTab), QCoreApplication::translate("GammaRay::PaintAnalyzerWidget", "Argument", nullptr));
        detailsTabWidget->setTabText(detailsTabWidget->indexOf(stackTraceTab), QCoreApplication::translate("GammaRay::PaintAnalyzerWidget", "Stack Trace", nullptr));
        (void)GammaRay__PaintAnalyzerWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PaintAnalyzerWidget: public Ui_PaintAnalyzerWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PAINTANALYZERWIDGET_H
