/********************************************************************************
** Form generated from reading UI file 'propertydoublepaireditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYDOUBLEPAIREDITOR_H
#define UI_PROPERTYDOUBLEPAIREDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_PropertyDoublePairEditor
{
public:
    QHBoxLayout *horizontalLayout;
    QDoubleSpinBox *xBox;
    QLabel *label;
    QDoubleSpinBox *yBox;

    void setupUi(QWidget *GammaRay__PropertyDoublePairEditor)
    {
        if (GammaRay__PropertyDoublePairEditor->objectName().isEmpty())
            GammaRay__PropertyDoublePairEditor->setObjectName("GammaRay__PropertyDoublePairEditor");
        GammaRay__PropertyDoublePairEditor->resize(400, 300);
        horizontalLayout = new QHBoxLayout(GammaRay__PropertyDoublePairEditor);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        xBox = new QDoubleSpinBox(GammaRay__PropertyDoublePairEditor);
        xBox->setObjectName("xBox");
        xBox->setFrame(false);
        xBox->setMinimum(-999999999.000000000000000);
        xBox->setMaximum(999999999.000000000000000);

        horizontalLayout->addWidget(xBox);

        label = new QLabel(GammaRay__PropertyDoublePairEditor);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(label);

        yBox = new QDoubleSpinBox(GammaRay__PropertyDoublePairEditor);
        yBox->setObjectName("yBox");
        yBox->setFrame(false);
        yBox->setMinimum(-999999999.000000000000000);
        yBox->setMaximum(999999999.000000000000000);

        horizontalLayout->addWidget(yBox);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(2, 1);

        retranslateUi(GammaRay__PropertyDoublePairEditor);

        QMetaObject::connectSlotsByName(GammaRay__PropertyDoublePairEditor);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__PropertyDoublePairEditor)
    {
        label->setText(QCoreApplication::translate("GammaRay::PropertyDoublePairEditor", "x", nullptr));
        (void)GammaRay__PropertyDoublePairEditor;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class PropertyDoublePairEditor: public Ui_PropertyDoublePairEditor {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_PROPERTYDOUBLEPAIREDITOR_H
