/********************************************************************************
** Form generated from reading UI file 'networkreplywidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NETWORKREPLYWIDGET_H
#define UI_NETWORKREPLYWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_NetworkReplyWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QCheckBox *captureResponse;
    QSplitter *splitter;
    QTreeView *replyView;
    QPlainTextEdit *responseTextEdit;
    QLabel *imageLabel;

    void setupUi(QWidget *GammaRay__NetworkReplyWidget)
    {
        if (GammaRay__NetworkReplyWidget->objectName().isEmpty())
            GammaRay__NetworkReplyWidget->setObjectName("GammaRay__NetworkReplyWidget");
        GammaRay__NetworkReplyWidget->resize(400, 300);
        verticalLayout_2 = new QVBoxLayout(GammaRay__NetworkReplyWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        captureResponse = new QCheckBox(GammaRay__NetworkReplyWidget);
        captureResponse->setObjectName("captureResponse");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(captureResponse->sizePolicy().hasHeightForWidth());
        captureResponse->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(captureResponse);

        splitter = new QSplitter(GammaRay__NetworkReplyWidget);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        replyView = new QTreeView(splitter);
        replyView->setObjectName("replyView");
        replyView->setContextMenuPolicy(Qt::CustomContextMenu);
        replyView->setUniformRowHeights(true);
        splitter->addWidget(replyView);
        responseTextEdit = new QPlainTextEdit(splitter);
        responseTextEdit->setObjectName("responseTextEdit");
        responseTextEdit->setReadOnly(true);
        splitter->addWidget(responseTextEdit);
        imageLabel = new QLabel(splitter);
        imageLabel->setObjectName("imageLabel");
        splitter->addWidget(imageLabel);

        verticalLayout->addWidget(splitter);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(GammaRay__NetworkReplyWidget);

        QMetaObject::connectSlotsByName(GammaRay__NetworkReplyWidget);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__NetworkReplyWidget)
    {
        captureResponse->setText(QCoreApplication::translate("GammaRay::NetworkReplyWidget", "Capture response body", nullptr));
        imageLabel->setText(QCoreApplication::translate("GammaRay::NetworkReplyWidget", "TextLabel", nullptr));
        (void)GammaRay__NetworkReplyWidget;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class NetworkReplyWidget: public Ui_NetworkReplyWidget {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_NETWORKREPLYWIDGET_H
