/****************************************************************************
** Meta object code from reading C++ file 'widgetinspectorserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../GammaRay/plugins/widgetinspector/widgetinspectorserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'widgetinspectorserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::WidgetInspectorServer::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::WidgetInspectorServer",
        "elementsAtReceived",
        "",
        "GammaRay::ObjectIds",
        "ids",
        "bestCandidate",
        "widgetSelectionChanged",
        "QItemSelection",
        "selection",
        "widgetSelected",
        "QWidget*",
        "widget",
        "objectSelected",
        "obj",
        "objectCreated",
        "object",
        "recreateOverlayWidget",
        "saveAsImage",
        "fileName",
        "saveAsSvg",
        "saveAsUiFile",
        "analyzePainting",
        "updateWidgetPreview",
        "requestElementsAt",
        "QPoint",
        "pos",
        "GammaRay::RemoteViewInterface::RequestMode",
        "mode",
        "pickElementId",
        "GammaRay::ObjectId",
        "id"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'elementsAtReceived'
        QtMocHelpers::SignalData<void(const GammaRay::ObjectIds &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'widgetSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'widgetSelected'
        QtMocHelpers::SlotData<void(QWidget *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'objectSelected'
        QtMocHelpers::SlotData<void(QObject *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 13 },
        }}),
        // Slot 'objectCreated'
        QtMocHelpers::SlotData<void(QObject *)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 15 },
        }}),
        // Slot 'recreateOverlayWidget'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAsImage'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'saveAsSvg'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'saveAsUiFile'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'analyzePainting'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWidgetPreview'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestElementsAt'
        QtMocHelpers::SlotData<void(const QPoint &, GammaRay::RemoteViewInterface::RequestMode)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Slot 'pickElementId'
        QtMocHelpers::SlotData<void(const GammaRay::ObjectId &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WidgetInspectorServer, qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::WidgetInspectorServer::staticMetaObject = { {
    QMetaObject::SuperData::link<WidgetInspectorInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t>.metaTypes,
    nullptr
} };

void GammaRay::WidgetInspectorServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WidgetInspectorServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->elementsAtReceived((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectIds>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->widgetSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 2: _t->widgetSelected((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 3: _t->objectSelected((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 4: _t->objectCreated((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 5: _t->recreateOverlayWidget(); break;
        case 6: _t->saveAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->saveAsSvg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->saveAsUiFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->analyzePainting(); break;
        case 10: _t->updateWidgetPreview(); break;
        case 11: _t->requestElementsAt((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GammaRay::RemoteViewInterface::RequestMode>>(_a[2]))); break;
        case 12: _t->pickElementId((*reinterpret_cast<std::add_pointer_t<GammaRay::ObjectId>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectIds >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::RemoteViewInterface::RequestMode >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< GammaRay::ObjectId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WidgetInspectorServer::*)(const GammaRay::ObjectIds & , int )>(_a, &WidgetInspectorServer::elementsAtReceived, 0))
            return;
    }
}

const QMetaObject *GammaRay::WidgetInspectorServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::WidgetInspectorServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay21WidgetInspectorServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.kdab.GammaRay.WidgetInspector"))
        return static_cast< GammaRay::WidgetInspectorInterface*>(this);
    return WidgetInspectorInterface::qt_metacast(_clname);
}

int GammaRay::WidgetInspectorServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WidgetInspectorInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::WidgetInspectorServer::elementsAtReceived(const GammaRay::ObjectIds & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
