/********************************************************************************
** Form generated from reading UI file 'attachdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ATTACHDIALOG_H
#define UI_ATTACHDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace GammaRay {

class Ui_AttachDialog
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *stackedWidget;
    QLabel *loadingLabel;
    QWidget *listViewPage;
    QVBoxLayout *verticalLayout_2;
    QLineEdit *filter;
    QTreeView *view;
    QWidget *settingsWidget;
    QVBoxLayout *verticalLayout_3;
    QFormLayout *formLayout;
    QLabel *label_2;
    QComboBox *probeBox;
    QLabel *label;
    QComboBox *accessMode;

    void setupUi(QWidget *GammaRay__AttachDialog)
    {
        if (GammaRay__AttachDialog->objectName().isEmpty())
            GammaRay__AttachDialog->setObjectName("GammaRay__AttachDialog");
        GammaRay__AttachDialog->resize(400, 323);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/gammaray/GammaRay-128x128.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        GammaRay__AttachDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(GammaRay__AttachDialog);
        verticalLayout->setObjectName("verticalLayout");
        stackedWidget = new QStackedWidget(GammaRay__AttachDialog);
        stackedWidget->setObjectName("stackedWidget");
        loadingLabel = new QLabel();
        loadingLabel->setObjectName("loadingLabel");
        stackedWidget->addWidget(loadingLabel);
        listViewPage = new QWidget();
        listViewPage->setObjectName("listViewPage");
        verticalLayout_2 = new QVBoxLayout(listViewPage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        filter = new QLineEdit(listViewPage);
        filter->setObjectName("filter");

        verticalLayout_2->addWidget(filter);

        view = new QTreeView(listViewPage);
        view->setObjectName("view");
        view->setRootIsDecorated(false);

        verticalLayout_2->addWidget(view);

        settingsWidget = new QWidget(listViewPage);
        settingsWidget->setObjectName("settingsWidget");
        verticalLayout_3 = new QVBoxLayout(settingsWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, -1, 0, 0);
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label_2 = new QLabel(settingsWidget);
        label_2->setObjectName("label_2");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_2);

        probeBox = new QComboBox(settingsWidget);
        probeBox->setObjectName("probeBox");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, probeBox);

        label = new QLabel(settingsWidget);
        label->setObjectName("label");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        accessMode = new QComboBox(settingsWidget);
        accessMode->addItem(QString());
        accessMode->addItem(QString());
        accessMode->addItem(QString());
        accessMode->setObjectName("accessMode");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, accessMode);


        verticalLayout_3->addLayout(formLayout);


        verticalLayout_2->addWidget(settingsWidget);

        stackedWidget->addWidget(listViewPage);
        listViewPage->raise();
        loadingLabel->raise();

        verticalLayout->addWidget(stackedWidget);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(probeBox);
        label->setBuddy(accessMode);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__AttachDialog);

        stackedWidget->setCurrentIndex(1);
        accessMode->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GammaRay__AttachDialog);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__AttachDialog)
    {
        loadingLabel->setText(QCoreApplication::translate("GammaRay::AttachDialog", "<html><head/><body><p align=\"center\"><img src=\":/gammaray/GammaRay-48x48.png\"/></p><p align=\"center\">Retrieving the list of processes which use Qt ...</p></body></html>", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::AttachDialog", "Pro&be:", nullptr));
        label->setText(QCoreApplication::translate("GammaRay::AttachDialog", "&Access mode:", nullptr));
        accessMode->setItemText(0, QCoreApplication::translate("GammaRay::AttachDialog", "Out-of-process, local debugging only", nullptr));
        accessMode->setItemText(1, QCoreApplication::translate("GammaRay::AttachDialog", "Out-of-process, remote debugging enabled", nullptr));
        accessMode->setItemText(2, QCoreApplication::translate("GammaRay::AttachDialog", "In-process", nullptr));

        (void)GammaRay__AttachDialog;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class AttachDialog: public Ui_AttachDialog {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_ATTACHDIALOG_H
