/****************************************************************************
** Meta object code from reading C++ file 'endpoint.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/common/endpoint.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'endpoint.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay8EndpointE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::Endpoint::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay8EndpointE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::Endpoint",
        "connectionEstablished",
        "",
        "disconnected",
        "objectRegistered",
        "objectName",
        "Protocol::ObjectAddress",
        "objectAddress",
        "objectUnregistered",
        "logTransmissionRate",
        "bytesRead",
        "bytesWritten",
        "sendMessage",
        "GammaRay::Message",
        "msg",
        "readyRead",
        "doLogTransmissionRate",
        "connectionClosed",
        "slotHandlerDestroyed",
        "obj",
        "slotObjectDestroyed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectionEstablished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'objectRegistered'
        QtMocHelpers::SignalData<void(const QString &, Protocol::ObjectAddress)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'objectUnregistered'
        QtMocHelpers::SignalData<void(const QString &, Protocol::ObjectAddress)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'logTransmissionRate'
        QtMocHelpers::SignalData<void(quint64, quint64)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 10 }, { QMetaType::ULongLong, 11 },
        }}),
        // Slot 'sendMessage'
        QtMocHelpers::SlotData<void(const GammaRay::Message &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'readyRead'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'doLogTransmissionRate'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'connectionClosed'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandlerDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 19 },
        }}),
        // Slot 'slotObjectDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Endpoint, qt_meta_tag_ZN8GammaRay8EndpointE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::Endpoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay8EndpointE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay8EndpointE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay8EndpointE_t>.metaTypes,
    nullptr
} };

void GammaRay::Endpoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Endpoint *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectionEstablished(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->objectRegistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Protocol::ObjectAddress>>(_a[2]))); break;
        case 3: _t->objectUnregistered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Protocol::ObjectAddress>>(_a[2]))); break;
        case 4: _t->logTransmissionRate((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 5: _t->sendMessage((*reinterpret_cast<std::add_pointer_t<GammaRay::Message>>(_a[1]))); break;
        case 6: _t->readyRead(); break;
        case 7: _t->doLogTransmissionRate(); break;
        case 8: _t->connectionClosed(); break;
        case 9: _t->slotHandlerDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 10: _t->slotObjectDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Endpoint::*)()>(_a, &Endpoint::connectionEstablished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Endpoint::*)()>(_a, &Endpoint::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Endpoint::*)(const QString & , Protocol::ObjectAddress )>(_a, &Endpoint::objectRegistered, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Endpoint::*)(const QString & , Protocol::ObjectAddress )>(_a, &Endpoint::objectUnregistered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Endpoint::*)(quint64 , quint64 )>(_a, &Endpoint::logTransmissionRate, 4))
            return;
    }
}

const QMetaObject *GammaRay::Endpoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::Endpoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay8EndpointE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::Endpoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::Endpoint::connectionEstablished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GammaRay::Endpoint::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GammaRay::Endpoint::objectRegistered(const QString & _t1, Protocol::ObjectAddress _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void GammaRay::Endpoint::objectUnregistered(const QString & _t1, Protocol::ObjectAddress _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void GammaRay::Endpoint::logTransmissionRate(quint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
