/****************************************************************************
** Meta object code from reading C++ file 'problemcollector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../GammaRay/core/problemcollector.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'problemcollector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t {};
} // unnamed namespace

template <> constexpr inline auto GammaRay::ProblemCollector::qt_create_metaobjectdata<qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GammaRay::ProblemCollector",
        "aboutToAddProblem",
        "",
        "row",
        "problemAdded",
        "aboutToRemoveProblems",
        "first",
        "count",
        "problemsRemoved",
        "problemScansFinished",
        "aboutToAddChecker",
        "checkerAdded",
        "requestScan"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToAddProblem'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'problemAdded'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToRemoveProblems'
        QtMocHelpers::SignalData<void(int, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Int, 7 },
        }}),
        // Signal 'aboutToRemoveProblems'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'problemsRemoved'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'problemScansFinished'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToAddChecker'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'checkerAdded'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestScan'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProblemCollector, qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GammaRay::ProblemCollector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t>.metaTypes,
    nullptr
} };

void GammaRay::ProblemCollector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProblemCollector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToAddProblem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->problemAdded(); break;
        case 2: _t->aboutToRemoveProblems((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->aboutToRemoveProblems((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->problemsRemoved(); break;
        case 5: _t->problemScansFinished(); break;
        case 6: _t->aboutToAddChecker(); break;
        case 7: _t->checkerAdded(); break;
        case 8: _t->requestScan(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)(int )>(_a, &ProblemCollector::aboutToAddProblem, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)()>(_a, &ProblemCollector::problemAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)(int , int )>(_a, &ProblemCollector::aboutToRemoveProblems, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)()>(_a, &ProblemCollector::problemsRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)()>(_a, &ProblemCollector::problemScansFinished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)()>(_a, &ProblemCollector::aboutToAddChecker, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProblemCollector::*)()>(_a, &ProblemCollector::checkerAdded, 7))
            return;
    }
}

const QMetaObject *GammaRay::ProblemCollector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaRay::ProblemCollector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8GammaRay16ProblemCollectorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GammaRay::ProblemCollector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void GammaRay::ProblemCollector::aboutToAddProblem(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GammaRay::ProblemCollector::problemAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GammaRay::ProblemCollector::aboutToRemoveProblems(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 4
void GammaRay::ProblemCollector::problemsRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void GammaRay::ProblemCollector::problemScansFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void GammaRay::ProblemCollector::aboutToAddChecker()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void GammaRay::ProblemCollector::checkerAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
