/********************************************************************************
** Form generated from reading UI file 'connectionstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONNECTIONSTAB_H
#define UI_CONNECTIONSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_ConnectionsTab
{
public:
    QVBoxLayout *verticalLayout_3;
    QSplitter *connectionsSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *inboundSearchLine;
    GammaRay::DeferredTreeView *inboundView;
    QWidget *layoutWidget_2;
    QVBoxLayout *verticalLayout_2;
    QLabel *label_2;
    QLineEdit *outboundSearchLine;
    GammaRay::DeferredTreeView *outboundView;

    void setupUi(QWidget *GammaRay__ConnectionsTab)
    {
        if (GammaRay__ConnectionsTab->objectName().isEmpty())
            GammaRay__ConnectionsTab->setObjectName("GammaRay__ConnectionsTab");
        GammaRay__ConnectionsTab->resize(410, 506);
        verticalLayout_3 = new QVBoxLayout(GammaRay__ConnectionsTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        connectionsSplitter = new QSplitter(GammaRay__ConnectionsTab);
        connectionsSplitter->setObjectName("connectionsSplitter");
        connectionsSplitter->setOrientation(Qt::Vertical);
        layoutWidget = new QWidget(connectionsSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(layoutWidget);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        inboundSearchLine = new QLineEdit(layoutWidget);
        inboundSearchLine->setObjectName("inboundSearchLine");

        verticalLayout->addWidget(inboundSearchLine);

        inboundView = new GammaRay::DeferredTreeView(layoutWidget);
        inboundView->setObjectName("inboundView");
        inboundView->setContextMenuPolicy(Qt::CustomContextMenu);
        inboundView->setRootIsDecorated(false);
        inboundView->setUniformRowHeights(true);

        verticalLayout->addWidget(inboundView);

        connectionsSplitter->addWidget(layoutWidget);
        layoutWidget_2 = new QWidget(connectionsSplitter);
        layoutWidget_2->setObjectName("layoutWidget_2");
        verticalLayout_2 = new QVBoxLayout(layoutWidget_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(layoutWidget_2);
        label_2->setObjectName("label_2");

        verticalLayout_2->addWidget(label_2);

        outboundSearchLine = new QLineEdit(layoutWidget_2);
        outboundSearchLine->setObjectName("outboundSearchLine");

        verticalLayout_2->addWidget(outboundSearchLine);

        outboundView = new GammaRay::DeferredTreeView(layoutWidget_2);
        outboundView->setObjectName("outboundView");
        outboundView->setContextMenuPolicy(Qt::CustomContextMenu);
        outboundView->setRootIsDecorated(false);
        outboundView->setUniformRowHeights(true);

        verticalLayout_2->addWidget(outboundView);

        connectionsSplitter->addWidget(layoutWidget_2);

        verticalLayout_3->addWidget(connectionsSplitter);

#if QT_CONFIG(shortcut)
        label->setBuddy(inboundView);
        label_2->setBuddy(outboundView);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GammaRay__ConnectionsTab);

        QMetaObject::connectSlotsByName(GammaRay__ConnectionsTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__ConnectionsTab)
    {
        label->setText(QCoreApplication::translate("GammaRay::ConnectionsTab", "&Inbound Connections:", nullptr));
        label_2->setText(QCoreApplication::translate("GammaRay::ConnectionsTab", "&Outbound Connections:", nullptr));
        (void)GammaRay__ConnectionsTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class ConnectionsTab: public Ui_ConnectionsTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_CONNECTIONSTAB_H
