/********************************************************************************
** Form generated from reading UI file 'enumstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ENUMSTAB_H
#define UI_ENUMSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/deferredtreeview.h>

namespace GammaRay {

class Ui_EnumsTab
{
public:
    QVBoxLayout *verticalLayout_8;
    QLineEdit *enumSearchLine;
    GammaRay::DeferredTreeView *enumView;

    void setupUi(QWidget *GammaRay__EnumsTab)
    {
        if (GammaRay__EnumsTab->objectName().isEmpty())
            GammaRay__EnumsTab->setObjectName("GammaRay__EnumsTab");
        GammaRay__EnumsTab->resize(400, 300);
        verticalLayout_8 = new QVBoxLayout(GammaRay__EnumsTab);
        verticalLayout_8->setObjectName("verticalLayout_8");
        enumSearchLine = new QLineEdit(GammaRay__EnumsTab);
        enumSearchLine->setObjectName("enumSearchLine");

        verticalLayout_8->addWidget(enumSearchLine);

        enumView = new GammaRay::DeferredTreeView(GammaRay__EnumsTab);
        enumView->setObjectName("enumView");

        verticalLayout_8->addWidget(enumView);


        retranslateUi(GammaRay__EnumsTab);

        QMetaObject::connectSlotsByName(GammaRay__EnumsTab);
    } // setupUi

    void retranslateUi(QWidget *GammaRay__EnumsTab)
    {
        (void)GammaRay__EnumsTab;
    } // retranslateUi

};

} // namespace GammaRay

namespace GammaRay {
namespace Ui {
    class EnumsTab: public Ui_EnumsTab {};
} // namespace Ui
} // namespace GammaRay

#endif // UI_ENUMSTAB_H
